@@ -10,20 +10,12 @@ typedef struct W_Frame {
 
     
     struct {
-	WMReliefType relief:3;
-	WMTitlePosition titlePosition:3;
+	WMReliefType relief:4;
+	WMTitlePosition titlePosition:4;
     } flags;
 } Frame;
 
 
-
-struct W_ViewProcedureTable _FrameViewProcedures = {
-	NULL,
-	NULL,
-	NULL
-};
-
-
 #define DEFAULT_RELIEF 	WRGroove
 #define DEFAULT_TITLE_POSITION	WTPAtTop
 #define DEFAULT_WIDTH		40
@@ -32,6 +24,7 @@ struct W_ViewProcedureTable _FrameViewProcedures = {
 
 static void destroyFrame(Frame *fPtr);
 static void paintFrame(Frame *fPtr);
+static void repaintFrame(Frame *fPtr);
 
 
 
@@ -39,9 +32,9 @@ void
 WMSetFrameTitlePosition(WMFrame *fPtr, WMTitlePosition position)
 {
     fPtr->flags.titlePosition = position;
-    
-   if (fPtr->view->flags.realized) {
-	paintFrame(fPtr);
+
+    if (fPtr->view->flags.realized) {
+	repaintFrame(fPtr);
     }
 }
 
@@ -52,7 +45,7 @@ WMSetFrameRelief(WMFrame *fPtr, WMReliefType relief)
     fPtr->flags.relief = relief;
     
    if (fPtr->view->flags.realized) {
-	paintFrame(fPtr);
+	repaintFrame(fPtr);
     }
 }
 
@@ -61,18 +54,29 @@ void
 WMSetFrameTitle(WMFrame *fPtr, char *title)
 {
     if (fPtr->caption)
-	free(fPtr->caption);
+	wfree(fPtr->caption);
     if (title)
 	fPtr->caption = wstrdup(title);
     else
 	fPtr->caption = NULL;
 
    if (fPtr->view->flags.realized) {
-	paintFrame(fPtr);
+	repaintFrame(fPtr);
     }
 }
 
 
+static void
+repaintFrame(Frame *fPtr)
+{
+    W_View *view = fPtr->view;
+    W_Screen *scrPtr = view->screen;
+
+    XClearWindow(scrPtr->display, view->window);
+    paintFrame(fPtr);
+}
+
+
 static void
 paintFrame(Frame *fPtr)
 {
@@ -80,9 +84,10 @@ paintFrame(Frame *fPtr)
     W_Screen *scrPtr = view->screen;
     int tx, ty, tw, th;
     int fy, fh;
-
+    Bool drawTitle;
+    
     if (fPtr->caption!=NULL)
-	th = scrPtr->normalFont->height;
+	th = WMFontHeight(scrPtr->normalFont);
     else {
 	th = 0;
     }
@@ -132,27 +137,63 @@ paintFrame(Frame *fPtr)
 	fy = 0;
 	fh = view->size.height;
     }
-/*
-    XClearArea(scrPtr->display, view->window, fy+2, 2, fh-4, view->size.width-4,
-	       False);
- */
-    XClearWindow(scrPtr->display, view->window);
-    
-    W_DrawRelief(scrPtr, view->window, 0, fy, view->size.width, fh, 
-		 fPtr->flags.relief);
 
     if (fPtr->caption!=NULL && fPtr->flags.titlePosition!=WTPNoTitle) {
-	
-	
 	tw = WMWidthOfString(scrPtr->normalFont, fPtr->caption, 
-			 strlen(fPtr->caption));
-    	
-	tx = (view->size.width - tw) / 2;
-    
-	XFillRectangle(scrPtr->display, view->window, W_GC(scrPtr->gray),
-		       tx, ty, tw, th);
+			     strlen(fPtr->caption));
 
-	WMDrawString(scrPtr, view->window, W_GC(scrPtr->black), 
+	tx = (view->size.width - tw) / 2;
+	
+	drawTitle = True;
+    } else {
+	drawTitle = False;
+    }
+
+    {
+	XRectangle rect;
+	Region region, tmp;
+	GC gc[4];
+	int i;
+
+	region = XCreateRegion();
+
+	if (drawTitle) {
+	    tmp = XCreateRegion();
+	    rect.x = tx;
+	    rect.y = ty;
+	    rect.width = tw;
+	    rect.height = th;
+	    XUnionRectWithRegion(&rect, tmp, tmp);
+	}
+	rect.x = 0;
+	rect.y = 0;
+	rect.width = view->size.width;
+	rect.height = view->size.height;
+	XUnionRectWithRegion(&rect, region, region);
+	if (drawTitle) {
+	    XSubtractRegion(region, tmp, region);
+	    XDestroyRegion(tmp);
+	}
+	gc[0] = WMColorGC(scrPtr->black);
+	gc[1] = WMColorGC(scrPtr->darkGray);
+	gc[2] = WMColorGC(scrPtr->gray);
+	gc[3] = WMColorGC(scrPtr->white);
+
+	for (i = 0; i < 4; i++) {
+	    XSetRegion(scrPtr->display, gc[i], region);
+	}
+	XDestroyRegion(region);
+
+	W_DrawReliefWithGC(scrPtr, view->window, 0, fy, view->size.width, fh, 
+			   fPtr->flags.relief, gc[0], gc[1], gc[2], gc[3]);
+
+	for (i = 0; i < 4; i++) {
+	    XSetClipMask(scrPtr->display, gc[i], None);
+	}
+    }
+
+    if (drawTitle) {
+	WMDrawString(scrPtr, view->window, WMColorGC(scrPtr->black),
 		     scrPtr->normalFont, tx, ty, fPtr->caption,
 		     strlen(fPtr->caption));
     }
@@ -171,9 +212,8 @@ handleEvents(XEvent *event, void *data)
 
     switch (event->type) {
      case Expose:
-	if (event->xexpose.count!=0)
-	    break;
-	paintFrame(fPtr);
+	if (event->xexpose.count == 0)
+	    paintFrame(fPtr);
 	break;
 	
      case DestroyNotify:
@@ -195,7 +235,7 @@ WMCreateFrame(WMWidget *parent)
 
     fPtr->view = W_CreateView(W_VIEW(parent));
     if (!fPtr->view) {
-	free(fPtr);
+	wfree(fPtr);
 	return NULL;
     }
     fPtr->view->self = fPtr;
@@ -217,7 +257,7 @@ static void
 destroyFrame(Frame *fPtr)
 {    
     if (fPtr->caption)
-	free(fPtr->caption);
+	wfree(fPtr->caption);
 
-    free(fPtr);
+    wfree(fPtr);
 }
