@@ -1,9 +1,11 @@
 
 
 #include "WINGsP.h"
+#include "wconfig.h"
 
 #include <X11/Xutil.h>
-
+#include <X11/Xatom.h>
+#include <X11/keysym.h>
 #include <X11/cursorfont.h>
 
 /********** data ************/
@@ -281,69 +283,49 @@ static char *PULLDOWN_INDICATOR[] = {
 #define PULLDOWN_INDICATOR_HEIGHT 	7
 
 
+#define CHECK_MARK_WIDTH 	8
+#define CHECK_MARK_HEIGHT 	10
+
+static char *CHECK_MARK[] = {
+"======== ",
+"======= #",
+"====== #%",
+"===== #%=",
+" #== #%==",
+" #% #%===",
+" % #%====",
+"  #%=====",
+" #%======",
+"#%======="};
+
+
 #define STIPPLE_WIDTH 8
 #define STIPPLE_HEIGHT 8
 static unsigned char STIPPLE_BITS[] = {
-   0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa};
+    0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa
+};
 
 
 
-extern W_ViewProcedureTable _WindowViewProcedures;
-extern W_ViewProcedureTable _FrameViewProcedures;
-extern W_ViewProcedureTable _LabelViewProcedures;
-extern W_ViewProcedureTable _ButtonViewProcedures;
-extern W_ViewProcedureTable _TextFieldViewProcedures;
-extern W_ViewProcedureTable _ScrollerViewProcedures;
-extern W_ViewProcedureTable _ScrollViewProcedures;
-extern W_ViewProcedureTable _ListViewProcedures;
-extern W_ViewProcedureTable _BrowserViewProcedures;
-extern W_ViewProcedureTable _PopUpButtonViewProcedures;
-extern W_ViewProcedureTable _ColorWellViewProcedures;
-extern W_ViewProcedureTable _ScrollViewViewProcedures;
-extern W_ViewProcedureTable _SliderViewProcedures;
-extern W_ViewProcedureTable _SplitViewViewProcedures;
 
-/*
- * All widget classes defined must have an entry here.
- */
-static W_ViewProcedureTable *procedureTables[16];
+extern void W_ReadConfigurations(void);
+
 
-static W_ViewProcedureTable **userProcedureTable = NULL;
 static int userWidgetCount=0;
 
 
 /*****  end data  ******/
 
 
-
-static void
-initProcedureTable()
-{
-    procedureTables[WC_Window] = &_WindowViewProcedures;
-    procedureTables[WC_Frame] = &_FrameViewProcedures;
-    procedureTables[WC_Label] = &_LabelViewProcedures;
-    procedureTables[WC_Button] = &_ButtonViewProcedures;
-    procedureTables[WC_TextField] = &_TextFieldViewProcedures;
-    procedureTables[WC_Scroller] = &_ScrollerViewProcedures;
-    procedureTables[WC_List] = &_ListViewProcedures;
-    procedureTables[WC_Browser] = &_BrowserViewProcedures;
-    procedureTables[WC_PopUpButton] = &_PopUpButtonViewProcedures;
-    procedureTables[WC_ColorWell] = &_ColorWellViewProcedures;
-    procedureTables[WC_ScrollView] = &_ScrollViewViewProcedures;
-    procedureTables[WC_Slider] = &_SliderViewProcedures;
-    procedureTables[WC_SplitView] = &_SplitViewViewProcedures;
-}
-
-
 static void
 renderPixmap(W_Screen *screen, Pixmap d, Pixmap mask, char **data, 
 	     int width, int height)
 {
     int x, y;
-    GC whiteGC = W_GC(screen->white);
-    GC blackGC = W_GC(screen->black);
-    GC lightGC = W_GC(screen->gray);
-    GC darkGC = W_GC(screen->darkGray);
+    GC whiteGC = WMColorGC(screen->white);
+    GC blackGC = WMColorGC(screen->black);
+    GC lightGC = WMColorGC(screen->gray);
+    GC darkGC = WMColorGC(screen->darkGray);
 
 
     if (mask)
@@ -403,45 +385,129 @@ makePixmap(W_Screen *sPtr, char **data, int width, int height, int masked)
 				      sPtr->depth);
 }
 
-#ifdef USE_TIFF
-#define WINGS_IMAGES_FILE  RESOURCE_PATH"/Images.tiff"
-#define DEFAULT_OBJECT_ICON_FILE RESOURCE_PATH"/defaultIcon.tiff"
-#else
-#define WINGS_IMAGES_FILE  RESOURCE_PATH"/Images.xpm"
-#define DEFAULT_OBJECT_ICON_FILE RESOURCE_PATH"/defaultIcon.xpm"
-#endif
+
+#define T_WINGS_IMAGES_FILE  RESOURCE_PATH"/Images.tiff"
+#define T_DEFAULT_OBJECT_ICON_FILE RESOURCE_PATH"/defaultIcon.tiff"
+
+#define X_WINGS_IMAGES_FILE  RESOURCE_PATH"/Images.xpm"
+#define X_DEFAULT_OBJECT_ICON_FILE RESOURCE_PATH"/defaultIcon.xpm"
+
 
 static Bool
 loadPixmaps(WMScreen *scr)
 {
     RImage *image, *tmp;
-    Pixmap pixmap;
     RColor gray;
-    
-    image = RLoadImage(scr->rcontext, WINGS_IMAGES_FILE, 0);
-    if (!image) {
-	wwarning("WINGs: could not load widget images file: %s", 
-		 RMessageForError(RErrorCode));
-	return False;
-    }
-    /* make it have a gray background */
+    RColor white;
+
     gray.red = 0xae;
     gray.green = 0xaa;
     gray.blue = 0xae;
-    RCombineImageWithColor(image, &gray);
-    tmp = RGetSubImage(image, 0, 0, 24, 24);
-    if (!RConvertImage(scr->rcontext, tmp, &pixmap)) {
-	scr->homeIcon = NULL;
-    } else {
-	scr->homeIcon = WMCreatePixmapFromXPixmaps(scr, pixmap, None, 24, 24,
-						   scr->depth);
+
+    white.red = 0xff;
+    white.green = 0xff;
+    white.blue = 0xff;
+
+    image = RLoadImage(scr->rcontext, T_WINGS_IMAGES_FILE, 0);
+    if (!image)
+	image = RLoadImage(scr->rcontext, X_WINGS_IMAGES_FILE, 0);
+    if (!image) {
+	wwarning(_("WINGs: could not load widget images file: %s"), 
+		 RMessageForError(RErrorCode));
+	return False;
     }
-    RDestroyImage(tmp);
-    RDestroyImage(image);
+    /* home icon */
+    /* make it have a gray background */
+    tmp = RGetSubImage(image, 0, 0, 24, 24);
+    RCombineImageWithColor(tmp, &gray);
+    scr->homeIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RReleaseImage(tmp);
+    /* make it have a white background */
+    tmp = RGetSubImage(image, 0, 0, 24, 24);
+    RCombineImageWithColor(tmp, &white);
+    scr->altHomeIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RReleaseImage(tmp);
+
+    /* trash can */
+    tmp = RGetSubImage(image, 104, 0, 24, 24);
+    RCombineImageWithColor(tmp, &white);
+    scr->trashcanIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RReleaseImage(tmp);
+    tmp = RGetSubImage(image, 104, 0, 24, 24);
+    RCombineImageWithColor(tmp, &white);
+    scr->altTrashcanIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RReleaseImage(tmp);
+    /* create dir */
+    tmp = RGetSubImage(image, 104, 24, 24, 24);
+    RCombineImageWithColor(tmp, &white);
+    scr->createDirIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RReleaseImage(tmp);
+    tmp = RGetSubImage(image, 104, 24, 24, 24);
+    RCombineImageWithColor(tmp, &white);
+    scr->altCreateDirIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RReleaseImage(tmp);
+    /* diskettes */
+    tmp = RGetSubImage(image, 24, 80, 24, 24);
+    RCombineImageWithColor(tmp, &white);
+    scr->disketteIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RReleaseImage(tmp);
+    tmp = RGetSubImage(image, 24, 80, 24, 24);
+    RCombineImageWithColor(tmp, &white);
+    scr->altDisketteIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RReleaseImage(tmp);
+    /* unmount */
+    tmp = RGetSubImage(image, 0, 80, 24, 24);
+    RCombineImageWithColor(tmp, &white);
+    scr->unmountIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RReleaseImage(tmp);
+    tmp = RGetSubImage(image, 0, 80, 24, 24);
+    RCombineImageWithColor(tmp, &white);
+    scr->altUnmountIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RReleaseImage(tmp);
+
+    /* Magnifying Glass Icon for ColorPanel */
+    tmp = RGetSubImage(image, 24, 0, 40, 32);
+    RCombineImageWithColor(tmp, &gray);
+    scr->magnifyIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RReleaseImage(tmp);
+    /* ColorWheel Icon for ColorPanel */
+    tmp = RGetSubImage(image, 0, 25, 24, 24);
+    scr->wheelIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RReleaseImage(tmp);
+    /* GrayScale Icon for ColorPanel */
+    tmp = RGetSubImage(image, 65, 0, 40, 24);
+    scr->grayIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RReleaseImage(tmp);
+    /* RGB Icon for ColorPanel */
+    tmp = RGetSubImage(image, 25, 33, 40, 24);
+    scr->rgbIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RReleaseImage(tmp);
+    /* CMYK Icon for ColorPanel */
+    tmp = RGetSubImage(image, 65, 25, 40, 24);
+    scr->cmykIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RReleaseImage(tmp);
+    /* HSB Icon for ColorPanel */
+    tmp = RGetSubImage(image, 0, 57, 40, 24);
+    scr->hsbIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RReleaseImage(tmp);
+    /* CustomColorPalette Icon for ColorPanel */
+    tmp = RGetSubImage(image, 81, 57, 40, 24);
+    scr->customPaletteIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RReleaseImage(tmp);
+    /* ColorList Icon for ColorPanel */
+    tmp = RGetSubImage(image, 41, 57, 40, 24);
+    scr->colorListIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RReleaseImage(tmp);
+    
+    RReleaseImage(image);
 
 #if 0
     scr->defaultObjectIcon =
-	WMCreatePixmapFromFile(scr, DEFAULT_OBJECT_ICON_FILE);
+	WMCreatePixmapFromFile(scr, T_DEFAULT_OBJECT_ICON_FILE);
+    if (!scr->defaultObjectIcon) { 
+	scr->defaultObjectIcon =
+	    WMCreatePixmapFromFile(scr, X_DEFAULT_OBJECT_ICON_FILE);
+    }
     if (!scr->defaultObjectIcon) {
 	wwarning("WINGs: could not load default icon file");
 	return False;
@@ -451,6 +517,20 @@ loadPixmaps(WMScreen *scr)
 }
 
 
+WMScreen*
+WMOpenScreen(const char *display)
+{
+    Display *dpy = XOpenDisplay(display);
+    
+    if (!dpy) {
+	wwarning("WINGs: could not open display %s",
+		 XDisplayName(display));
+	return NULL;
+    }
+    
+    return WMCreateSimpleApplicationScreen(dpy);
+}
+
 
 WMScreen*
 WMCreateSimpleApplicationScreen(Display *display)
@@ -482,12 +562,36 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context)
     XGCValues gcv;
     Pixmap stipple;
     static int initialized = 0;
+    static char *atomNames[] = {
+	"_GNUSTEP_WM_ATTR",
+	    "WM_DELETE_WINDOW",
+	    "WM_PROTOCOLS",
+	    "CLIPBOARD",
+	    "XdndAware",
+	    "XdndSelection",
+	    "XdndEnter",
+	    "XdndLeave",
+	    "XdndPosition",
+	    "XdndDrop",
+	    "XdndFinished",
+	    "XdndTypeList",
+	    "XdndActionCopy",
+	    "XdndActionMove",
+	    "XdndActionLink",
+	    "XdndActionAsk",
+	    "XdndActionPrivate",
+	    "XdndStatus",
+	    "_WINGS_DND_MOUSE_OFFSET",
+	    "WM_STATE"
+    };
+    Atom atoms[sizeof(atomNames)/sizeof(char*)];
+    int i;
 
     if (!initialized) {
 
 	initialized = 1;
-	
-	initProcedureTable();
+
+	W_ReadConfigurations();
 
 	assert(W_ApplicationInitialized());
     }
@@ -512,7 +616,87 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context)
 
     scrPtr->rootWin = RootWindow(display, screen);
 
+    scrPtr->fontCache = WMCreateHashTable(WMStringPointerHashCallbacks);
+    
+    
+    /* create input method stuff */
+    W_InitIMStuff(scrPtr);
 
+    /* Create missing CUT_BUFFERs */
+    {
+        Atom *rootWinProps;
+        int exists[8] = {0, 0, 0, 0, 0, 0, 0, 0};
+        int count;
+
+        rootWinProps = XListProperties(display, scrPtr->rootWin, &count);
+        for (i=0; i<count; i++) {
+            switch(rootWinProps[i]) {
+            case XA_CUT_BUFFER0:
+                exists[0] = 1; break;
+            case XA_CUT_BUFFER1:
+                exists[1] = 1; break;
+            case XA_CUT_BUFFER2:
+                exists[2] = 1; break;
+            case XA_CUT_BUFFER3:
+                exists[3] = 1; break;
+            case XA_CUT_BUFFER4:
+                exists[4] = 1; break;
+            case XA_CUT_BUFFER5:
+                exists[5] = 1; break;
+            case XA_CUT_BUFFER6:
+                exists[6] = 1; break;
+            case XA_CUT_BUFFER7:
+                exists[7] = 1; break;
+            default:
+                break;
+            }
+        }
+        if (rootWinProps) {
+            XFree(rootWinProps);
+        }
+        for (i=0; i<8; i++) {
+            if (!exists[i]) {
+                XStoreBuffer(display, "", 0, i);
+            }
+        }
+    }
+
+    scrPtr->ignoredModifierMask = 0;
+    {
+	int i;
+	XModifierKeymap *modmap;
+	KeyCode nlock, slock;
+	static int mask_table[8] = {
+	    ShiftMask,LockMask,ControlMask,Mod1Mask,
+		Mod2Mask, Mod3Mask, Mod4Mask, Mod5Mask
+	};
+	unsigned int numLockMask, scrollLockMask;
+	
+	nlock = XKeysymToKeycode(display, XK_Num_Lock);
+	slock = XKeysymToKeycode(display, XK_Scroll_Lock);
+	
+	/*
+	 * Find out the masks for the NumLock and ScrollLock modifiers,
+	 * so that we can bind the grabs for when they are enabled too.
+	 */
+	modmap = XGetModifierMapping(display);
+	
+	if (modmap!=NULL && modmap->max_keypermod>0) {
+	    for (i=0; i<8*modmap->max_keypermod; i++) {
+		if (modmap->modifiermap[i]==nlock && nlock!=0)
+		    numLockMask = mask_table[i/modmap->max_keypermod];
+		else if (modmap->modifiermap[i]==slock && slock!=0)
+		    scrollLockMask = mask_table[i/modmap->max_keypermod];
+	    }
+	}
+	
+	if (modmap)
+	    XFreeModifiermap(modmap);
+	
+	
+	scrPtr->ignoredModifierMask = numLockMask|scrollLockMask|LockMask;
+    }
+    
     /* initially allocate some colors */
     WMWhiteColor(scrPtr);
     WMBlackColor(scrPtr);
@@ -560,12 +744,25 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context)
     /* we need a 1bpp drawable for the monoGC, so borrow this one */
     scrPtr->monoGC = XCreateGC(display, stipple, 0, NULL);
 
-    XFreePixmap(display, stipple);
-    
-    
-    scrPtr->normalFont = WMSystemFontOfSize(scrPtr, 12);
+    scrPtr->stipple = stipple;
 
-    scrPtr->boldFont = WMBoldSystemFontOfSize(scrPtr, 12);
+    scrPtr->useMultiByte = WINGsConfiguration.useMultiByte;
+
+    scrPtr->normalFont = WMSystemFontOfSize(scrPtr, 
+		WINGsConfiguration.defaultFontSize);
+
+    scrPtr->boldFont = WMBoldSystemFontOfSize(scrPtr, 
+		WINGsConfiguration.defaultFontSize);
+
+    if (!scrPtr->boldFont)
+	scrPtr->boldFont = scrPtr->normalFont;
+
+    if (!scrPtr->normalFont) {
+	wwarning(_("could not load any fonts. Make sure your font installation"
+		 "and locale settings are correct."));
+
+	return NULL;
+    }
 
     scrPtr->checkButtonImageOn = makePixmap(scrPtr, CHECK_BUTTON_ON,
 					    CHECK_BUTTON_ON_WIDTH,
@@ -636,30 +833,97 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context)
     scrPtr->pullDownIndicator = makePixmap(scrPtr, PULLDOWN_INDICATOR,
 					   PULLDOWN_INDICATOR_WIDTH,
 					   PULLDOWN_INDICATOR_HEIGHT, True);
+
+    scrPtr->checkMark = makePixmap(scrPtr, CHECK_MARK,
+				   CHECK_MARK_WIDTH,
+				   CHECK_MARK_HEIGHT, True);
+
     loadPixmaps(scrPtr);
 
     scrPtr->defaultCursor = XCreateFontCursor(display, XC_left_ptr);
 
     scrPtr->textCursor = XCreateFontCursor(display, XC_xterm);
 
-    scrPtr->internalMessage = XInternAtom(display, "_WINGS_MESSAGE", False);
+    {
+	XColor bla;
+	Pixmap blank;
+	
+	blank = XCreatePixmap(display, scrPtr->stipple, 1, 1, 1);
+	XSetForeground(display, scrPtr->monoGC, 0);
+	XFillRectangle(display, blank, scrPtr->monoGC, 0, 0, 1, 1);
 
-    scrPtr->attribsAtom = XInternAtom(display, "_GNUSTEP_WM_ATTR", False);
-    
-    scrPtr->deleteWindowAtom = XInternAtom(display, "WM_DELETE_WINDOW", False);
+	scrPtr->invisibleCursor = XCreatePixmapCursor(display, blank, blank,
+						      &bla, &bla, 0, 0);
+	XFreePixmap(display, blank);
+    }
 
-    scrPtr->protocolsAtom = XInternAtom(display, "WM_PROTOCOLS", False);
-    
-    scrPtr->clipboardAtom = XInternAtom(display, "CLIPBOARD", False);
+#ifdef HAVE_XINTERNATOMS
+    XInternAtoms(display, atomNames, sizeof(atomNames)/sizeof(char*), False,
+		 atoms);
+#else
+    for (i = 0; i < sizeof(atomNames)/sizeof(char*); i++) {
+        atoms[i] = XInternAtom(display, atomNames[i], False);
+    }
+#endif
+
+    i = 0;
+    scrPtr->attribsAtom = atoms[i++];
+
+    scrPtr->deleteWindowAtom = atoms[i++];
+
+    scrPtr->protocolsAtom = atoms[i++];
     
+    scrPtr->clipboardAtom = atoms[i++];
+
+    scrPtr->xdndAwareAtom = atoms[i++];
+    scrPtr->xdndSelectionAtom = atoms[i++];
+    scrPtr->xdndEnterAtom = atoms[i++];
+    scrPtr->xdndLeaveAtom = atoms[i++];
+    scrPtr->xdndPositionAtom = atoms[i++];
+    scrPtr->xdndDropAtom = atoms[i++];
+    scrPtr->xdndFinishedAtom = atoms[i++];
+    scrPtr->xdndTypeListAtom = atoms[i++];
+    scrPtr->xdndStatusAtom = atoms[i++];
+
+    scrPtr->xdndActionCopy = atoms[i++];
+    scrPtr->xdndActionMove = atoms[i++];
+    scrPtr->xdndActionLink = atoms[i++];
+    scrPtr->xdndActionAsk = atoms[i++];
+    scrPtr->xdndActionPrivate = atoms[i++];
+
+    scrPtr->wmIconDragOffsetAtom = atoms[i++];
+
+    scrPtr->wmStateAtom = atoms[i++];
+
     scrPtr->rootView = W_CreateRootView(scrPtr);
 
+    scrPtr->balloon = W_CreateBalloon(scrPtr);
+
+
     W_InitApplication(scrPtr);
-    
+
     return scrPtr;
 }
 
 
+
+void
+WMSetWidgetDefaultFont(WMScreen *scr, WMFont *font)
+{
+    WMReleaseFont(scr->normalFont);
+    scr->normalFont = WMRetainFont(font);
+}
+
+
+void
+WMSetWidgetDefaultBoldFont(WMScreen *scr, WMFont *font)
+{
+    WMReleaseFont(scr->boldFont);
+    scr->boldFont = WMRetainFont(font);
+}
+
+
+
 void 
 WMHangData(WMWidget *widget, void *data)
 {
@@ -678,6 +942,7 @@ WMGetHangedData(WMWidget *widget)
 void
 WMDestroyWidget(WMWidget *widget)
 {
+    W_UnmapView(W_VIEW(widget));
     W_DestroyView(W_VIEW(widget));
 }
 
@@ -707,6 +972,13 @@ WMMapWidget(WMWidget *w)
 }
 
 
+void
+WMReparentWidget(WMWidget *w, WMWidget *newParent, int x, int y)
+{
+    W_ReparentView(W_VIEW(w), W_VIEW(newParent), x, y);
+}
+
+
 static void
 makeChildrenAutomap(W_View *view, int flag)
 {
@@ -753,79 +1025,55 @@ WMUnmapWidget(WMWidget *w)
 }
 
 
+Bool
+WMWidgetIsMapped(WMWidget *w)
+{
+    return W_VIEW(w)->flags.mapped;
+}
+
+
 void
 WMSetWidgetBackgroundColor(WMWidget *w, WMColor *color)
 {    
-    if (W_CLASS(w) < WC_UserWidget
-	&& procedureTables[W_CLASS(w)]->setBackgroundColor) {
-	
-	(*procedureTables[W_CLASS(w)]->setBackgroundColor)(w, color);
-	
-    } else if (W_CLASS(w) >= WC_UserWidget
-	&& userProcedureTable[W_CLASS(w)-WC_UserWidget]->setBackgroundColor) {
-	
-	(*userProcedureTable[W_CLASS(w)-WC_UserWidget]->setBackgroundColor)(w, color);
-	
-    } else {
-	W_SetViewBackgroundColor(W_VIEW(w), color);
-    }
+    W_SetViewBackgroundColor(W_VIEW(w), color);
+    if (W_VIEW(w)->flags.mapped)
+	WMRedisplayWidget(w);
+}
+
+
+void
+WMRaiseWidget(WMWidget *w)
+{
+     W_RaiseView(W_VIEW(w));
+}
+
+
+void
+WMLowerWidget(WMWidget *w)
+{
+     W_LowerView(W_VIEW(w));
 }
 
 
 void
 WMMoveWidget(WMWidget *w, int x, int y)
 {
-    if (W_CLASS(w) < WC_UserWidget
-	&& procedureTables[W_CLASS(w)]->move) {
-	
-	(*procedureTables[W_CLASS(w)]->move)(w, x, y);
-	
-    } else if (W_CLASS(w) >= WC_UserWidget
-	&& userProcedureTable[W_CLASS(w)-WC_UserWidget]->move) {
-
-	(*userProcedureTable[W_CLASS(w)-WC_UserWidget]->move)(w, x, y);
-
-    } else {
-	W_MoveView(W_VIEW(w), x, y);
-    }
+    W_MoveView(W_VIEW(w), x, y);
 }
 
 
 void
 WMResizeWidget(WMWidget *w, unsigned int width, unsigned int height)
 {
-    if (W_CLASS(w) < WC_UserWidget
-	&& procedureTables[W_CLASS(w)]->resize) {
-	
-	(*procedureTables[W_CLASS(w)]->resize)(w, width, height);
-	
-    } else if (W_CLASS(w) >= WC_UserWidget
-	       && userProcedureTable[W_CLASS(w)-WC_UserWidget]->resize) {
-
-	(*userProcedureTable[W_CLASS(w)-WC_UserWidget]->resize)(w, width, height);
-
-    } else {
-	W_ResizeView(W_VIEW(w), width, height);
-    }
+    W_ResizeView(W_VIEW(w), width, height);
 }
 
 
 
 W_Class
-W_RegisterUserWidget(W_ViewProcedureTable *procTable)
+W_RegisterUserWidget(void)
 {
-    W_ViewProcedureTable **newTable;
-    
     userWidgetCount++;
-    newTable = wmalloc(sizeof(W_ViewProcedureTable*)*userWidgetCount);
-    memcpy(newTable, userProcedureTable, 
-	   sizeof(W_ViewProcedureTable*)*(userWidgetCount-1));
-
-    newTable[userWidgetCount-1] = procTable;
-    
-    free(userProcedureTable);
-    
-    userProcedureTable = newTable;
 
     return userWidgetCount + WC_UserWidget - 1;
 }
@@ -881,6 +1129,35 @@ WMScreenMainLoop(WMScreen *scr)
 }
 
 
+void
+WMBreakModalLoop(WMScreen *scr)
+{
+    scr->modalLoop = 0;
+}
+
+
+void
+WMRunModalLoop(WMScreen *scr, WMView *view)
+{
+    /* why is scr passed if is determined from the view? */
+    /*WMScreen *scr = view->screen;*/
+    int oldModalLoop = scr->modalLoop;
+    WMView *oldModalView = scr->modalView;
+
+    scr->modalView = view;
+
+    scr->modalLoop = 1;
+    while (scr->modalLoop) {
+	XEvent event;
+	
+	WMNextEvent(scr->display, &event);
+	WMHandleEvent(&event);
+    }
+
+    scr->modalView = oldModalView;
+    scr->modalLoop = oldModalLoop;
+}
+
 
 Display*
 WMScreenDisplay(WMScreen *scr)
@@ -896,6 +1173,20 @@ WMScreenDepth(WMScreen *scr)
 }
 
 
+unsigned int 
+WMScreenWidth(WMScreen *scr)
+{
+    return scr->rootView->size.width;
+}
+
+
+unsigned int
+WMScreenHeight(WMScreen *scr)
+{
+    return scr->rootView->size.height;
+}
+
+
 void 
 WMRedisplayWidget(WMWidget *w)
 {
