@@ -8,6 +8,17 @@
 void
 W_DrawRelief(W_Screen *scr, Drawable d, int x, int y, unsigned int width,
 	     unsigned int height, WMReliefType relief)
+{
+    W_DrawReliefWithGC(scr, d, x, y, width, height, relief,
+		       WMColorGC(scr->black), WMColorGC(scr->darkGray),
+		       WMColorGC(scr->gray), WMColorGC(scr->white));
+}
+
+
+void
+W_DrawReliefWithGC(W_Screen *scr, Drawable d, int x, int y, unsigned int width,
+		   unsigned int height, WMReliefType relief,
+		   GC black, GC dark, GC light, GC white)
 {
     Display *dpy = scr->display;
     GC bgc;
@@ -17,37 +28,36 @@ W_DrawRelief(W_Screen *scr, Drawable d, int x, int y, unsigned int width,
 
     switch (relief) {
      case WRSimple:
-	XDrawRectangle(dpy, d, W_GC(scr->black), x, y, width-1, height-1);
+	XDrawRectangle(dpy, d, black, x, y, width-1, height-1);
 	return;
-	break;
 	
      case WRRaised:
-	bgc = W_GC(scr->black);
-	dgc = W_GC(scr->darkGray);
-	wgc = W_GC(scr->white);
-	lgc = W_GC(scr->gray);
+	bgc = black;
+	dgc = dark;
+	wgc = white;
+	lgc = light;
 	break;
 	
      case WRSunken:
-	wgc = W_GC(scr->darkGray);
-	lgc = W_GC(scr->black);
-	bgc = W_GC(scr->white);
-	dgc = W_GC(scr->gray);
+	wgc = dark;
+	lgc = black;
+	bgc = white;
+	dgc = light;
 	break;
 	
      case WRPushed:
-	lgc = wgc = W_GC(scr->black);
-	dgc = bgc = W_GC(scr->white);
+	lgc = wgc = black;
+	dgc = bgc = white;
 	break;
 	
      case WRRidge:
-	lgc = bgc = W_GC(scr->darkGray);
-	dgc = wgc = W_GC(scr->white);
+	lgc = bgc = dark;
+	dgc = wgc = white;
 	break;
 
      case WRGroove:
-	wgc = dgc = W_GC(scr->darkGray);
-	lgc = bgc = W_GC(scr->white);
+	wgc = dgc = dark;
+	lgc = bgc = white;
 	break;
 	
      default:
@@ -72,26 +82,35 @@ W_DrawRelief(W_Screen *scr, Drawable d, int x, int y, unsigned int width,
 
     XDrawLine(dpy, d, bgc, x+width-1, y, x+width-1, y+height-1);
     if (height > 2 && relief!=WRPushed) {
-	XDrawLine(dpy, d, dgc, x+width-2, y+2, x+width-2, y+height-3);
+	XDrawLine(dpy, d, dgc, x+width-2, y+1, x+width-2, y+height-2);
     }
 }
 
 
+
+
 static int
 fitText(char *text, WMFont *font, int width, int wrap)
 {
     int i, j;
     int w;
-
+    
     if (text[0]==0)
 	return 0;
+    
     i = 0;
     if (wrap) {
+	if (text[0]=='\n')
+	    return 1;
+	
 	do {
 	    i++;
 	    w = WMWidthOfString(font, text, i);
 	} while (w < width && text[i]!='\n' && text[i]!=0);
 
+	if (text[i]=='\n')
+	    return i;
+	
 	/* keep words complete */
 	if (!isspace(text[i])) {
 	    j = i;
@@ -104,7 +123,6 @@ fitText(char *text, WMFont *font, int width, int wrap)
 	while (text[i]!='\n' && text[i]!=0)
 	    i++;
     }
-
     return i;
 }
 
@@ -116,12 +134,13 @@ W_GetTextHeight(WMFont *font, char *text, int width, int wrap)
     int count;
     int length = strlen(text);
     int h;
+    int fheight = WMFontHeight(font);
 
     h = 0;
     while (length > 0) {
 	count = fitText(ptr, font, width, wrap);
 
-	h += font->height;
+	h += fheight;
 
 	if (isspace(ptr[count]))
 	    count++;
@@ -142,6 +161,7 @@ W_PaintText(W_View *view, Drawable d, WMFont *font,  int x, int y,
     int line_width;
     int line_x;
     int count;
+    int fheight = WMFontHeight(font);
 
     while (length > 0) {
 	count = fitText(ptr, font, width, wrap);
@@ -156,7 +176,7 @@ W_PaintText(W_View *view, Drawable d, WMFont *font,  int x, int y,
 
 	WMDrawString(view->screen, d, gc, font, line_x, y, ptr, count);
 
-	y += font->height;
+	y += fheight;
 
 	if (isspace(ptr[count]))
 	    count++;
@@ -170,7 +190,7 @@ W_PaintText(W_View *view, Drawable d, WMFont *font,  int x, int y,
 void
 W_PaintTextAndImage(W_View *view, int wrap, GC textGC, W_Font *font,
 		    WMReliefType relief, char *text,
-		    WMAlignment alignment,  W_Pixmap *image, 
+		    WMAlignment alignment,  W_Pixmap *image,
 		    WMImagePosition position, GC backGC, int ofs)
 {
     W_Screen *screen = view->screen;
@@ -211,10 +231,10 @@ W_PaintTextAndImage(W_View *view, int wrap, GC textGC, W_Font *font,
 	w = view->size.width;
 	h = view->size.height;
     } else {
-	x = 2;
-	y = 2;
-	w = view->size.width - 4;
-	h = view->size.height - 4;	
+	x = 1;
+	y = 1;
+	w = view->size.width - 3;
+	h = view->size.height - 3;	
     }
 
     /* calc. image alignment */
@@ -262,7 +282,7 @@ W_PaintTextAndImage(W_View *view, int wrap, GC textGC, W_Font *font,
 	
 	ix += ofs;
 	iy += ofs;
-    
+
 	XSetClipOrigin(screen->display, screen->clipGC, ix, iy);
 	XSetClipMask(screen->display, screen->clipGC, image->mask);
 
@@ -295,3 +315,32 @@ W_PaintTextAndImage(W_View *view, int wrap, GC textGC, W_Font *font,
 }
 
 
+
+WMPoint 
+wmkpoint(int x, int y)
+{
+    WMPoint point;
+    
+    point.x = x;
+    point.y = y;
+    
+    return point;
+}
+
+
+WMSize 
+wmksize(unsigned int width, unsigned int height)
+{
+    WMSize size;
+    
+    size.width = width;
+    size.height = height;
+    
+    return size;
+}
+
+
+
+
+
+
