@@ -25,6 +25,11 @@ typedef struct W_Window {
     WMSize baseSize;
     WMSize minSize;
     WMSize maxSize;
+    WMPoint minAspect;
+    WMPoint maxAspect;
+
+    WMPoint upos;
+    WMPoint ppos;
 
     WMAction *closeAction;
     void *closeData;
@@ -35,7 +40,10 @@ typedef struct W_Window {
 	unsigned style:4;
 	unsigned configured:1;
 	unsigned documentEdited:1;
-	unsigned moved:1;
+
+        unsigned setUPos:1;
+        unsigned setPPos:1;
+	unsigned setAspect:1;
     } flags;
 } _Window;
 
@@ -70,13 +78,14 @@ typedef struct {
 
 
 
-static void resizeWindow(WMWidget *, unsigned, unsigned);
-static void moveWindow(WMWidget *, int, int);
+static void willResizeWindow(W_ViewDelegate *, WMView *, unsigned*, unsigned*);
 
-struct W_ViewProcedureTable _WindowViewProcedures = {
+struct W_ViewDelegate _WindowViewDelegate = {
     NULL,
-	resizeWindow,
-	moveWindow
+	NULL,
+	NULL,
+	NULL,
+	willResizeWindow
 };
 
 
@@ -157,11 +166,13 @@ WMCreateWindowWithStyle(WMScreen *screen, char *name, int style)
 
     win->view = W_CreateTopView(screen);
     if (!win->view) {
-	free(win);
+	wfree(win);
 	return NULL;
     }
     win->view->self = win;
 
+    win->view->delegate = &_WindowViewDelegate;
+
     win->wname = wstrdup(name);
 
     /* add to the window list of the screen (application) */
@@ -169,8 +180,8 @@ WMCreateWindowWithStyle(WMScreen *screen, char *name, int style)
     screen->windowList = win;
 
     WMCreateEventHandler(win->view, ExposureMask|StructureNotifyMask
-			 |ClientMessageMask|FocusChangeMask, handleEvents, 
-			 win);
+			 |ClientMessageMask|FocusChangeMask, 
+			 handleEvents, win);
 
     W_ResizeView(win->view, DEFAULT_WIDTH, DEFAULT_HEIGHT);
 
@@ -191,15 +202,28 @@ WMCreateWindowWithStyle(WMScreen *screen, char *name, int style)
 void
 WMSetWindowTitle(WMWindow *win, char *title)
 {
+    XTextProperty property;
+    int result;
+
     if (win->title!=NULL)
-	free(win->title);
+	wfree(win->title);
     if (title!=NULL)
 	win->title = wstrdup(title);
     else
 	win->title = NULL;
 
     if (win->view->flags.realized) {
-	XStoreName(win->view->screen->display, win->view->window, title);
+	result = XmbTextListToTextProperty (win->view->screen->display,
+					    &title, 1, XStdICCTextStyle,
+					    &property);
+	if (result == XNoMemory || result == XLocaleNotSupported) {
+	    wwarning("window title conversion error... using STRING encoding");
+	    XStoreName(win->view->screen->display, win->view->window, title);
+	} else {
+	    XSetWMName(win->view->screen->display, win->view->window, &property);
+	    if (property.value)
+	        XFree(property.value);
+	}
     }
 }
 
@@ -227,7 +251,7 @@ WMSetWindowCloseAction(WMWindow *win, WMAction *action, void *clientData)
 	    XSetWMProtocols(scr->display, win->view->window, newAtoms, count);
 	    if (atoms)
 		XFree(atoms);
-	    free(newAtoms);
+	    wfree(newAtoms);
 	} else if (!action && win->closeAction) {
 	    int i, ncount;
 	    
@@ -245,7 +269,7 @@ WMSetWindowCloseAction(WMWindow *win, WMAction *action, void *clientData)
 				ncount);
 		if (atoms)
 		    XFree(atoms);
-		free(newAtoms);
+		wfree(newAtoms);
 	    }
 	}
     }
@@ -254,35 +278,26 @@ WMSetWindowCloseAction(WMWindow *win, WMAction *action, void *clientData)
 }
 
 
-static void
-moveWindow(WMWidget *w, int x, int y)
-{
-    ((WMWindow*)w)->flags.moved = 1;
-
-    W_MoveView(W_VIEW(w), x, y);
-}
-
 
 static void
-resizeWindow(WMWidget *w, unsigned width, unsigned height)
+willResizeWindow(W_ViewDelegate *self, WMView *view,
+	     unsigned *width, unsigned *height)
 {
-    WMWindow *win = (WMWindow*)w;
+    WMWindow *win = (WMWindow*)view->self;
 
     if (win->minSize.width > 0 && win->minSize.height > 0) {
-	if (width < win->minSize.width)
-	    width = win->minSize.width;
-	if (height < win->minSize.height)
-	    height = win->minSize.height;
+	if (*width < win->minSize.width)
+	    *width = win->minSize.width;
+	if (*height < win->minSize.height)
+	    *height = win->minSize.height;
     }
     
     if (win->maxSize.width > 0 && win->maxSize.height > 0) {
-	if (width > win->maxSize.width)
-	    width = win->maxSize.width;
-	if (height > win->maxSize.height)
-	    height = win->maxSize.height;
+	if (*width > win->maxSize.width)
+	    *width = win->maxSize.width;
+	if (*height > win->maxSize.height)
+	    *height = win->maxSize.height;
     }
-    
-    W_ResizeView(win->view, width, height);
 }
 
 
@@ -298,6 +313,17 @@ setSizeHints(WMWindow *win)
     }
 
     hints->flags = 0;
+
+    if (win->flags.setPPos) {
+	hints->flags |= PPosition;
+	hints->x = win->ppos.x;
+	hints->y = win->ppos.y;
+    }
+    if (win->flags.setUPos) {
+	hints->flags |= USPosition;
+	hints->x = win->upos.x;
+	hints->y = win->upos.y;
+    }
     if (win->minSize.width>0 && win->minSize.height>0) {
 	hints->flags |= PMinSize;
 	hints->min_width = win->minSize.width;
@@ -318,7 +344,15 @@ setSizeHints(WMWindow *win)
 	hints->width_inc = win->resizeIncrement.width;
 	hints->height_inc = win->resizeIncrement.height;
     }
-    
+    if (win->flags.setAspect) {
+	hints->flags |= PAspect;
+	hints->min_aspect.x = win->minAspect.x;
+	hints->min_aspect.y = win->minAspect.y;
+	hints->max_aspect.x = win->maxAspect.x;
+	hints->max_aspect.y = win->maxAspect.y;
+    }
+
+
     if (hints->flags) {
 	XSetWMNormalHints(win->view->screen->display, win->view->window, hints);
     }
@@ -331,7 +365,7 @@ static void
 writeGNUstepWMAttr(WMScreen *scr, Window window, GNUstepWMAttributes *attr)
 {
     unsigned long data[9];
-        
+
     /* handle idiot compilers where array of CARD32 != struct of CARD32 */
     data[0] = attr->flags;
     data[1] = attr->window_style;
@@ -376,12 +410,6 @@ realizeWindow(WMWindow *win)
     Atom atoms[4];
     int count;
 
-    if (!win->flags.moved && win->owner!=NULL) {
-	W_MoveView(win->view,
-		   (scr->rootView->size.width-win->view->size.width)/2,
-		   (scr->rootView->size.height-win->view->size.height)/2);
-    }
-
     classHint = XAllocClassHint();
     classHint->res_name = win->wname;
     classHint->res_class = WMGetApplicationName();
@@ -414,11 +442,9 @@ realizeWindow(WMWindow *win)
     if (count>0)
 	XSetWMProtocols(scr->display, win->view->window, atoms, count);
 
-    if (win->title)
-	XStoreName(scr->display, win->view->window, win->title);
-
-    if (win->miniTitle)
-	XSetIconName(scr->display, win->view->window, win->miniTitle);
+    if (win->title || win->miniTitle)
+	XmbSetWMProperties(scr->display, win->view->window, win->title, 
+			   win->miniTitle, NULL, 0, NULL, NULL, NULL);
 	
     setWindowMakerHints(win);
 
@@ -431,15 +457,49 @@ realizeWindow(WMWindow *win)
 }
 
 
+
 void
-WMHideWindow(WMWindow *win)
+WMSetWindowAspectRatio(WMWindow *win, int minX, int minY,
+		       int maxX, int maxY)
 {
-    WMUnmapWidget(win);
-    XWithdrawWindow(win->view->screen->display, win->view->window,
-		    win->view->screen->screen);
+    win->flags.setAspect = 1;
+    win->minAspect.x = minX;
+    win->minAspect.y = minY;
+    win->maxAspect.x = maxX;
+    win->maxAspect.y = maxY;
+    if (win->view->flags.realized)
+        setSizeHints(win);
 }
 
 
+
+void
+WMSetWindowInitialPosition(WMWindow *win, int x, int y)
+{
+    win->flags.setPPos = 1;
+    win->ppos.x = x;
+    win->ppos.y = y;
+    if (win->view->flags.realized)
+	setSizeHints(win);
+    WMMoveWidget(win, x, y);
+}
+
+
+
+void
+WMSetWindowUserPosition(WMWindow *win, int x, int y)
+{
+    win->flags.setUPos = 1;
+    win->upos.x = x;
+    win->upos.y = y;
+    if (win->view->flags.realized)
+	setSizeHints(win);
+    WMMoveWidget(win, x, y);
+}
+
+
+
+
 void
 WMSetWindowMinSize(WMWindow *win, unsigned width, unsigned height)
 {
@@ -503,7 +563,7 @@ WMSetWindowDocumentEdited(WMWindow *win, Bool flag)
 
 
 void
-WMSetWindowMiniwindowImage(WMWindow *win, WMPixmap *pixmap)
+WMSetWindowMiniwindowPixmap(WMWindow *win, WMPixmap *pixmap)
 {
     if ((win->miniImage && !pixmap) || (!win->miniImage && pixmap)) {
 	if (win->miniImage)
@@ -544,10 +604,13 @@ WMSetWindowMiniwindowImage(WMWindow *win, WMPixmap *pixmap)
 void
 WMSetWindowMiniwindowTitle(WMWindow *win, char *title)
 {
+    XTextProperty property;
+    int result;
+
     if ((win->miniTitle && !title) || (!win->miniTitle && title)
 	|| (title && win->miniTitle && strcoll(title, win->miniTitle)!=0)) {
 	if (win->miniTitle)
-	    free(win->miniTitle);
+	    wfree(win->miniTitle);
 	
 	if (title)
 	    win->miniTitle = wstrdup(title);
@@ -555,7 +618,19 @@ WMSetWindowMiniwindowTitle(WMWindow *win, char *title)
 	    win->miniTitle = NULL;
 
 	if (win->view->flags.realized) {
-	    XSetIconName(win->view->screen->display, win->view->window, title);
+	    result = XmbTextListToTextProperty (win->view->screen->display,
+						&title, 1, XStdICCTextStyle,
+						&property);
+	    if (result == XNoMemory || result == XLocaleNotSupported) {
+		wwarning("icon title conversion error..using STRING encoding");
+		XSetIconName(win->view->screen->display, win->view->window,
+			     title);
+	    } else {
+		XSetWMIconName(win->view->screen->display, win->view->window,
+			       &property);
+		if (property.value)
+		    XFree(property.value);
+	    }
 	}
     }
 }
@@ -564,6 +639,7 @@ WMSetWindowMiniwindowTitle(WMWindow *win, char *title)
 void
 WMCloseWindow(WMWindow *win)
 {
+    WMUnmapWidget(win);
     /* withdraw the window */
     if (win->view->flags.realized)
 	XWithdrawWindow(win->view->screen->display, win->view->window,
@@ -575,6 +651,7 @@ static void
 handleEvents(XEvent *event, void *clientData)
 {
     _Window *win = (_Window*)clientData;
+    W_View *view = win->view;
     
     
     switch (event->type) {
@@ -588,9 +665,52 @@ handleEvents(XEvent *event, void *clientData)
 	    }
 	}
 	break;
+/*
+   // was causing windows to ignore commands like closeWindow
+ * // after the windows is iconized/restored or a workspace change
+ * // if this is really needed, put the MapNotify portion too and
+ * // fix the restack bug in wmaker
+     case UnmapNotify:
+	WMUnmapWidget(win);
+	break;
+ * 
+     case MapNotify:
+	WMMapWidget(win);
+	break;
+ 
+*/
      case DestroyNotify:
 	destroyWindow(win);
 	break;
+	
+     case ConfigureNotify:
+	if (event->xconfigure.width != view->size.width
+	    || event->xconfigure.height != view->size.height) {
+
+	    view->size.width = event->xconfigure.width;
+	    view->size.height = event->xconfigure.height;
+
+	    if (view->flags.notifySizeChanged) {
+		WMPostNotificationName(WMViewSizeDidChangeNotification,
+				       view, NULL);
+	    }
+	}
+	if (event->xconfigure.x != view->pos.x
+	    || event->xconfigure.y != view->pos.y) {
+
+	    if (event->xconfigure.send_event) {
+		view->pos.x = event->xconfigure.x;
+		view->pos.y = event->xconfigure.y;
+	    } else {
+		Window foo;
+
+		XTranslateCoordinates(view->screen->display,
+				      view->window, view->screen->rootWin,
+				      event->xconfigure.x, event->xconfigure.y,
+				      &view->pos.x, &view->pos.y, &foo);
+	    }
+	}
+	break;
     }
 }
 
@@ -610,21 +730,23 @@ destroyWindow(_Window *win)
 	WMWindow *ptr;
 	ptr = scr->windowList;
 	
-	while (ptr->nextPtr) {
-	    if (ptr->nextPtr==win) {
-		ptr->nextPtr = ptr->nextPtr->nextPtr;
-		break;
+	if (ptr) {
+	    while (ptr->nextPtr) {
+		if (ptr->nextPtr==win) {
+		    ptr->nextPtr = ptr->nextPtr->nextPtr;
+		    break;
+		}
+		ptr = ptr->nextPtr;
 	    }
-	    ptr = ptr->nextPtr;
 	}
     }
 
     if (win->title) {
-	free(win->title);
+	wfree(win->title);
     }
     
     if (win->miniTitle) {
-	free(win->miniTitle);
+	wfree(win->miniTitle);
     }
     
     if (win->miniImage) {
@@ -632,9 +754,9 @@ destroyWindow(_Window *win)
     }
     
     if (win->wname)
-	free(win->wname);
+	wfree(win->wname);
 
-    free(win);
+    wfree(win);
 }
 
 
