@@ -2,7 +2,7 @@
  * 
  *  WPrefs - Window Maker Preferences Program
  * 
- *  Copyright (c) 1998 Alfredo K. Kojima
+ *  Copyright (c) 1998-2000 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,35 +25,43 @@
 #include <assert.h>
 
 
-extern Panel *InitWindowHandling(WMScreen *scr, WMWindow *win);
+extern Panel *InitWindowHandling(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitKeyboardSettings(WMScreen *scr, WMWindow *win);
+extern Panel *InitKeyboardSettings(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitMouseSettings(WMScreen *scr, WMWindow *win);
+extern Panel *InitMouseSettings(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitKeyboardShortcuts(WMScreen *scr, WMWindow *win);
+extern Panel *InitKeyboardShortcuts(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitWorkspace(WMScreen *scr, WMWindow *win);
+extern Panel *InitWorkspace(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitFocus(WMScreen *scr, WMWindow *win);
+extern Panel *InitFocus(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitPreferences(WMScreen *scr, WMWindow *win);
+extern Panel *InitPreferences(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitTextureAndColor(WMScreen *scr, WMWindow *win);
+extern Panel *InitFont(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitText(WMScreen *scr, WMWindow *win);
+extern Panel *InitConfigurations(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitConfigurations(WMScreen *scr, WMWindow *win);
+extern Panel *InitPaths(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitPaths(WMScreen *scr, WMWindow *win);
+extern Panel *InitMenu(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitMenu(WMScreen *scr, WMWindow *win);
+extern Panel *InitExpert(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitExpert(WMScreen *scr, WMWindow *win);
+extern Panel *InitMenuPreferences(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitMenuPreferences(WMScreen *scr, WMWindow *win);
+extern Panel *InitIcons(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitIcons(WMScreen *scr, WMWindow *win);
+extern Panel *InitThemes(WMScreen *scr, WMWidget *parent);
+
+extern Panel *InitAppearance(WMScreen *scr, WMWidget *parent);
+
+
+
+
+#define ICON_TITLE_FONT "-adobe-helvetica-bold-r-*-*-10-*"
+#define ICON_TITLE_VFONT "-adobe-helvetica-bold-r-*-*-10-[]-*"
 
 
 #define MAX_SECTIONS 16
@@ -72,13 +80,15 @@ typedef struct _WPrefs {
     WMButton *closeBtn;
     WMButton *undoBtn;
     WMButton *undosBtn;
-    
+
+    WMButton *balloonBtn;
+
     WMFrame *banner;
     WMLabel *nameL;
     WMLabel *versionL;
     WMLabel *creditsL;
     WMLabel *statusL;
-    
+        
     Panel *currentPanel;
 } _WPrefs;
 
@@ -86,9 +96,10 @@ typedef struct _WPrefs {
 static _WPrefs WPrefs;
 
 /* system wide defaults dictionary. Read-only */
-static proplist_t GlobalDB = NULL;
+static WMPropList *GlobalDB = NULL;
 /* user defaults dictionary */
-static proplist_t WindowMakerDB = NULL;
+static WMPropList *WindowMakerDB = NULL;
+static char *WindowMakerDBPath = NULL;
 
 
 static Bool TIFFOK = False;
@@ -103,10 +114,13 @@ static void loadConfigurations(WMScreen *scr, WMWindow *mainw);
 
 static void savePanelData(Panel *panel);
 
+static void prepareForClose();
 
 void
 quit(WMWidget *w, void *data)
 {
+    prepareForClose();
+
     exit(0);
 }
 
@@ -115,9 +129,11 @@ static void
 save(WMWidget *w, void *data)
 {
     int i;
-    proplist_t p1, p2;
-    proplist_t keyList;
-    proplist_t key;
+    WMPropList *p1, *p2;
+    WMPropList *keyList;
+    WMPropList *key;
+    char *msg = "Reconfigure";
+    XEvent ev;
 
 
 /*    puts("gathering data");*/
@@ -128,32 +144,50 @@ save(WMWidget *w, void *data)
     }
 /*    puts("compressing data");*/
     /* compare the user dictionary with the global and remove redundant data */
-    keyList = PLGetAllDictionaryKeys(GlobalDB);
-/*    puts(PLGetDescription(WindowMakerDB));*/
-    for (i=0; i<PLGetNumberOfElements(keyList); i++) {
-	key = PLGetArrayElement(keyList, i);
+    keyList = WMGetPLDictionaryKeys(GlobalDB);
+/*    puts(WMGetPropListDescription(WindowMakerDB, False));*/
+    for (i=0; i<WMGetPropListItemCount(keyList); i++) {
+	key = WMGetFromPLArray(keyList, i);
 	
 	/* We don't have this value anyway, so no problem. 
 	 * Probably  a new option */
-	p1 = PLGetDictionaryEntry(WindowMakerDB, key);
+	p1 = WMGetFromPLDictionary(WindowMakerDB, key);
 	if (!p1)
 	    continue;
 	/* The global doesn't have it, so no problem either. */
-	p2 = PLGetDictionaryEntry(GlobalDB, key);
+	p2 = WMGetFromPLDictionary(GlobalDB, key);
 	if (!p2)
 	    continue;
 	/* If both values are the same, don't save. */
-	if (PLIsEqual(p1, p2))
-	    PLRemoveDictionaryEntry(WindowMakerDB, key);
+	if (WMIsPropListEqualTo(p1, p2))
+	    WMRemoveFromPLDictionary(WindowMakerDB, key);
     }
-/*    puts(PLGetDescription(WindowMakerDB));*/
-    PLRelease(keyList);
+/*    puts(WMGetPropListDescription(WindowMakerDB, False));*/
+    WMReleasePropList(keyList);
 /*    puts("storing data");*/
 
-    PLSave(WindowMakerDB, YES);
+    WMWritePropListToFile(WindowMakerDB, WindowMakerDBPath, True);
+
+
+    memset(&ev, 0, sizeof(XEvent));
+	    
+    ev.xclient.type = ClientMessage;
+    ev.xclient.message_type = XInternAtom(WMScreenDisplay(WMWidgetScreen(w)),
+					  "_WINDOWMAKER_COMMAND", False);
+    ev.xclient.window = DefaultRootWindow(WMScreenDisplay(WMWidgetScreen(w)));
+    ev.xclient.format = 8;
+
+    for (i = 0; i <= strlen(msg); i++) {
+	ev.xclient.data.b[i] = msg[i];
+    }
+    XSendEvent(WMScreenDisplay(WMWidgetScreen(w)), 
+	       DefaultRootWindow(WMScreenDisplay(WMWidgetScreen(w))),
+	       False, SubstructureRedirectMask, &ev);
+    XFlush(WMScreenDisplay(WMWidgetScreen(w)));
 }
 
 
+
 static void
 undo(WMWidget *w, void *data)
 {
@@ -184,6 +218,36 @@ undoAll(WMWidget *w, void *data)
 }
 
 
+
+static void
+prepareForClose()
+{
+    int i;
+
+    for (i=0; i<WPrefs.sectionCount; i++) {
+	PanelRec *rec = WMGetHangedData(WPrefs.sectionB[i]);
+
+	if (rec->callbacks.prepareForClose 
+	    && (rec->callbacks.flags & INITIALIZED_PANEL))
+	    (*rec->callbacks.prepareForClose)((Panel*)rec);
+    }
+}
+
+
+void
+toggleBalloons(WMWidget *w, void *data)
+{
+    WMUserDefaults *udb = WMGetStandardUserDefaults();
+    Bool flag;
+
+    flag = WMGetButtonSelected(WPrefs.balloonBtn);
+
+    WMSetBalloonEnabled(WMWidgetScreen(WPrefs.win), flag);
+
+    WMSetUDBoolForKey(udb, flag, "BalloonHelp");
+}
+
+
 static void
 createMainWindow(WMScreen *scr)
 {
@@ -198,7 +262,7 @@ createMainWindow(WMScreen *scr)
     WMSetWindowMaxSize(WPrefs.win, 520, 390);
     WMSetWindowMinSize(WPrefs.win, 520, 390);
     WMSetWindowMiniwindowTitle(WPrefs.win, "Preferences");
-    WMSetWindowMiniwindowImage(WPrefs.win, WMGetApplicationIconImage(scr));
+    WMSetWindowMiniwindowPixmap(WPrefs.win, WMGetApplicationIconPixmap(scr));
     
     WPrefs.scrollV = WMCreateScrollView(WPrefs.win);
     WMResizeWidget(WPrefs.scrollV, 500, 87);
@@ -237,14 +301,29 @@ createMainWindow(WMScreen *scr)
     WMMoveWidget(WPrefs.closeBtn, 425, 350);
     WMSetButtonText(WPrefs.closeBtn, _("Close"));
     WMSetButtonAction(WPrefs.closeBtn, quit, NULL);
-    
+
+
+    WPrefs.balloonBtn = WMCreateSwitchButton(WPrefs.win);
+    WMResizeWidget(WPrefs.balloonBtn, 200, 28);
+    WMMoveWidget(WPrefs.balloonBtn, 15, 350);
+    WMSetButtonText(WPrefs.balloonBtn, _("Balloon Help"));
+    WMSetButtonAction(WPrefs.balloonBtn, toggleBalloons, NULL);
+    {
+	WMUserDefaults *udb = WMGetStandardUserDefaults();
+	Bool flag = WMGetUDBoolForKey(udb, "BalloonHelp");
+
+	WMSetButtonSelected(WPrefs.balloonBtn, flag);
+	WMSetBalloonEnabled(scr, flag);
+    }
+
     /* banner */
     WPrefs.banner = WMCreateFrame(WPrefs.win);
     WMResizeWidget(WPrefs.banner, FRAME_WIDTH, FRAME_HEIGHT);
     WMMoveWidget(WPrefs.banner, FRAME_LEFT, FRAME_TOP);
     WMSetFrameRelief(WPrefs.banner, WRFlat);
 
-    font = WMCreateFont(scr, "-*-times-bold-r-*-*-24-*-*-*-*-*-*-*");
+    font = WMCreateFont(scr, "-*-times-bold-r-*-*-24-*-*-*-*-*-*-*,"
+			"-*-fixed-medium-r-normal-*-24-*");
     if (!font)
 	font = WMBoldSystemFontOfSize(scr, 24);
     WPrefs.nameL = WMCreateLabel(WPrefs.banner);
@@ -252,14 +331,15 @@ createMainWindow(WMScreen *scr)
     WMResizeWidget(WPrefs.nameL, FRAME_WIDTH-20, 30);
     WMMoveWidget(WPrefs.nameL, 10, 25);
     WMSetLabelFont(WPrefs.nameL, font);
-    WMSetLabelText(WPrefs.nameL, "Window Maker Preferences Utility");
+    WMSetLabelText(WPrefs.nameL, _("Window Maker Preferences Utility"));
     WMReleaseFont(font);
 
     WPrefs.versionL = WMCreateLabel(WPrefs.banner);
     WMResizeWidget(WPrefs.versionL, FRAME_WIDTH-20, 20);
     WMMoveWidget(WPrefs.versionL, 10, 65);
     WMSetLabelTextAlignment(WPrefs.versionL, WACenter);
-    sprintf(buffer, _("Version %s for Window Maker %s"), WVERSION, WMVERSION);
+    sprintf(buffer, _("Version %s for Window Maker %s or newer"), WVERSION,
+								 WMVERSION);
     WMSetLabelText(WPrefs.versionL, buffer);
 
     WPrefs.statusL = WMCreateLabel(WPrefs.banner);
@@ -273,8 +353,8 @@ createMainWindow(WMScreen *scr)
     WMMoveWidget(WPrefs.creditsL, 10, FRAME_HEIGHT-60);
     WMSetLabelTextAlignment(WPrefs.creditsL, WACenter);
     WMSetLabelText(WPrefs.creditsL, _("Programming/Design: Alfredo K. Kojima\n"
-		   		"Artwork: Marco van Hylckama Vlieg\n"
-				      "More Programming: James Thompson"));
+		   		"Artwork: Marco van Hylckama Vlieg, Largo et al\n"
+				      "More Programming: James Thompson et al"));
    
     
     WMMapSubwidgets(WPrefs.win);
@@ -297,7 +377,10 @@ showPanel(Panel *panel)
 
     WMSetWindowTitle(WPrefs.win, rec->sectionName);
     
-    WMMapWidget(rec->frame);
+    if (rec->callbacks.showPanel)
+	(*rec->callbacks.showPanel)(panel);
+
+    WMMapWidget(rec->box);
 }
 
 
@@ -307,7 +390,10 @@ hidePanel(Panel *panel)
 {
     PanelRec *rec = (PanelRec*)panel;    
     
-    WMUnmapWidget(rec->frame);
+    WMUnmapWidget(rec->box);
+    
+    if (rec->callbacks.hidePanel)
+	(*rec->callbacks.hidePanel)(panel);
 }
 
 
@@ -325,17 +411,25 @@ savePanelData(Panel *panel)
 static void 
 changeSection(WMWidget *self, void *data)
 {
-    if (WPrefs.banner) {
-	WMDestroyWidget(WPrefs.banner);
-	WPrefs.banner = NULL;
+    if (WPrefs.currentPanel == data)
+	return;
+	
+    if (WPrefs.currentPanel == NULL) {
+	WMDestroyWidget(WPrefs.nameL);
+	WMDestroyWidget(WPrefs.creditsL);
+	WMDestroyWidget(WPrefs.versionL);
+	WMDestroyWidget(WPrefs.statusL);
+
+	WMSetFrameRelief(WPrefs.banner, WRGroove);
+
 /*	WMMapWidget(WPrefs.undosBtn);
 	WMMapWidget(WPrefs.undoBtn);
  */
 	WMMapWidget(WPrefs.saveBtn);
     }
-    
+
     showPanel(data);
-    
+
     if (WPrefs.currentPanel)
 	hidePanel(WPrefs.currentPanel);
     WPrefs.currentPanel = data;
@@ -343,8 +437,6 @@ changeSection(WMWidget *self, void *data)
 
 
 
-
-
 char*
 LocateImage(char *name)
 {
@@ -358,7 +450,7 @@ LocateImage(char *name)
 	sprintf(tmp, "%s.xpm", name);
 	path = WMPathForResourceOfType(tmp, "xpm");
     }
-    free(tmp);
+    wfree(tmp);
     if (!path) {
 	wwarning(_("could not locate image file %s\n"), name);
     }
@@ -367,51 +459,126 @@ LocateImage(char *name)
 }
 
 
+
+static WMPixmap*
+makeTitledIcon(WMScreen *scr, WMPixmap *icon, char *title1, char *title2)
+{
+    return WMRetainPixmap(icon);
+    
+#if 0
+    static GC gc = NULL;
+    static XFontStruct *hfont = NULL;
+    static XFontStruct *vfont = NULL;
+    WMPixmap *tmp;
+    Pixmap pix, mask;
+    Display *dpy = WMScreenDisplay(scr);
+    WMColor *black = WMBlackColor(scr);
+    GC fgc;
+    WMSize size = WMGetPixmapSize(icon);
+
+    
+    tmp = WMCreatePixmap(scr, 60, 60, WMScreenDepth(scr), True);
+
+    pix = WMGetPixmapXID(tmp);
+    mask = WMGetPixmapMaskXID(tmp);
+
+    if (gc == NULL) {
+	gc = XCreateGC(dpy, mask, 0, NULL);
+
+	hfont = XLoadQueryFont(dpy, ICON_TITLE_FONT);
+	vfont = XLoadQueryFont(dpy, ICON_TITLE_VFONT);
+    }
+    
+    if (hfont == NULL) {
+	return WMRetainPixmap(icon);
+    }
+    
+    XSetForeground(dpy, gc, 0);
+    XFillRectangle(dpy, mask, gc, 0, 0, 60, 60);
+    
+    fgc = WMColorGC(black);
+    
+    XSetForeground(dpy, gc, 1);
+    
+    XCopyArea(dpy, WMGetPixmapXID(icon), pix, fgc, 0, 0,
+	      size.width, size.height, 12, 12);
+    
+    if (WMGetPixmapMaskXID(icon) != None)
+	XCopyPlane(dpy, WMGetPixmapMaskXID(icon), mask, gc, 0, 0,
+		   size.width, size.height, 12, 12, 1);
+    else
+	XFillRectangle(dpy, mask, gc, 12, 12, 48, 48);
+
+
+    if (title1) {
+	XSetFont(dpy, fgc, vfont->fid);
+	XSetFont(dpy, gc, vfont->fid);
+
+	XDrawString(dpy, pix, fgc, 0, vfont->ascent, 
+		    title1, strlen(title1));
+	
+	XDrawString(dpy, mask, gc, 0, vfont->ascent, 
+		    title1, strlen(title1));
+    }
+
+    if (title2) {
+	XSetFont(dpy, fgc, hfont->fid);
+	XSetFont(dpy, gc, hfont->fid);
+
+	XDrawString(dpy, pix, fgc, (title1 ? 12 : 0), hfont->ascent, 
+		    title2, strlen(title2));
+	
+	XDrawString(dpy, mask, gc, (title1 ? 12 : 0), hfont->ascent, 
+		    title2, strlen(title2));
+    }
+
+    return tmp;
+#endif
+}
+
+
 void
-AddSection(Panel *panel, char *iconFile)
+SetButtonAlphaImage(WMScreen *scr, WMButton *bPtr, char *file,
+		    char *title1, char *title2)
 {
-    WMButton *bPtr;
     WMPixmap *icon;
+    WMPixmap *icon2;
     RColor color;
     char *iconPath;
 
-    assert(WPrefs.sectionCount < MAX_SECTIONS);
-    
-    iconPath = LocateImage(iconFile);
-
-    bPtr = WMCreateCustomButton(WPrefs.buttonF,	WBBStateLightMask
-				|WBBStateChangeMask);
-    WMResizeWidget(bPtr, 64, 64);
-    WMMoveWidget(bPtr, WPrefs.sectionCount*64, 0);
-    WMSetButtonImagePosition(bPtr, WIPImageOnly);
-    WMSetButtonAction(bPtr, changeSection, panel);
-    WMHangData(bPtr, panel);
+    iconPath = LocateImage(file);
 
     color.red = 0xae;
     color.green = 0xaa;
     color.blue = 0xae;
     color.alpha = 0;
     if (iconPath) {
-	icon = WMCreateBlendedPixmapFromFile(WMWidgetScreen(WPrefs.win),
-					     iconPath, &color);
+	icon = WMCreateBlendedPixmapFromFile(scr, iconPath, &color);
 	if (!icon)
 	    wwarning(_("could not load icon file %s"), iconPath);
     } else {
 	icon = NULL;
     }
+    
+    if (icon) {
+	icon2 = makeTitledIcon(scr, icon, title1, title2);
+	if (icon)
+	    WMReleasePixmap(icon);
+    } else {
+	icon2 = NULL;
+    }
+    
+    WMSetButtonImage(bPtr, icon2);
 
-    WMSetButtonImage(bPtr, icon);
-
-    if (icon)
-	WMReleasePixmap(icon);
+    if (icon2)
+	WMReleasePixmap(icon2);
 
     color.red = 0xff;
     color.green = 0xff;
     color.blue = 0xff;
     color.alpha = 0;
     if (iconPath) {
-	icon = WMCreateBlendedPixmapFromFile(WMWidgetScreen(WPrefs.win),
-					     iconPath, &color);
+	icon = WMCreateBlendedPixmapFromFile(scr, iconPath, &color);
 	if (!icon)
 	    wwarning(_("could not load icon file %s"), iconPath);
     } else {
@@ -423,6 +590,43 @@ AddSection(Panel *panel, char *iconFile)
     if (icon)
 	WMReleasePixmap(icon);
 
+    if (iconPath)
+	wfree(iconPath);
+}
+
+
+void
+AddSection(Panel *panel, char *iconFile)
+{
+    WMButton *bPtr;
+
+    assert(WPrefs.sectionCount < MAX_SECTIONS);
+    
+
+    bPtr = WMCreateCustomButton(WPrefs.buttonF,	WBBStateLightMask
+				|WBBStateChangeMask);
+    WMResizeWidget(bPtr, 64, 64);
+    WMMoveWidget(bPtr, WPrefs.sectionCount*64, 0);
+    WMSetButtonImagePosition(bPtr, WIPImageOnly);
+    WMSetButtonAction(bPtr, changeSection, panel);
+    WMHangData(bPtr, panel);
+
+    WMSetBalloonTextForView(((PanelRec*)panel)->description, 
+		    WMWidgetView(bPtr));
+
+    {
+	char *t1, *t2;
+	
+	t1 = wstrdup(((PanelRec*)panel)->sectionName);
+	t2 = strchr(t1, ' ');
+	if (t2) {
+	    *t2 = 0;
+	    t2++;
+	}
+	SetButtonAlphaImage(WMWidgetScreen(bPtr), bPtr, iconFile,
+			    t1, t2);
+	wfree(t1);
+    }
     WMMapWidget(bPtr);
 
     WPrefs.sectionB[WPrefs.sectionCount] = bPtr;
@@ -430,12 +634,10 @@ AddSection(Panel *panel, char *iconFile)
     if (WPrefs.sectionCount > 0) {
 	WMGroupButtons(WPrefs.sectionB[0], bPtr);
     }
-    
-    WPrefs.sectionCount++;
-    
-    WMResizeWidget(WPrefs.buttonF, WPrefs.sectionCount*64, 64);
 
-    free(iconPath);
+    WPrefs.sectionCount++;
+
+    WMResizeWidget(WPrefs.buttonF, WPrefs.sectionCount*64, 64);
 }
 
 
@@ -455,7 +657,6 @@ Initialize(WMScreen *scr)
 	    break;
 	}
     }
-    RFreeStringList(list);
     
     if (TIFFOK)
     	path = WMPathForResourceOfType("WPrefs.tiff", NULL);
@@ -466,17 +667,17 @@ Initialize(WMScreen *scr)
 	
 	tmp = RLoadImage(WMScreenRContext(scr), path, 0);
 	if (!tmp) {
-	    wwarning("could not load image file %s:%s", path, 
+	    wwarning(_("could not load image file %s:%s"), path,
 		     RMessageForError(RErrorCode));
 	} else {
 	    icon = WMCreatePixmapFromRImage(scr, tmp, 0);
-	    RDestroyImage(tmp);
+	    RReleaseImage(tmp);
 	    if (icon) {
-	        WMSetApplicationIconImage(scr, icon);
+	        WMSetApplicationIconPixmap(scr, icon);
 	        WMReleasePixmap(icon);
 	    }
 	}
-	free(path);
+	wfree(path);
     }
     
     memset(&WPrefs, 0, sizeof(_WPrefs));
@@ -491,30 +692,40 @@ Initialize(WMScreen *scr)
 
     WMSetLabelText(WPrefs.statusL, _("Initializing configuration panels..."));
 
-    InitWindowHandling(scr, WPrefs.win);
-    InitFocus(scr, WPrefs.win);
-    InitMenuPreferences(scr, WPrefs.win);
-    InitIcons(scr, WPrefs.win);
-    InitPreferences(scr, WPrefs.win);
+    InitFocus(scr, WPrefs.banner);
+    InitWindowHandling(scr, WPrefs.banner);
+
+    InitMenuPreferences(scr, WPrefs.banner);
+    InitIcons(scr, WPrefs.banner);
+    InitPreferences(scr, WPrefs.banner);
+
+    InitPaths(scr, WPrefs.banner);    
+    InitWorkspace(scr, WPrefs.banner);
+    InitConfigurations(scr, WPrefs.banner);
+
+    InitMenu(scr, WPrefs.banner);
 
-    InitPaths(scr, WPrefs.win);    
-    InitWorkspace(scr, WPrefs.win);
-    InitConfigurations(scr, WPrefs.win);
-    InitMenu(scr, WPrefs.win);
 #ifdef not_yet_fully_implemented
-    InitKeyboardSettings(scr, WPrefs.win);
+    InitKeyboardSettings(scr, WPrefs.banner);
+#endif
+    InitKeyboardShortcuts(scr, WPrefs.banner);
+    InitMouseSettings(scr, WPrefs.banner);
+
+    InitAppearance(scr, WPrefs.banner);
+    
+#ifdef akk
+    InitFont(scr, WPrefs.banner);
 #endif
-    InitKeyboardShortcuts(scr, WPrefs.win);
-    InitMouseSettings(scr, WPrefs.win);
 #ifdef not_yet_fully_implemented
-    InitTextureAndColor(scr, WPrefs.win);
-    InitText(scr, WPrefs.win);
+    InitThemes(scr, WPrefs.banner);
 #endif
-    InitExpert(scr, WPrefs.win);
+    InitExpert(scr, WPrefs.banner);
 
     WMRealizeWidget(WPrefs.scrollV);
 
-    WMSetLabelText(WPrefs.statusL, "This program is still under development. Backup your ~/GNUstep/Defaults directory, before using it.");
+    WMSetLabelText(WPrefs.statusL, 
+		   _("WPrefs is free software and is distributed WITHOUT ANY\n"
+	   "WARRANTY under the terms of the GNU General Public License."));
 }
 
 
@@ -528,7 +739,7 @@ GetWindow(Panel *panel)
 static void
 loadConfigurations(WMScreen *scr, WMWindow *mainw)
 {
-    proplist_t db, gdb;
+    WMPropList *db, *gdb;
     char *path;
     FILE *file;
     char buffer[1024];
@@ -536,11 +747,12 @@ loadConfigurations(WMScreen *scr, WMWindow *mainw)
     int v1, v2, v3;
     
     path = wdefaultspathfordomain("WindowMaker");
+    WindowMakerDBPath = path;
     
-    db = PLGetProplistWithPath(path);
+    db = WMReadPropListFromFile(path);
     if (db) {
-	if (!PLIsDictionary(db)) {
-	    PLRelease(db);
+	if (!WMIsPLDictionary(db)) {
+	    WMReleasePropList(db);
 	    db = NULL;
 	    sprintf(mbuf, _("Window Maker domain (%s) is corrupted!"), path);
 	    WMRunAlertPanel(scr, mainw, _("Error"), mbuf, _("OK"), NULL, NULL);
@@ -550,12 +762,20 @@ loadConfigurations(WMScreen *scr, WMWindow *mainw)
 		path);
 	WMRunAlertPanel(scr, mainw, _("Error"), mbuf, _("OK"), NULL, NULL);
     }
-    free(path);
 
-    file = popen("wmaker -version", "r");
+    path = getenv("WMAKER_BIN_NAME");
+    if (!path)
+	path = "wmaker";
+    {
+	char *command;
+
+	command = wstrconcat(path, " --version");
+	file = popen(command, "r");
+	wfree(command);
+    }
     if (!file || !fgets(buffer, 1023, file)) {
 	wsyserror(_("could not extract version information from Window Maker"));
-	wfatal(_("Make sure Window Maker is in your search path."));
+	wfatal(_("Make sure wmaker is in your search path."));
 	
 	WMRunAlertPanel(scr, mainw, _("Error"),
 			_("Could not extract version from Window Maker. Make sure it is correctly installed and is in your PATH environment variable."),
@@ -568,8 +788,10 @@ loadConfigurations(WMScreen *scr, WMWindow *mainw)
     if (sscanf(buffer, "Window Maker %i.%i.%i",&v1,&v2,&v3)!=3
 	&& sscanf(buffer, "WindowMaker %i.%i.%i",&v1,&v2,&v3)!=3) {
 	WMRunAlertPanel(scr, mainw, _("Error"),
-			_("Could not extract version from Window Maker. Make sure it is correctly installed."),
-			_("OK"), NULL, NULL);
+			_("Could not extract version from Window Maker. "
+			  "Make sure it is correctly installed and the path "
+			  "where it installed is in the PATH environment "
+			  "variable."), _("OK"), NULL, NULL);
 	exit(1);
     }
     if (v1 == 0 && (v2 < 18 || v3 < 0)) {
@@ -585,18 +807,32 @@ loadConfigurations(WMScreen *scr, WMWindow *mainw)
 	WMRunAlertPanel(scr, mainw, _("Warning"), mbuf, _("OK"), NULL, NULL);
     }
 
-    file = popen("wmaker -global_defaults_path", "r");
-    if (!file || !fgets(buffer, 1023, file)) {
-	wsyserror(_("could not run \"wmaker -global_defaults_path\"."));
-	exit(1);
+    {
+	char *command;
+	
+	command = wstrconcat(path, " --global_defaults_path");
+	file = popen(command, "r");
+	wfree(command);
     }
+    if (!file || !fgets(buffer, 1023, file)) {
+	wsyserror(_("could not run \"%s --global_defaults_path\"."), path);
+	exit(1);
+    } else {
+	char *ptr;
+	ptr = strchr(buffer, '\n');
+	if (ptr)
+	    *ptr = 0;
+	strcat(buffer, "/WindowMaker");
+    }
+
     if (file)
 	pclose(file);
 
-    gdb = PLGetProplistWithPath(buffer);
+    gdb = WMReadPropListFromFile(buffer);
+
     if (gdb) {
-	if (!PLIsDictionary(gdb)) {
-	    PLRelease(gdb);
+	if (!WMIsPLDictionary(gdb)) {
+	    WMReleasePropList(gdb);
 	    gdb = NULL;
 	    sprintf(mbuf, _("Window Maker domain (%s) is corrupted!"), buffer);
 	    WMRunAlertPanel(scr, mainw, _("Error"), mbuf, _("OK"), NULL, NULL);
@@ -608,10 +844,10 @@ loadConfigurations(WMScreen *scr, WMWindow *mainw)
     }
 
     if (!db) {
-	db = PLMakeDictionaryFromEntries(NULL, NULL, NULL);
+	db = WMCreatePLDictionary(NULL, NULL, NULL);
     }
     if (!gdb) {
-	gdb = PLMakeDictionaryFromEntries(NULL, NULL, NULL);
+	gdb = WMCreatePLDictionary(NULL, NULL, NULL);
     }
 
     GlobalDB = gdb;
@@ -620,89 +856,89 @@ loadConfigurations(WMScreen *scr, WMWindow *mainw)
 }
 
 
-proplist_t
+WMPropList*
 GetObjectForKey(char *defaultName)
 {
-    proplist_t object = NULL;
-    proplist_t key = PLMakeString(defaultName);
+    WMPropList *object = NULL;
+    WMPropList *key = WMCreatePLString(defaultName);
 
-    object = PLGetDictionaryEntry(WindowMakerDB, key);
+    object = WMGetFromPLDictionary(WindowMakerDB, key);
     if (!object)
-	object = PLGetDictionaryEntry(GlobalDB, key);
+	object = WMGetFromPLDictionary(GlobalDB, key);
 
-    PLRelease(key);
+    WMReleasePropList(key);
 
     return object;
 }
 
 
 void
-SetObjectForKey(proplist_t object, char *defaultName)
+SetObjectForKey(WMPropList *object, char *defaultName)
 {
-    proplist_t key = PLMakeString(defaultName);
+    WMPropList *key = WMCreatePLString(defaultName);
 
-    PLInsertDictionaryEntry(WindowMakerDB, key, object);
-    PLRelease(key);
+    WMPutInPLDictionary(WindowMakerDB, key, object);
+    WMReleasePropList(key);
 }
 
 
 void
 RemoveObjectForKey(char *defaultName)
 {
-    proplist_t key = PLMakeString(defaultName);
+    WMPropList *key = WMCreatePLString(defaultName);
     
-    PLRemoveDictionaryEntry(WindowMakerDB, key);
+    WMRemoveFromPLDictionary(WindowMakerDB, key);
     
-    PLRelease(key);
+    WMReleasePropList(key);
 }
 
 
 char*
 GetStringForKey(char *defaultName)
 {
-    proplist_t val;
+    WMPropList *val;
     
     val = GetObjectForKey(defaultName);
 
     if (!val)
 	return NULL;
 
-    if (!PLIsString(val))
+    if (!WMIsPLString(val))
 	return NULL;
 
-    return PLGetString(val);
+    return WMGetFromPLString(val);
 }
 
 
 
-proplist_t
+WMPropList*
 GetArrayForKey(char *defaultName)
 {
-    proplist_t val;
+    WMPropList *val;
     
     val = GetObjectForKey(defaultName);
     
     if (!val)
 	return NULL;
 
-    if (!PLIsArray(val))
+    if (!WMIsPLArray(val))
 	return NULL;
 
     return val;
 }
 
 
-proplist_t
+WMPropList*
 GetDictionaryForKey(char *defaultName)
 {
-    proplist_t val;
+    WMPropList *val;
 
     val = GetObjectForKey(defaultName);
     
     if (!val)
 	return NULL;
 
-    if (!PLIsDictionary(val))
+    if (!WMIsPLDictionary(val))
 	return NULL;
 
     return val;
@@ -712,7 +948,7 @@ GetDictionaryForKey(char *defaultName)
 int
 GetIntegerForKey(char *defaultName)
 {
-    proplist_t val;
+    WMPropList *val;
     char *str;
     int value;
 
@@ -721,10 +957,10 @@ GetIntegerForKey(char *defaultName)
     if (!val)
 	return 0;
 
-    if (!PLIsString(val))
+    if (!WMIsPLString(val))
 	return 0;
     
-    str = PLGetString(val);
+    str = WMGetFromPLString(val);
     if (!str)
 	return 0;
     
@@ -762,14 +998,14 @@ GetBoolForKey(char *defaultName)
 void
 SetIntegerForKey(int value, char *defaultName)
 {
-    proplist_t object;
+    WMPropList *object;
     char buffer[128];
 
     sprintf(buffer, "%i", value);
-    object = PLMakeString(buffer);
+    object = WMCreatePLString(buffer);
  
     SetObjectForKey(object, defaultName);
-    PLRelease(object);
+    WMReleasePropList(object);
 }
 
 
@@ -777,23 +1013,23 @@ SetIntegerForKey(int value, char *defaultName)
 void
 SetStringForKey(char *value, char *defaultName)
 {
-    proplist_t object;
+    WMPropList *object;
 
-    object = PLMakeString(value);
+    object = WMCreatePLString(value);
  
     SetObjectForKey(object, defaultName);
-    PLRelease(object);
+    WMReleasePropList(object);
 }
 
 
 void
 SetBoolForKey(Bool value, char *defaultName)
 {
-    static proplist_t yes = NULL, no = NULL;
+    static WMPropList *yes = NULL, *no = NULL;
 
     if (!yes) {
-	yes = PLMakeString("YES");
-	no = PLMakeString("NO");
+	yes = WMCreatePLString("YES");
+	no = WMCreatePLString("NO");
     }
 
     SetObjectForKey(value ? yes : no, defaultName);
@@ -837,6 +1073,9 @@ GetSpeedForKey(char *defaultName)
     int i;
     
     str = GetStringForKey(defaultName);
+    if (!str)
+	return 2;
+
     if (strcasecmp(str, "ultraslow")==0)
 	i = 0;
     else if (strcasecmp(str, "slow")==0)
