@@ -23,6 +23,9 @@
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
+#ifdef KEEP_XKB_LOCK_STATUS     
+#include <X11/XKBlib.h>         
+#endif /* KEEP_XKB_LOCK_STATUS */
 
 #include <stdlib.h>
 #include <string.h>
@@ -60,9 +63,10 @@ static void updateTitlebar(WFrameWindow *fwin);
 
 WFrameWindow*
 wFrameWindowCreate(WScreen *scr, int wlevel, int x, int y, 
-		   int width, int height, int flags,
+		   int width, int height, int *clearance, int flags,
 		   WTexture **title_texture, WTexture **resize_texture,
-		   unsigned long *color, GC *gc, WFont **font)
+		   unsigned long *color,
+           GC *gc, WMFont **font)
 {
     WFrameWindow *fwin;
     
@@ -70,18 +74,23 @@ wFrameWindowCreate(WScreen *scr, int wlevel, int x, int y,
     memset(fwin, 0, sizeof(WFrameWindow));
     
     fwin->screen_ptr = scr;
-    fwin->window_level = wlevel;
 
     fwin->flags.single_texture = (flags & WFF_SINGLE_STATE) ? 1 : 0;
     
     fwin->title_texture = title_texture;
     fwin->resizebar_texture = resize_texture;
     fwin->title_pixel = color;
+    fwin->title_clearance = clearance;
     fwin->title_gc = gc;
     fwin->font = font;
-    
+#ifdef KEEP_XKB_LOCK_STATUS
+    fwin->languagemode = XkbGroup1Index;
+    fwin->last_languagemode = XkbGroup2Index;
+#endif
+
     fwin->core = wCoreCreateTopLevel(scr, x, y, width, height, 
-				     FRAME_BORDER_WIDTH);
+				     (flags & WFF_BORDER) 
+				     ? FRAME_BORDER_WIDTH : 0);
     if (wPreferences.use_saveunders) {
 	unsigned long vmask;
 	XSetWindowAttributes attribs;
@@ -115,12 +124,15 @@ wFrameWindowUpdateBorders(WFrameWindow *fwin, int flags)
     int width, height;
     int i;
     WScreen *scr = fwin->screen_ptr;
-    
+
     width = fwin->core->width;
-    height = fwin->core->height - fwin->top_width - fwin->bottom_width;
-    
+    if (flags & WFF_IS_SHADED)
+	height = -1;
+    else
+	height = fwin->core->height - fwin->top_width - fwin->bottom_width;
+
     if (flags & WFF_TITLEBAR)
-	theight = (*fwin->font)->height + TITLEBAR_EXTRA_HEIGHT;
+	theight = WMFontHeight(*fwin->font) + (*fwin->title_clearance + TITLEBAR_EXTEND_SPACE) * 2;
     else
 	theight = 0;
     
@@ -142,6 +154,16 @@ wFrameWindowUpdateBorders(WFrameWindow *fwin, int flags)
 		if (fwin->left_button) {
 		    wCoreConfigure(fwin->left_button, 0, 0, bsize, bsize);
 		}
+#ifdef XKB_BUTTON_HINT
+		if (fwin->language_button)
+		    if (fwin->flags.hide_left_button || !fwin->left_button
+			|| fwin->flags.lbutton_dont_fit) {
+			wCoreConfigure(fwin->language_button, 0, 0, bsize, bsize);
+		    } else {
+			wCoreConfigure(fwin->language_button, bsize, 0, bsize, bsize);
+		    }
+#endif
+
 		
 		if (fwin->right_button) {
 		    wCoreConfigure(fwin->right_button, width-bsize+1,
@@ -152,6 +174,13 @@ wFrameWindowUpdateBorders(WFrameWindow *fwin, int flags)
 		    wCoreConfigure(fwin->left_button, 3, (theight-bsize)/2,
 				   bsize, bsize);
 		}
+        
+#ifdef XKB_BUTTON_HINT
+		if (fwin->language_button) {
+		    wCoreConfigure(fwin->language_button, 6 + bsize, (theight-bsize)/2,
+				   bsize, bsize);
+		}
+#endif
 		
 		if (fwin->right_button) {
 		    wCoreConfigure(fwin->right_button, width-bsize-3,
@@ -163,15 +192,24 @@ wFrameWindowUpdateBorders(WFrameWindow *fwin, int flags)
 	    /* we had a titlebar, but now we don't need it anymore */
 	    for (i=0; i < (fwin->flags.single_texture ? 1 : 3); i++) {
 		FREE_PIXMAP(fwin->title_back[i]);
-                if (wPreferences.new_style) {
+		if (wPreferences.new_style) {
 		    FREE_PIXMAP(fwin->lbutton_back[i]);
 		    FREE_PIXMAP(fwin->rbutton_back[i]);
-                }
+#ifdef XKB_BUTTON_HINT
+		    FREE_PIXMAP(fwin->languagebutton_back[i]);
+#endif
+		}
 	    }
 	    if (fwin->left_button)
 		wCoreDestroy(fwin->left_button);
 	    fwin->left_button = NULL;
 	    
+#ifdef XKB_BUTTON_HINT
+	    if (fwin->language_button)
+		wCoreDestroy(fwin->language_button);
+	    fwin->language_button = NULL;
+#endif
+
 	    if (fwin->right_button)
 		wCoreDestroy(fwin->right_button);
 	    fwin->right_button = NULL;
@@ -189,33 +227,61 @@ wFrameWindowUpdateBorders(WFrameWindow *fwin, int flags)
 	    
 	    fwin->flags.titlebar = 1;
 	    fwin->titlebar = wCoreCreate(fwin->core, 0, 0, width+1, theight);
-	    
+
 	    if (flags & WFF_LEFT_BUTTON) {
 		fwin->flags.left_button = 1;
-                if (wPreferences.new_style) {
-                    fwin->left_button = wCoreCreate(fwin->core, 0, 0,
-                                                    bsize, bsize);
+		if (wPreferences.new_style) {
+		    fwin->left_button = wCoreCreate(fwin->core, 0, 0,
+						    bsize, bsize);
 		    if (width < theight*4) {
 			fwin->flags.lbutton_dont_fit = 1;
 		    } else {
 			XMapRaised(dpy, fwin->left_button->window);
 		    }
-                } else {
+		} else {
 		    fwin->left_button = 
 			wCoreCreate(fwin->titlebar, 3, (theight-bsize)/2,
 				    bsize, bsize);
-
+		    
 		    XSetWindowBackground(dpy, fwin->left_button->window,
 					 scr->widget_texture->normal.pixel);
-
+		    
 		    if (width < theight*3) {
 			fwin->flags.lbutton_dont_fit = 1;
 		    } else {
 			XMapRaised(dpy, fwin->left_button->window);
 		    }
-                }
+		}
 	    }
-	    
+        
+#ifdef XKB_BUTTON_HINT
+	    if (flags & WFF_LANGUAGE_BUTTON) {
+		fwin->flags.language_button = 1;
+		if (wPreferences.new_style) {
+		    fwin->language_button = wCoreCreate(fwin->core,
+							bsize, 0, bsize, bsize);
+		    
+		    if (width < theight*4) {
+			fwin->flags.languagebutton_dont_fit = 1;
+		    } else {
+			XMapRaised(dpy, fwin->language_button->window);
+		    }
+		} else {
+		    fwin->language_button = 
+			wCoreCreate(fwin->titlebar, bsize + 6, (theight-bsize)/2,
+				    bsize, bsize); 
+		    
+		    XSetWindowBackground(dpy, fwin->language_button->window,
+					 scr->widget_texture->normal.pixel); 
+		    
+		    if (width < theight*3) {
+			fwin->flags.languagebutton_dont_fit = 1;
+		    } else {
+			XMapRaised(dpy, fwin->language_button->window);
+		    }
+		}
+	    }
+#endif
 	    
 	    if (flags & WFF_RIGHT_BUTTON) {
 		fwin->flags.right_button = 1;
@@ -247,14 +313,10 @@ wFrameWindowUpdateBorders(WFrameWindow *fwin, int flags)
 	}
     }
     checkTitleSize(fwin);
-
-    if (flags & WFF_RESIZEBAR) {
-	fwin->bottom_width = RESIZEBAR_HEIGHT;
-    } else {
-	fwin->bottom_width = 0;
-    } 
     
     if (flags & WFF_RESIZEBAR) {
+	fwin->bottom_width = RESIZEBAR_HEIGHT;
+	
 	if (!fwin->resizebar) {
 	    fwin->flags.resizebar = 1;
 	    fwin->resizebar = wCoreCreate(fwin->core, 0,
@@ -278,18 +340,28 @@ wFrameWindowUpdateBorders(WFrameWindow *fwin, int flags)
 	    }
 	}
     } else {
+	fwin->bottom_width = 0;
+	
 	if (fwin->resizebar) {
 	    fwin->bottom_width = 0;
 	    wCoreDestroy(fwin->resizebar);
 	    fwin->resizebar = NULL;
 	}
     }
-    
-    if (height + fwin->top_width + fwin->bottom_width != fwin->core->height) {
+
+    if (height + fwin->top_width + fwin->bottom_width != fwin->core->height
+	&& !(flags & WFF_IS_SHADED)) {
 	wFrameWindowResize(fwin, width,
 			   height + fwin->top_width + fwin->bottom_width);
     }
     
+    
+    if (flags & WFF_BORDER) {
+	XSetWindowBorderWidth(dpy, fwin->core->window, FRAME_BORDER_WIDTH);
+    } else {
+	XSetWindowBorderWidth(dpy, fwin->core->window, 0);
+    }
+    
     /* setup object descriptors */
     
     if (fwin->titlebar) {
@@ -313,6 +385,16 @@ wFrameWindowUpdateBorders(WFrameWindow *fwin, int flags)
 	fwin->left_button->descriptor.handle_mousedown = buttonMouseDown;
     }
     
+#ifdef XKB_BUTTON_HINT
+    if (fwin->language_button) {
+	fwin->language_button->descriptor.handle_expose = handleButtonExpose;
+	fwin->language_button->descriptor.parent = fwin;
+	fwin->language_button->descriptor.parent_type = WCLASS_FRAME;
+	fwin->language_button->descriptor.handle_mousedown = buttonMouseDown;
+    }
+#endif
+
+    
     if (fwin->right_button) {
 	fwin->right_button->descriptor.parent = fwin;
 	fwin->right_button->descriptor.parent_type = WCLASS_FRAME;
@@ -333,6 +415,11 @@ wFrameWindowDestroy(WFrameWindow *fwin)
     if (fwin->left_button)
 	wCoreDestroy(fwin->left_button);
     
+#ifdef XKB_BUTTON_HINT
+    if (fwin->language_button)
+    wCoreDestroy(fwin->language_button);
+#endif
+    
     if (fwin->right_button)
 	wCoreDestroy(fwin->right_button);
     
@@ -347,17 +434,20 @@ wFrameWindowDestroy(WFrameWindow *fwin)
     wCoreDestroy(fwin->core);
     
     if (fwin->title)
-	free(fwin->title);
+	wfree(fwin->title);
     
     for (i=0; i < (fwin->flags.single_texture ? 1 : 3); i++) {
 	FREE_PIXMAP(fwin->title_back[i]);
         if (wPreferences.new_style) {
 	    FREE_PIXMAP(fwin->lbutton_back[i]);
+#ifdef XKB_BUTTON_HINT
+	    FREE_PIXMAP(fwin->languagebutton_back[i]);
+#endif
 	    FREE_PIXMAP(fwin->rbutton_back[i]);
         }
     }
     
-    free(fwin);
+    wfree(fwin);
 }
 
 
@@ -380,7 +470,7 @@ updateTitlebar(WFrameWindow *fwin)
     int x, w;
     int theight;
     
-    theight = (*fwin->font)->height + TITLEBAR_EXTRA_HEIGHT;
+	theight = WMFontHeight(*fwin->font) + (*fwin->title_clearance + TITLEBAR_EXTEND_SPACE) * 2;
 
     x = 0;
     w = fwin->core->width + 1;
@@ -389,11 +479,50 @@ updateTitlebar(WFrameWindow *fwin)
 	if (fwin->flags.hide_left_button || !fwin->left_button
 	    || fwin->flags.lbutton_dont_fit) {
 	    x = 0;
+#ifdef XKB_BUTTON_HINT
+        if(fwin->language_button)
+            wCoreConfigure(fwin->language_button, 0, 0,
+                    fwin->language_button->width,
+                    fwin->language_button->width);
+#endif
 	} else {
+#ifdef XKB_BUTTON_HINT
+        if(fwin->language_button)
+            wCoreConfigure(fwin->language_button, fwin->left_button->width, 0,
+                    fwin->language_button->width,
+                    fwin->language_button->width);
+#endif
 	    x = fwin->left_button->width;
 	    w -= fwin->left_button->width;
 	}
+#ifdef XKB_BUTTON_HINT
+	if (fwin->flags.hide_language_button || !fwin->language_button
+	    || fwin->flags.languagebutton_dont_fit) {
+    } else {
+	    x += fwin->language_button->width;
+	    w -= fwin->language_button->width;
     }
+#endif
+    }
+#ifdef XKB_BUTTON_HINT
+    else {
+        int bsize = theight - 7;
+        if (fwin->flags.hide_left_button || !fwin->left_button
+            || fwin->flags.lbutton_dont_fit) {
+            if(fwin->language_button)
+                wCoreConfigure(fwin->language_button, 3, (theight-bsize)/2,
+                        fwin->language_button->width,
+                        fwin->language_button->width);
+        }
+        else {
+            if(fwin->language_button)
+                wCoreConfigure(fwin->language_button,
+                        6 + fwin->left_button->width, (theight-bsize)/2,
+                        fwin->language_button->width,
+                        fwin->language_button->width);
+        }
+    }
+#endif
     
     if (wPreferences.new_style) {
 	if (!fwin->flags.hide_right_button && fwin->right_button
@@ -421,13 +550,24 @@ wFrameWindowHideButton(WFrameWindow *fwin, int flags)
 	XUnmapWindow(dpy, fwin->left_button->window);
 	fwin->flags.hide_left_button = 1;
     }
+    
+#ifdef XKB_BUTTON_HINT
+    if ((flags & WFF_LANGUAGE_BUTTON) && fwin->language_button) {
+	XUnmapWindow(dpy, fwin->language_button->window);
+    fwin->flags.hide_language_button = 1;
+    }
+#endif
 
     if (fwin->titlebar) {
 	if (wPreferences.new_style) {
 	    updateTitlebar(fwin);
 	} else {
+#ifdef XKB_BUTTON_HINT
+	    updateTitlebar(fwin);
+#else
 	    XClearWindow(dpy, fwin->titlebar->window);
 	    wFrameWindowPaint(fwin);
+#endif
 	}
 	checkTitleSize(fwin);
     }
@@ -445,6 +585,17 @@ wFrameWindowShowButton(WFrameWindow *fwin, int flags)
 	
 	fwin->flags.hide_right_button = 0;
     }
+
+#ifdef XKB_BUTTON_HINT
+    if ((flags & WFF_LANGUAGE_BUTTON) && fwin->language_button
+	&& fwin->flags.hide_language_button) {
+	
+	if (!fwin->flags.languagebutton_dont_fit)
+	    XMapWindow(dpy, fwin->language_button->window);
+	
+	fwin->flags.hide_language_button = 0;
+    }
+#endif
     
     if ((flags & WFF_LEFT_BUTTON) && fwin->left_button
 	&& fwin->flags.hide_left_button) {
@@ -469,61 +620,33 @@ wFrameWindowShowButton(WFrameWindow *fwin, int flags)
 
 
 static void
+#ifdef XKB_BUTTON_HINT
+renderTexture(WScreen *scr, WTexture *texture, int width, int height, 
+	      int bwidth, int bheight, int left, int language, int right,
+	      Pixmap *title, Pixmap *lbutton, Pixmap *languagebutton, Pixmap *rbutton)
+#else
 renderTexture(WScreen *scr, WTexture *texture, int width, int height, 
 	      int bwidth, int bheight, int left, int right,
 	      Pixmap *title, Pixmap *lbutton, Pixmap *rbutton)
+#endif
 {
     RImage *img;
     RImage *limg, *rimg, *mimg;
-    RColor color1, color2;
-    int i, x, w;
-    
+#ifdef XKB_BUTTON_HINT
+    RImage *timg;
+#endif
+    int x, w;
+
     *title = None;
     *lbutton = None;
     *rbutton = None;
-    
-    switch (texture->any.type) {
-     case WTEX_DGRADIENT:
-     case WTEX_VGRADIENT:
-     case WTEX_HGRADIENT:
-     case WTEX_MHGRADIENT:
-     case WTEX_MVGRADIENT:
-     case WTEX_MDGRADIENT:
-	if (texture->any.type==WTEX_DGRADIENT
-	    || texture->any.type==WTEX_MDGRADIENT)
-	    i = RGRD_DIAGONAL;
-	else if (texture->any.type==WTEX_HGRADIENT 
-		 || texture->any.type==WTEX_MHGRADIENT)
-	    i = RGRD_HORIZONTAL;
-	else
-	    i = RGRD_VERTICAL;
-	
-	if (texture->any.type==WTEX_MHGRADIENT
-	    || texture->any.type==WTEX_MDGRADIENT
-	    || texture->any.type==WTEX_MVGRADIENT)
-	    img = RRenderMultiGradient(width, height,
-				       &(texture->mgradient.colors[1]), i);
-	else {
-	    color1.red = texture->gradient.color1.red >> 8;
-	    color1.green = texture->gradient.color1.green >> 8;
-	    color1.blue = texture->gradient.color1.blue >> 8;
-	    color2.red = texture->gradient.color2.red >> 8;
-	    color2.green = texture->gradient.color2.green >> 8;
-	    color2.blue = texture->gradient.color2.blue >> 8;	    
-	    img = RRenderGradient(width, height, &color1, &color2, i);
-        }
-        break;
-	
-     case WTEX_PIXMAP:
-        img = wTextureRenderImage(texture, width, height, WREL_FLAT);
-        break;
-	
-     default:
-        img = NULL;
-    }
-    
+#ifdef XKB_BUTTON_HINT
+    *languagebutton = None;
+#endif
+
+    img = wTextureRenderImage(texture, width, height, WREL_FLAT);
     if (!img) {
-        wwarning(_("could not render gradient: %s"), RMessageForError(RErrorCode));
+        wwarning(_("could not render texture: %s"), RMessageForError(RErrorCode));
         return;
     }
     
@@ -535,6 +658,13 @@ renderTexture(WScreen *scr, WTexture *texture, int width, int height,
 	
 	x = 0;
 	w = img->width;
+    
+#ifdef XKB_BUTTON_HINT
+    if (language) {
+	    timg = RGetSubImage(img, bwidth * left, 0, bwidth, bheight);
+	} else
+	    timg = NULL;
+#endif
 	
 	if (limg) {
 	    RBevelImage(limg, RBEV_RAISED2);
@@ -543,8 +673,20 @@ renderTexture(WScreen *scr, WTexture *texture, int width, int height,
 	    }
 	    x += limg->width;
 	    w -= limg->width;
-	    RDestroyImage(limg);
+	    RReleaseImage(limg);
 	}
+
+#ifdef XKB_BUTTON_HINT
+    if (timg) {
+	    RBevelImage(timg, RBEV_RAISED2);
+	    if (!RConvertImage(scr->rcontext, timg, languagebutton)) {
+		wwarning(_("error rendering image:%s"), RMessageForError(RErrorCode));
+	    }
+	    x += timg->width;
+	    w -= timg->width;
+	    RReleaseImage(timg);
+    }
+#endif
 	
 	if (right) {
 	    rimg = RGetSubImage(img, width - bwidth, 0,	bwidth, bheight);
@@ -557,7 +699,7 @@ renderTexture(WScreen *scr, WTexture *texture, int width, int height,
 		wwarning(_("error rendering image:%s"), RMessageForError(RErrorCode));
 	    }
 	    w -= rimg->width;
-	    RDestroyImage(rimg);
+	    RReleaseImage(rimg);
 	}
 	
 	if (w!=width) {
@@ -567,7 +709,7 @@ renderTexture(WScreen *scr, WTexture *texture, int width, int height,
 	    if (!RConvertImage(scr->rcontext, mimg, title)) {
 		wwarning(_("error rendering image:%s"), RMessageForError(RErrorCode));
 	    }
-	    RDestroyImage(mimg);
+	    RReleaseImage(mimg);
 	} else {
 	    RBevelImage(img, RBEV_RAISED2);
 	    
@@ -583,10 +725,63 @@ renderTexture(WScreen *scr, WTexture *texture, int width, int height,
 	}
     }
 
-    RDestroyImage(img);
+    RReleaseImage(img);
 }
 
 
+static void
+renderResizebarTexture(WScreen *scr, WTexture *texture, int width, int height,
+		       int cwidth, Pixmap *pmap)
+{
+    RImage *img;
+    RColor light;
+    RColor dark;
+
+    *pmap = None;
+
+    img = wTextureRenderImage(texture, width, height, WREL_FLAT);
+    if (!img) {
+        wwarning(_("could not render texture: %s"),
+		 RMessageForError(RErrorCode));
+        return;
+    }
+
+    light.alpha = 0;
+    light.red = light.green = light.blue = 80;
+
+    dark.alpha = 0;
+    dark.red = dark.green = dark.blue = 40;
+
+    ROperateLine(img, RSubtractOperation, 0, 0, width-1, 0, &dark);
+    ROperateLine(img, RAddOperation, 0, 1, width-1, 1, &light);
+
+    ROperateLine(img, RSubtractOperation, cwidth, 2, cwidth, height-1, &dark);
+    ROperateLine(img, RAddOperation, cwidth+1, 2, cwidth+1, height-1, &light);
+
+    if (width > 1)
+	ROperateLine(img, RSubtractOperation, width-cwidth-2, 2, 
+		     width-cwidth-2, height-1, &dark);
+    ROperateLine(img, RAddOperation, width-cwidth-1, 2, width-cwidth-1, 
+		 height-1, &light);
+
+#ifdef SHADOW_RESIZEBAR
+    ROperateLine(img, RAddOperation, 0, 1, 0, height-1, &light);
+    ROperateLine(img, RSubtractOperation, width-1, 1, width-1, height-1,
+		 &dark);
+    ROperateLine(img, RSubtractOperation, 0, height-1, width-1, height-1,
+		 &dark);
+#endif /* SHADOW_RESIZEBAR */
+
+
+    if (!RConvertImage(scr->rcontext, img, pmap)) {
+	wwarning(_("error rendering image: %s"), RMessageForError(RErrorCode));
+    }
+
+    RReleaseImage(img);
+}
+
+
+
 static void
 updateTexture(WFrameWindow *fwin)
 {
@@ -602,6 +797,13 @@ updateTexture(WFrameWindow *fwin)
 		if (fwin->left_button && fwin->lbutton_back[i])
 		    XSetWindowBackgroundPixmap(dpy, fwin->left_button->window,
 					       fwin->lbutton_back[i]);
+
+#ifdef XKB_BUTTON_HINT
+		if (fwin->language_button && fwin->languagebutton_back[i]) {
+		    XSetWindowBackgroundPixmap(dpy, fwin->language_button->window,
+					       fwin->languagebutton_back[i]);
+        }
+#endif
 		
 		if (fwin->right_button && fwin->rbutton_back[i])
 		    XSetWindowBackgroundPixmap(dpy, fwin->right_button->window,
@@ -614,6 +816,11 @@ updateTexture(WFrameWindow *fwin)
 		if (fwin->left_button)
 		    XSetWindowBackground(dpy, fwin->left_button->window,
 					 pixel);
+#ifdef XKB_BUTTON_HINT
+		if (fwin->language_button)
+		    XSetWindowBackground(dpy, fwin->language_button->window,
+					 pixel);
+#endif
 		if (fwin->right_button)
 		    XSetWindowBackground(dpy, fwin->right_button->window,
 					 pixel);
@@ -625,17 +832,17 @@ updateTexture(WFrameWindow *fwin)
 	    XClearWindow(dpy, fwin->left_button->window);
 	    handleButtonExpose(&fwin->left_button->descriptor, NULL);
 	}
+#ifdef XKB_BUTTON_HINT
+       if (fwin->language_button) {
+           XClearWindow(dpy, fwin->language_button->window);
+           handleButtonExpose(&fwin->language_button->descriptor, NULL);
+       }
+#endif
 	if (fwin->right_button) {
 	    XClearWindow(dpy, fwin->right_button->window);
 	    handleButtonExpose(&fwin->right_button->descriptor, NULL);
 	}
     }
-    
-    if (fwin->resizebar) {
-	XSetWindowBackground(dpy, fwin->resizebar->window, 
-			     fwin->resizebar_texture[0]->solid.normal.pixel);
-	XClearWindow(dpy, fwin->resizebar->window);
-    }
 }
 
 
@@ -644,44 +851,95 @@ static void
 remakeTexture(WFrameWindow *fwin, int state)
 {
     Pixmap pmap, lpmap, rpmap;
+#ifdef XKB_BUTTON_HINT
+    Pixmap tpmap;
+#endif
 
     if (fwin->title_texture[state] && fwin->titlebar) {
 	FREE_PIXMAP(fwin->title_back[state]);
 	if (wPreferences.new_style) {
 	    FREE_PIXMAP(fwin->lbutton_back[state]);
 	    FREE_PIXMAP(fwin->rbutton_back[state]);
+#ifdef XKB_BUTTON_HINT
+	    FREE_PIXMAP(fwin->languagebutton_back[state]);
+#endif
 	}
 
 	if (fwin->title_texture[state]->any.type!=WTEX_SOLID) {
 	    int left, right;
+#ifdef XKB_BUTTON_HINT
+        int language;
+#endif
 	    int width;
 
 	    /* eventually surrounded by if new_style */
 	    left = fwin->left_button && !fwin->flags.hide_left_button
 		&& !fwin->flags.lbutton_dont_fit;
+#ifdef XKB_BUTTON_HINT
+	    language = fwin->language_button && !fwin->flags.hide_language_button
+		&& !fwin->flags.languagebutton_dont_fit;
+#endif
 	    right = fwin->right_button && !fwin->flags.hide_right_button
 		&& !fwin->flags.rbutton_dont_fit;
-	    
+
 	    width = fwin->core->width+1;
-	    
+
+#ifdef XKB_BUTTON_HINT
+	    renderTexture(fwin->screen_ptr, fwin->title_texture[state],
+			  width, fwin->titlebar->height,
+			  fwin->titlebar->height, fwin->titlebar->height,
+			  left, language, right, &pmap, &lpmap, &tpmap, &rpmap);
+#else
 	    renderTexture(fwin->screen_ptr, fwin->title_texture[state],
 			  width, fwin->titlebar->height,
 			  fwin->titlebar->height, fwin->titlebar->height,
 			  left, right, &pmap, &lpmap, &rpmap);
-	    
+#endif
+
 	    fwin->title_back[state] = pmap;
 	    if (wPreferences.new_style) {
 		fwin->lbutton_back[state] = lpmap;
 		fwin->rbutton_back[state] = rpmap;
+#ifdef XKB_BUTTON_HINT
+		fwin->languagebutton_back[state] = tpmap;
+#endif
 	    }
 	}
     }
+    if (fwin->resizebar_texture && fwin->resizebar_texture[0] 
+	&& fwin->resizebar && state == 0) {
+
+	FREE_PIXMAP(fwin->resizebar_back[0]);
+
+	if (fwin->resizebar_texture[0]->any.type!=WTEX_SOLID) {
+
+	    renderResizebarTexture(fwin->screen_ptr,
+				   fwin->resizebar_texture[0],
+				   fwin->resizebar->width,
+				   fwin->resizebar->height,
+				   fwin->resizebar_corner_width,
+				   &pmap);
+
+	    fwin->resizebar_back[0] = pmap;
+	}
+	
+	/* this part should be in updateTexture() */
+	if (fwin->resizebar_texture[0]->any.type!=WTEX_SOLID) {
+	    XSetWindowBackgroundPixmap(dpy, fwin->resizebar->window,
+				       fwin->resizebar_back[0]);
+	} else {
+	    XSetWindowBackground(dpy, fwin->resizebar->window,
+				 fwin->resizebar_texture[0]->solid.normal.pixel);
+	}
+	XClearWindow(dpy, fwin->resizebar->window);
+    }
 }
 
 
 void
 wFrameWindowPaint(WFrameWindow *fwin)
 {
+    WScreen *scr = fwin->screen_ptr;
 
     if (fwin->flags.is_client_window_frame)
 	fwin->flags.justification = wPreferences.title_justification;
@@ -713,30 +971,30 @@ wFrameWindowPaint(WFrameWindow *fwin)
 	
 	updateTexture(fwin);
     }
-    
+
     if (fwin->titlebar && !fwin->flags.repaint_only_resizebar
 	&& fwin->title_texture[fwin->flags.state]->any.type==WTEX_SOLID) {
-	wDrawBevel(fwin->titlebar->window, fwin->titlebar->width,
-		   fwin->titlebar->height,
-		   (WTexSolid*)fwin->title_texture[fwin->flags.state], 
-		   WREL_RAISED);
+        wDrawBevel(fwin->titlebar->window, fwin->titlebar->width,
+                fwin->titlebar->height,
+                (WTexSolid*)fwin->title_texture[fwin->flags.state], 
+                WREL_RAISED);
     }
-    
-    if (fwin->resizebar
-	&& !fwin->flags.repaint_only_titlebar) {
+
+    if (fwin->resizebar	&& !fwin->flags.repaint_only_titlebar
+	&& fwin->resizebar_texture[0]->any.type == WTEX_SOLID) {
 	Window win;
 	int w, h;
 	int cw;
 	GC light_gc, dim_gc;
 	WTexSolid *texture = (WTexSolid*)fwin->resizebar_texture[0];
-	
+
 	w = fwin->resizebar->width;
 	h = fwin->resizebar->height;
 	cw = fwin->resizebar_corner_width;
 	light_gc = texture->light_gc;
 	dim_gc = texture->dim_gc;
 	win = fwin->resizebar->window;
-	
+
 	XDrawLine(dpy, win, dim_gc, 0, 0, w, 0);
 	XDrawLine(dpy, win, light_gc, 0, 1, w, 1);
 	
@@ -756,13 +1014,11 @@ wFrameWindowPaint(WFrameWindow *fwin)
     }
     
     
-    if (fwin->title && fwin->titlebar
-	&& !fwin->flags.repaint_only_resizebar) {
-	int x, w;
-	int lofs = 6, rofs = 6;
-	int titlelen;
-	char *title;
-	int allButtons = 1;
+    if (fwin->titlebar && !fwin->flags.repaint_only_resizebar) {
+        int x, w;
+        int lofs = 6, rofs = 6;
+        int titlelen;
+        int allButtons = 1;
 
 
         if (!wPreferences.new_style) {
@@ -771,6 +1027,14 @@ wFrameWindowPaint(WFrameWindow *fwin)
 		lofs += fwin->left_button->width + 3;
 	    else
 		allButtons = 0;
+
+#ifdef XKB_BUTTON_HINT
+	    if (fwin->language_button && !fwin->flags.hide_language_button
+		&& !fwin->flags.languagebutton_dont_fit)
+		lofs += fwin->language_button->width;
+	    else
+		allButtons = 0;
+#endif
 	    
 	    if (fwin->right_button && !fwin->flags.hide_right_button
 		&& !fwin->flags.rbutton_dont_fit)
@@ -778,48 +1042,71 @@ wFrameWindowPaint(WFrameWindow *fwin)
 	    else
 		allButtons = 0;
         }
-	title = ShrinkString(*fwin->font, fwin->title,
-			     fwin->titlebar->width - lofs - rofs);
-	titlelen = strlen(title);
-	w = wTextWidth((*fwin->font)->font, title, titlelen);
 
-	switch (fwin->flags.justification) {
-	 case WTJ_LEFT:
-	    x = lofs;
-	    break;
+#ifdef XKB_BUTTON_HINT
+        fwin->languagebutton_image =
+            scr->b_pixmaps[WBUT_XKBGROUP1 + fwin->languagemode];
+#endif
 
-	 case WTJ_RIGHT:
-	    x = fwin->titlebar->width - w - rofs;
-	    break;
-
-	 default:
-	    if (!allButtons)
-		x = lofs + (fwin->titlebar->width - w - lofs - rofs) / 2;
-	    else
-		x = (fwin->titlebar->width - w) / 2;
-	    break;
+	if (fwin->title) {
+	    char *title;
+	    
+	    title = ShrinkString(*fwin->font, fwin->title,
+				 fwin->titlebar->width - lofs - rofs);
+	    titlelen = strlen(title);
+	    w = WMWidthOfString(*fwin->font, title, titlelen);
+	    
+	    switch (fwin->flags.justification) {
+	     case WTJ_LEFT:
+		x = lofs;
+		break;
+		
+	     case WTJ_RIGHT:
+		x = fwin->titlebar->width - w - rofs;
+		break;
+		
+	     default:
+		if (!allButtons)
+		    x = lofs + (fwin->titlebar->width - w - lofs - rofs) / 2;
+		else
+		    x = (fwin->titlebar->width - w) / 2;
+		break;
+	    }
+	    
+	    XSetForeground(dpy, *fwin->title_gc, 
+			   fwin->title_pixel[fwin->flags.state]);
+	    
+	    WMDrawString(scr->wmscreen, fwin->titlebar->window, 
+			 *fwin->title_gc, *fwin->font, x, *fwin->title_clearance + TITLEBAR_EXTEND_SPACE, 
+			 title, titlelen);
+	    
+	    wfree(title);
 	}
-
-	XSetForeground(dpy, *fwin->title_gc, 
-		       fwin->title_pixel[fwin->flags.state]);
 	
-	wDrawString(fwin->titlebar->window, *fwin->font,
-		    *fwin->title_gc, x, 
-		    (*fwin->font)->y + TITLEBAR_EXTRA_HEIGHT/2, title,
-		    titlelen);
-	
-	free(title);
+	if (fwin->left_button)
+	    handleButtonExpose(&fwin->left_button->descriptor, NULL);
+	if (fwin->right_button)
+	    handleButtonExpose(&fwin->right_button->descriptor, NULL);
+#ifdef XKB_BUTTON_HINT
+	if (fwin->language_button)
+	    handleButtonExpose(&fwin->language_button->descriptor, NULL);
+#endif 
     }
 }
 
 
-void 
-wFrameWindowResize(WFrameWindow *fwin, int width, int height)
+static void
+reconfigure(WFrameWindow *fwin, int x, int y, int width, int height,
+	    Bool dontMove)
 {
     int k = (wPreferences.new_style ? 4 : 3);
     int resizedHorizontally = 0;
     
-    XResizeWindow(dpy, fwin->core->window, width, height);
+    if (dontMove)
+	XResizeWindow(dpy, fwin->core->window, width, height);
+    else
+	XMoveResizeWindow(dpy, fwin->core->window, x, y, width, height);
+
     /*
      if (fwin->core->height != height && fwin->resizebar)
      XMoveWindow(dpy, fwin->resizebar->window, 0,
@@ -852,6 +1139,24 @@ wFrameWindowResize(WFrameWindow *fwin, int width, int height)
 		fwin->flags.lbutton_dont_fit = 0;
 	    }
 	}
+    
+#ifdef XKB_BUTTON_HINT
+	if (fwin->language_button) {
+	    if (width < fwin->top_width*k && !fwin->flags.languagebutton_dont_fit) {
+		
+		if (!fwin->flags.hide_language_button) {
+		    XUnmapWindow(dpy, fwin->language_button->window);
+		}
+		fwin->flags.languagebutton_dont_fit = 1;
+	    } else if (width >= fwin->top_width*k && fwin->flags.languagebutton_dont_fit) {
+		
+		if (!fwin->flags.hide_language_button) {
+		    XMapWindow(dpy, fwin->language_button->window);
+		}
+		fwin->flags.languagebutton_dont_fit = 0;
+	    }
+	}
+#endif
 	
 	if (fwin->right_button) {
 	    if (width < fwin->top_width*2 && !fwin->flags.rbutton_dont_fit) {
@@ -895,12 +1200,16 @@ wFrameWindowResize(WFrameWindow *fwin, int width, int height)
     }
 }
 
+void 
+wFrameWindowConfigure(WFrameWindow *fwin, int x, int y, int width, int height)
+{
+    reconfigure(fwin, x, y, width, height, False);
+}
 
 void
-wFrameWindowResizeInternal(WFrameWindow *fwin, int iwidth, int iheight)
+wFrameWindowResize(WFrameWindow *fwin, int width, int height)
 {
-    wFrameWindowResize(fwin, iwidth,
-		       iheight + fwin->top_width + fwin->bottom_width);
+    reconfigure(fwin, 0, 0, width, height, True);
 }
 
 
@@ -919,21 +1228,45 @@ wFrameWindowChangeTitle(WFrameWindow *fwin, char *new_title)
     }
 
     if (fwin->title)
-	free(fwin->title);
-    
+	wfree(fwin->title);
+
     fwin->title = wstrdup(new_title);
-    
+
     if (fwin->titlebar) {
 	XClearWindow(dpy, fwin->titlebar->window);
 
 	wFrameWindowPaint(fwin);
     }
     checkTitleSize(fwin);
-    
+
     return 1;
 }
 
 
+#ifdef OLWM_HINTS
+void
+wFrameWindowUpdatePushButton(WFrameWindow *fwin, Bool pushed)
+{
+    fwin->flags.right_button_pushed_in = pushed;
+
+    paintButton(fwin->right_button, fwin->title_texture[fwin->flags.state],
+		fwin->title_pixel[fwin->flags.state],
+		fwin->rbutton_image, pushed);
+}
+#endif /* OLWM_HINTS */
+
+
+#ifdef XKB_BUTTON_HINT
+void
+wFrameWindowUpdateLanguageButton(WFrameWindow *fwin)
+{
+    paintButton(fwin->language_button, fwin->title_texture[fwin->flags.state],
+		fwin->title_pixel[fwin->flags.state],
+		fwin->languagebutton_image, True);
+}
+#endif /* XKB_BUTTON_HINT */
+
+
 /*********************************************************************/
 
 static void 
@@ -973,19 +1306,25 @@ checkTitleSize(WFrameWindow *fwin)
 	if (fwin->left_button && !fwin->flags.hide_left_button
 	    && !fwin->flags.lbutton_dont_fit)
 	    width -= fwin->left_button->width + 3;
+
+#ifdef XKB_BUTTON_HINT
+	if (fwin->language_button && !fwin->flags.hide_language_button
+	    && !fwin->flags.languagebutton_dont_fit)
+	    width -= fwin->language_button->width + 3;
+#endif
 	
 	if (fwin->right_button && !fwin->flags.hide_right_button
 	    && !fwin->flags.rbutton_dont_fit)
 	    width -= fwin->right_button->width + 3;
     }
-    if (wTextWidth((*fwin->font)->font, fwin->title, 
-		    strlen(fwin->title)) > width) {
+    if (WMWidthOfString(*fwin->font, fwin->title, strlen(fwin->title)) > width) {
 	fwin->flags.incomplete_title = 1;
     } else {
 	fwin->flags.incomplete_title = 0;
     }
 }
 
+
 static void
 paintButton(WCoreWindow *button, WTexture *texture, unsigned long color,
 	    WPixmap *image, int pushed)
@@ -1084,15 +1423,31 @@ handleButtonExpose(WObjDescriptor *desc, XEvent *event)
 {
     WFrameWindow *fwin = (WFrameWindow*)desc->parent;
     WCoreWindow *button = (WCoreWindow*)desc->self;
-    
+
+#ifdef XKB_BUTTON_HINT
+    if (button == fwin->language_button) {
+        if (wPreferences.modelock){
+            paintButton(button, fwin->title_texture[fwin->flags.state],
+                    fwin->title_pixel[fwin->flags.state],
+                    fwin->languagebutton_image, False);
+        }
+    } else 
+#endif
     if (button == fwin->left_button) {
-	paintButton(button, fwin->title_texture[fwin->flags.state],
-		    fwin->title_pixel[fwin->flags.state],
-		    fwin->lbutton_image, False);
+        paintButton(button, fwin->title_texture[fwin->flags.state],
+                fwin->title_pixel[fwin->flags.state],
+                fwin->lbutton_image, False);
     } else {
+        Bool pushed = False;
+
+#ifdef OLWM_HINTS
+	if (fwin->flags.right_button_pushed_in)
+	    pushed = True;
+#endif
+	/* emulate the olwm pushpin in the "out" state */
 	paintButton(button, fwin->title_texture[fwin->flags.state],
 		    fwin->title_pixel[fwin->flags.state],
-		    fwin->rbutton_image, False);
+		    fwin->rbutton_image, pushed);
     }
 }
 
@@ -1148,6 +1503,12 @@ buttonMouseDown(WObjDescriptor *desc, XEvent *event)
     } else {
 	image = fwin->rbutton_image;
     } 
+#ifdef XKB_BUTTON_HINT
+    if (button == fwin->language_button) {
+    if (!wPreferences.modelock) return;
+	image = fwin->languagebutton_image;
+    }
+#endif
     
     pixel = fwin->title_pixel[fwin->flags.state];
     texture = fwin->title_texture[fwin->flags.state];
@@ -1189,6 +1550,13 @@ buttonMouseDown(WObjDescriptor *desc, XEvent *event)
 	    if (fwin->on_click_right)
 		(*fwin->on_click_right)(button, fwin->child, &ev);
 	}
+#ifdef XKB_BUTTON_HINT
+	else if (button == fwin->language_button) {
+	    if (fwin->on_click_language)
+		(*fwin->on_click_language)(button, fwin->child, &ev);
+	}
+#endif
+
     }
 }
 
