@@ -19,6 +19,8 @@
  *  USA.
  */
 
+
+
 /* anywhere */
 #define WKBD_ROOTMENU		0
 #define WKBD_WINDOWMENU		1
@@ -28,53 +30,61 @@
 #define WKBD_HIDE		4
 #define WKBD_MAXIMIZE		5
 #define WKBD_VMAXIMIZE		6
-#define WKBD_SELECT		7
+#define WKBD_HMAXIMIZE		7
+#define WKBD_SELECT		8
 /* Clip */
-#define WKBD_CLIPLOWER		8
-#define WKBD_CLIPRAISE		9
-#define WKBD_CLIPRAISELOWER	10
-/* window, menu */
-#define WKBD_RAISE		11
-#define WKBD_LOWER		12
-#define WKBD_RAISELOWER		13
+#define WKBD_CLIPLOWER		9
+#define WKBD_CLIPRAISE		10
+#define WKBD_CLIPRAISELOWER	11
 /* window */
-#define WKBD_SHADE		14
+#define WKBD_RAISE		12
+#define WKBD_LOWER		13
+#define WKBD_RAISELOWER		14
+#define WKBD_MOVERESIZE		15
+#define WKBD_SHADE		16
 /* window, menu */
-#define WKBD_CLOSE		15
+#define WKBD_CLOSE		17
 /* window */
-#define WKBD_FOCUSNEXT		16
-#define WKBD_FOCUSPREV		17
+#define WKBD_FOCUSNEXT		18
+#define WKBD_FOCUSPREV		19
 
-#define WKBD_WORKSPACE1		20
-#define WKBD_WORKSPACE2		21
-#define WKBD_WORKSPACE3		22
-#define WKBD_WORKSPACE4		23
-#define WKBD_WORKSPACE5		24
-#define WKBD_WORKSPACE6		25
-#define WKBD_WORKSPACE7		26
-#define WKBD_WORKSPACE8		27
-#define WKBD_WORKSPACE9		28
-#define WKBD_WORKSPACE10	29
-#define WKBD_NEXTWORKSPACE	30
-#define WKBD_PREVWORKSPACE	31
-#define WKBD_NEXTWSLAYER	32
-#define WKBD_PREVWSLAYER	33
+#define WKBD_WORKSPACE1		21
+#define WKBD_WORKSPACE2		22
+#define WKBD_WORKSPACE3		23
+#define WKBD_WORKSPACE4		24
+#define WKBD_WORKSPACE5		25
+#define WKBD_WORKSPACE6		26
+#define WKBD_WORKSPACE7		27
+#define WKBD_WORKSPACE8		28
+#define WKBD_WORKSPACE9		29
+#define WKBD_WORKSPACE10	30
+#define WKBD_NEXTWORKSPACE	31
+#define WKBD_PREVWORKSPACE	32
+#define WKBD_NEXTWSLAYER	33
+#define WKBD_PREVWSLAYER	34
 
 /* window shortcuts */
-#define WKBD_WINDOW1		34
-#define WKBD_WINDOW2		35
-#define WKBD_WINDOW3		36
-#define WKBD_WINDOW4		37
+#define WKBD_WINDOW1		35
+#define WKBD_WINDOW2		36
+#define WKBD_WINDOW3		37
+#define WKBD_WINDOW4		38
+#define WKBD_WINDOW5		39
+#define WKBD_WINDOW6		40
+#define WKBD_WINDOW7		41
+#define WKBD_WINDOW8		42
+#define WKBD_WINDOW9		43
+#define WKBD_WINDOW10		44
+
+#define WKBD_SWITCH_SCREEN      45
 
 #ifdef KEEP_XKB_LOCK_STATUS
-# define WKBD_TOGGLE             38
-# define WKBD_LAST               39
+# define WKBD_TOGGLE             46
+# define WKBD_LAST               47
 #else
-# define WKBD_LAST               38
+# define WKBD_LAST               46
 #endif /* KEEP_XKB_LOCK_STATUS */
 
 
-
 typedef struct WShortKey {
     unsigned int modifier;
     KeyCode keycode;
