@@ -26,23 +26,29 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <limits.h>
 
 #include "WindowMaker.h"
 #include "wcore.h"
 #include "framewin.h"
 #include "window.h"
 #include "icon.h"
+#include "appicon.h"
 #include "actions.h"
 #include "funcs.h"
 #include "application.h"
 #include "appicon.h"
 #include "dock.h"
 
+
 extern WPreferences wPreferences;
 
 
-#define X_ORIGIN     wPreferences.window_place_origin.x
-#define Y_ORIGIN     wPreferences.window_place_origin.y
+#define X_ORIGIN(scr) WMAX((scr)->totalUsableArea.x1,\
+				wPreferences.window_place_origin.x)
+
+#define Y_ORIGIN(scr) WMAX((scr)->totalUsableArea.y1,\
+				wPreferences.window_place_origin.y)
 
 
 /*
@@ -67,22 +73,33 @@ iconPosition(WCoreWindow *wcore, int sx1, int sy1, int sx2, int sy2,
     parent = wcore->descriptor.parent;
 
     /* if it is an application icon */
-    if (wcore->descriptor.parent_type == WCLASS_APPICON) {
+    if (wcore->descriptor.parent_type == WCLASS_APPICON
+	&& !((WAppIcon*)parent)->docked) {
 	*retX = ((WAppIcon*)parent)->x_pos;
 	*retY = ((WAppIcon*)parent)->y_pos;
 
 	ok = 1;
     } else if (wcore->descriptor.parent_type == WCLASS_MINIWINDOW &&
-	       (((WIcon*)parent)->owner->frame->workspace==workspace
-		|| ((WIcon*)parent)->owner->window_flags.omnipresent
+	       (((WIcon*)parent)->owner->frame->workspace == workspace
+		|| IS_OMNIPRESENT(((WIcon*)parent)->owner)
 		|| wPreferences.sticky_icons)
-	       && !((WIcon*)parent)->owner->flags.hidden) {
+	       && ((WIcon*)parent)->mapped) {
 
 	*retX = ((WIcon*)parent)->owner->icon_x;
 	*retY = ((WIcon*)parent)->owner->icon_y;
 
 	ok = 1;
+    } else if (wcore->descriptor.parent_type == WCLASS_WINDOW
+	       && ((WWindow*)parent)->flags.icon_moved
+	       && (((WWindow*)parent)->frame->workspace == workspace
+		   || IS_OMNIPRESENT((WWindow*)parent)
+		   || wPreferences.sticky_icons)) {
+	*retX = ((WWindow*)parent)->icon_x;
+	*retY = ((WWindow*)parent)->icon_y;
+
+	ok = 1;	
     }
+		
 
     /*
      * Check if it is inside the screen.
@@ -122,7 +139,7 @@ PlaceIcon(WScreen *scr, int *x_ret, int *y_ret)
     int x, y;
     int isize = wPreferences.icon_size;
     int done = 0;
-    int level;
+    WMBagIterator iter;
 
     /*
      * Find out screen boundaries.
@@ -179,9 +196,8 @@ PlaceIcon(WScreen *scr, int *x_ret, int *y_ret)
 
 #define INDEX(x,y)	(((y)+1)*(sw+2) + (x) + 1)
 
-    for (level = WMNormalLevel; level >= WMDesktopLevel; level--) {
-	obj = scr->stacking_list[level];
-	
+    WM_ETARETI_BAG(scr->stacking_list, obj, iter) {
+
 	while (obj) {
 	    int x, y;
 
@@ -231,90 +247,265 @@ PlaceIcon(WScreen *scr, int *x_ret, int *y_ret)
 	    break;
     }
 
-    free(map);
+    wfree(map);
 }
 
 
+/*
+ * This function calculates the length of the intersection of two
+ * line sections. (Hey, is that english?)
+ */
+static int
+calcIntersectionLength(int p1, int l1, int p2, int l2)
+{
+    int isect;
+    int tmp;
+    
+    if (p1 > p2) {
+        tmp = p1;
+        p1 = p2;
+        p2 = tmp;
+        tmp = l1;
+        l1 = l2;
+        l2 = tmp;
+    }
+
+    if (p1 + l1 < p2)
+        isect = 0;
+    else if (p2 + l2 < p1 + l1)
+        isect = l2;
+    else
+        isect = p1 + l1 - p2;
+        
+    return isect;
+}
+
+
+/*
+ * This function calculates the area of the intersection of two rectangles.
+ */
+static int
+calcIntersectionArea(int x1, int y1, int w1, int h1,
+                                int x2, int y2, int w2, int h2)
+{
+    return calcIntersectionLength(x1, w1, x2, w2)
+           * calcIntersectionLength(y1, h1, y2, h2);
+}
 
 
 static int
+calcSumOfCoveredAreas(WWindow *wwin, int x, int y, int w, int h)
+{
+    int sum_isect = 0;
+    WWindow *test_window;
+    int tw,tx,ty,th;
+       
+    test_window = wwin->screen_ptr->focused_window;
+    for(;test_window != NULL && test_window->prev != NULL;)
+         test_window = test_window->prev;
+       
+    for(; test_window != NULL; test_window = test_window->next) {
+        if (test_window->frame->core->stacking->window_level
+            < WMNormalLevel) {
+            continue;
+        }
+
+#if 0
+        tw = test_window->client.width;
+        if (test_window->flags.shaded)
+            th = test_window->frame->top_width;
+        else
+            th = test_window->client.height + extra_height;
+#else
+        tw = test_window->frame->core->width;
+        th = test_window->frame->core->height;
+#endif
+        tx = test_window->frame_x;
+        ty = test_window->frame_y;
+
+        if (test_window->flags.mapped ||
+            (test_window->flags.shaded &&
+             !(test_window->flags.miniaturized ||
+               test_window->flags.hidden))) {
+            sum_isect += calcIntersectionArea(tx, ty, tw, th, x, y, w, h);
+        }
+    }
+    
+    return sum_isect;
+}
+
+
+static void
 smartPlaceWindow(WWindow *wwin, int *x_ret, int *y_ret,
                  unsigned int width, unsigned int height)
 {
     WScreen *scr = wwin->screen_ptr;
-    int test_x = 0, test_y = Y_ORIGIN;
+    int test_x = 0, test_y = Y_ORIGIN(scr);
+    int from_x, to_x, from_y, to_y;
+    int sx;
+    int min_isect, min_isect_x, min_isect_y;
+    int sum_isect;
+    int extra_height;
+    WArea usableArea = scr->totalUsableArea;
+
+    if (wwin->frame)
+	extra_height = wwin->frame->top_width + wwin->frame->bottom_width;
+    else
+        extra_height = 24; /* random value */
+
+    sx = X_ORIGIN(scr);
+
+    min_isect = INT_MAX;
+    min_isect_x = sx;
+    min_isect_y = test_y;
+    
+    height += extra_height;
+
+    while (((test_y + height) < usableArea.y2)) {
+
+	test_x = sx;
+
+	while ((test_x + width) < usableArea.x2) {
+
+            sum_isect = calcSumOfCoveredAreas(wwin, test_x, test_y,
+                                              width, height);
+
+	    if ( sum_isect < min_isect ) {
+	        min_isect = sum_isect;
+	        min_isect_x = test_x;
+	        min_isect_y = test_y;
+	    }
+	    
+	    test_x += PLACETEST_HSTEP;
+	}
+	test_y += PLACETEST_VSTEP;
+    }
+
+    from_x = min_isect_x - PLACETEST_HSTEP + 1;
+    from_x = WMAX(from_x, X_ORIGIN(scr));
+    to_x = min_isect_x + PLACETEST_HSTEP;
+    if (to_x + width > usableArea.x2)
+        to_x = usableArea.x2 - width;
+
+    from_y = min_isect_y - PLACETEST_VSTEP + 1;
+    from_y = WMAX(from_y, Y_ORIGIN(scr));
+    to_y = min_isect_y + PLACETEST_VSTEP;
+    if (to_y + height > usableArea.y2)
+        to_y = usableArea.y2 - height;
+
+    for (test_x = from_x; test_x < to_x; test_x++) {
+        for (test_y = from_y; test_y < to_y; test_y++) {
+            sum_isect = calcSumOfCoveredAreas(wwin, test_x, test_y,
+                                              width, height);
+
+	    if ( sum_isect < min_isect ) {
+	        min_isect = sum_isect;
+	        min_isect_x = test_x;
+	        min_isect_y = test_y;
+	    }
+        }
+    }
+
+    *x_ret = min_isect_x;
+    *y_ret = min_isect_y;
+}
+
+
+static Bool
+autoPlaceWindow(WWindow *wwin, int *x_ret, int *y_ret,
+                unsigned int width, unsigned int height, int tryCount)
+{
+    WScreen *scr = wwin->screen_ptr;
+    int test_x = 0, test_y = Y_ORIGIN(scr);
     int loc_ok = False, tw,tx,ty,th;
     int swidth, sx;
     WWindow *test_window;
     int extra_height;
-
+    WArea usableArea = scr->totalUsableArea;
+    
     if (wwin->frame)
 	extra_height = wwin->frame->top_width + wwin->frame->bottom_width + 2;
     else
 	extra_height = 24; /* random value */
-    
-    swidth = scr->scr_width;
-    sx = X_ORIGIN;
-    if (scr->dock && !scr->dock->lowered) {
-        if (scr->dock->on_right_side)
-            swidth -= wPreferences.icon_size + DOCK_EXTRA_SPACE;
-        else if (X_ORIGIN < wPreferences.icon_size + DOCK_EXTRA_SPACE)
-            sx += wPreferences.icon_size + DOCK_EXTRA_SPACE - X_ORIGIN;
-    }
+
+    swidth = usableArea.x2-usableArea.x1;
+    sx = X_ORIGIN(scr);
 
     /* this was based on fvwm2's smart placement */
 
     height += extra_height;
 
     while (((test_y + height) < (scr->scr_height)) && (!loc_ok)) {
-	
+
 	test_x = sx;
-	
+
 	while (((test_x + width) < swidth) && (!loc_ok)) {
-	    
+
 	    loc_ok = True;
 	    test_window = scr->focused_window;
-            
-	    while ((test_window != (WWindow *) 0) && (loc_ok == True)) {
-		
+
+	    while ((test_window != NULL) && (loc_ok == True)) {
+
+		if (test_window->frame->core->stacking->window_level
+		    < WMNormalLevel && tryCount > 0) {
+		    test_window = test_window->next;
+		    continue;
+		}
+#if 0
                 tw = test_window->client.width;
                 if (test_window->flags.shaded)
                     th = test_window->frame->top_width;
                 else
-                    th = test_window->client.height+extra_height;
+                    th = test_window->client.height + extra_height;
+#else
+		tw = test_window->frame->core->width;
+		th = test_window->frame->core->height;
+#endif
 		tx = test_window->frame_x;
 		ty = test_window->frame_y;
-                
-		if((tx<(test_x+width))&&((tx + tw) > test_x)&&
-		   (ty<(test_y+height))&&((ty + th)>test_y)&&
-                   (test_window->flags.mapped ||
-                    (test_window->flags.shaded &&
-                     !(test_window->flags.miniaturized ||
-                       test_window->flags.hidden)))) {
+
+		if ((tx < (test_x + width)) && ((tx + tw) > test_x) &&
+		    (ty < (test_y + height)) && ((ty + th) > test_y) &&
+		    (test_window->flags.mapped ||
+                     (test_window->flags.shaded &&
+                      test_window->frame->workspace==scr->current_workspace &&
+		      !(test_window->flags.miniaturized ||
+			test_window->flags.hidden)))) {
 
                     loc_ok = False;
 		}
 		test_window = test_window->next;
 	    }
-	    
+
 	    test_window = scr->focused_window;
-	    
-	    while ((test_window != (WWindow *) 0) && (loc_ok == True))  {
-		
+
+	    while ((test_window != NULL) && (loc_ok == True))  {
+
+		if (test_window->frame->core->stacking->window_level
+		    < WMNormalLevel && tryCount > 0) {
+		    test_window = test_window->prev;
+		    continue;
+		}
+#if 0
                 tw = test_window->client.width;
                 if (test_window->flags.shaded)
                     th = test_window->frame->top_width;
                 else
-                    th = test_window->client.height+extra_height;
+                    th = test_window->client.height + extra_height;
+#else
+		tw = test_window->frame->core->width;
+		th = test_window->frame->core->height;
+#endif
 		tx = test_window->frame_x;
 		ty = test_window->frame_y;
-		
-		if((tx<(test_x+width))&&((tx + tw) > test_x)&&
-		   (ty<(test_y+height))&&((ty + th)>test_y)&&
-                   (test_window->flags.mapped ||
-                    (test_window->flags.shaded &&
-                     !(test_window->flags.miniaturized ||
-                       test_window->flags.hidden)))) {
+
+		if ((tx < (test_x + width)) && ((tx + tw) > test_x) &&
+		    (ty < (test_y + height)) && ((ty + th) > test_y) &&
+		    (test_window->flags.mapped ||
+		     (test_window->flags.shaded &&
+                      test_window->frame->workspace==scr->current_workspace &&
+		      !(test_window->flags.miniaturized ||
+			test_window->flags.hidden)))) {
 
                     loc_ok = False;
 		}
@@ -329,6 +520,7 @@ smartPlaceWindow(WWindow *wwin, int *x_ret, int *y_ret,
 	}
 	test_y += PLACETEST_VSTEP;
     }
+
     return loc_ok;
 }
 
@@ -338,31 +530,21 @@ cascadeWindow(WScreen *scr, WWindow *wwin, int *x_ret, int *y_ret,
               unsigned int width, unsigned int height, int h)
 {
     unsigned int extra_height;
-    unsigned int scr_width;
-    int xoffset = 0;
-
-    scr_width = scr->scr_width;
-    if (scr->dock && !scr->dock->lowered) {
-	if (scr->dock->on_right_side) {
-	    scr_width -= wPreferences.icon_size;
-	} else {
-	    xoffset = wPreferences.icon_size;
-	}
-    }
+    WArea usableArea = scr->totalUsableArea;
 
     if (wwin->frame)
 	extra_height = wwin->frame->top_width + wwin->frame->bottom_width;
     else
 	extra_height = 24; /* random value */
     
-    *x_ret = h * scr->cascade_index + X_ORIGIN;
-    *y_ret = h * scr->cascade_index + Y_ORIGIN;
+    *x_ret = h * scr->cascade_index + X_ORIGIN(scr);
+    *y_ret = h * scr->cascade_index + Y_ORIGIN(scr);
     height += extra_height;
-    
-    if (width + *x_ret > scr_width || height + *y_ret > scr->scr_height) {
+
+    if (width + *x_ret > usableArea.x2 || height + *y_ret > usableArea.y2) {
 	scr->cascade_index = 0;
-	*x_ret = h*scr->cascade_index + X_ORIGIN;
-	*y_ret = h*scr->cascade_index + Y_ORIGIN;
+	*x_ret = X_ORIGIN(scr);
+	*y_ret = Y_ORIGIN(scr);
     }
 }
 
@@ -372,7 +554,7 @@ PlaceWindow(WWindow *wwin, int *x_ret, int *y_ret,
             unsigned width, unsigned height)
 {
     WScreen *scr = wwin->screen_ptr;
-    int h = scr->title_font->height+TITLEBAR_EXTRA_HEIGHT;
+    int h = WMFontHeight(scr->title_font) + (wPreferences.window_title_clearance + TITLEBAR_EXTEND_SPACE) * 2;
 
     switch (wPreferences.window_placement) {
      case WPM_MANUAL:
@@ -380,59 +562,45 @@ PlaceWindow(WWindow *wwin, int *x_ret, int *y_ret,
 	break;
 
      case WPM_SMART:
-	if (smartPlaceWindow(wwin, x_ret, y_ret, width, height))
-	  break;
+	smartPlaceWindow(wwin, x_ret, y_ret, width, height);
+	break;
+
+     case WPM_AUTO:
+	if (autoPlaceWindow(wwin, x_ret, y_ret, width, height, 0)) {
+	    break;
+	} else if (autoPlaceWindow(wwin, x_ret, y_ret, width, height, 1)) {
+	    break;
+	}
 	/* there isn't a break here, because if we fail, it should fall
 	   through to cascade placement, as people who want tiling want
 	   automagicness aren't going to want to place their window */
 
      case WPM_CASCADE:
-        if (wPreferences.window_placement == WPM_SMART)
+        if (wPreferences.window_placement == WPM_AUTO)
             scr->cascade_index++;
 
 	cascadeWindow(scr, wwin, x_ret, y_ret, width, height, h);
 
         if (wPreferences.window_placement == WPM_CASCADE)
             scr->cascade_index++;
-
-        if (scr->dock && !scr->dock->lowered) {
-            int x2;
-
-            x2 = *x_ret + width;
-            if (scr->dock->on_right_side
-                && x2 > scr->scr_width - wPreferences.icon_size -
-                        DOCK_EXTRA_SPACE)
-                *x_ret = scr->scr_width - width
-                         - wPreferences.icon_size - DOCK_EXTRA_SPACE;
-            else if (!scr->dock->on_right_side &&
-                     X_ORIGIN < wPreferences.icon_size + DOCK_EXTRA_SPACE)
-                *x_ret += wPreferences.icon_size + DOCK_EXTRA_SPACE - X_ORIGIN;
-        }
 	break;
 
      case WPM_RANDOM:
 	{
-	    int w, h;
-	    
-	    w = (scr->scr_width - width);
-	    h = (scr->scr_height - height);
+	    int w, h, extra_height;
+	    WArea usableArea = scr->totalUsableArea;
+
+            if (wwin->frame)
+                extra_height = wwin->frame->top_width + wwin->frame->bottom_width + 2;
+            else
+                extra_height = 24; /* random value */
+
+	    w = ((usableArea.x2-X_ORIGIN(scr)) - width);
+	    h = ((usableArea.y2-Y_ORIGIN(scr)) - height - extra_height);
 	    if (w<1) w = 1;
 	    if (h<1) h = 1;
-	    *x_ret = rand()%w;
-	    *y_ret = rand()%h;
-	    if (scr->dock && !scr->dock->lowered) {
-		int x2;
-		    
-		x2 = *x_ret + width;
-		if (scr->dock->on_right_side 
-                    && x2 > scr->scr_width - wPreferences.icon_size -
-                            DOCK_EXTRA_SPACE)
-                    *x_ret = scr->scr_width - width
-                             - wPreferences.icon_size - DOCK_EXTRA_SPACE;
-		else if (!scr->dock->on_right_side 
-			 && *x_ret < wPreferences.icon_size)
-		    *x_ret = wPreferences.icon_size + DOCK_EXTRA_SPACE;
-	    }
+	    *x_ret = X_ORIGIN(scr) + rand()%w;
+	    *y_ret = Y_ORIGIN(scr) + rand()%h;
 	}
 	break;
 
@@ -444,15 +612,15 @@ PlaceWindow(WWindow *wwin, int *x_ret, int *y_ret,
 #endif
     }
     
-    if (*x_ret < 0)
-	*x_ret = 0;
-    else if (*x_ret + width > scr->scr_width)
+    if (*x_ret + width > scr->scr_width)
 	*x_ret = scr->scr_width - width;
-    
-    if (*y_ret < 0)
-	*y_ret = 0;
-    else if (*y_ret + height > scr->scr_height)
+    if (*x_ret < 0)
+        *x_ret = 0;
+
+    if (*y_ret + height > scr->scr_height)
 	*y_ret = scr->scr_height - height;
+    if (*y_ret < 0)
+        *y_ret = 0;
 }
 
 
