@@ -30,9 +30,6 @@
 #include "WindowMaker.h"
 #include "window.h"
 #include "GNUstep.h"
-#ifdef MWM_HINTS
-#  include "motif.h"
-#endif
 
 
 /* atoms */
@@ -43,7 +40,7 @@ extern Atom _XA_WM_CLIENT_LEADER;
 extern Atom _XA_WM_TAKE_FOCUS;
 extern Atom _XA_WM_DELETE_WINDOW;
 extern Atom _XA_WM_SAVE_YOURSELF;
-#ifdef R6SM
+#ifdef XSMP_ENABLED
 extern Atom _XA_WM_WINDOW_ROLE;
 extern Atom _XA_SM_CLIENT_ID;
 #endif
@@ -55,7 +52,9 @@ extern Atom _XA_GNUSTEP_WM_MINIATURIZE_WINDOW;
 extern Atom _XA_WINDOWMAKER_WM_FUNCTION;
 extern Atom _XA_WINDOWMAKER_MENU;
 extern Atom _XA_WINDOWMAKER_WM_PROTOCOLS;
-
+extern Atom _XA_WINDOWMAKER_NOTICEBOARD;
+extern Atom _XA_WINDOWMAKER_ICON_TILE;
+extern Atom _XA_WINDOWMAKER_ICON_SIZE;
 
 int
 PropGetNormalHints(Window window, XSizeHints *size_hints, int *pre_iccm)
@@ -94,6 +93,7 @@ PropGetWMClass(Window window, char **wm_class, char **wm_instance)
     return True;
 }
 
+
 void
 PropGetProtocols(Window window, WProtocols *prots)
 {
@@ -118,22 +118,52 @@ PropGetProtocols(Window window, WProtocols *prots)
 }
 
 
-int
-PropGetGNUstepWMAttr(Window window, GNUstepWMAttributes **attr)
+unsigned char*
+PropGetCheckProperty(Window window, Atom hint, Atom type, int format,
+		     int count, int *retCount)
 {
     Atom type_ret;
     int fmt_ret;
     unsigned long nitems_ret;
     unsigned long bytes_after_ret;
-    unsigned long *data;
+    unsigned char *data;
+    int tmp;
 
-    if (XGetWindowProperty(dpy, window, _XA_GNUSTEP_WM_ATTR, 0, 9,
-			   False, _XA_GNUSTEP_WM_ATTR,
+    if (count <= 0)
+	tmp = 0xffffff;
+    else
+	tmp = count;
+
+    if (XGetWindowProperty(dpy, window, hint, 0, tmp, False, type,
 			   &type_ret, &fmt_ret, &nitems_ret, &bytes_after_ret,
 			   (unsigned char **)&data)!=Success || !data)
-      return False;
-    if (type_ret!=_XA_GNUSTEP_WM_ATTR || !data || fmt_ret!=32)
-      return False;
+	return NULL;
+
+    if ((type!=AnyPropertyType && type!=type_ret)
+	|| (count > 0 && nitems_ret != count)
+	|| (format != 0 && format != fmt_ret)) {
+	XFree(data);
+	return NULL;
+    }
+
+    if (retCount)
+	*retCount = nitems_ret;
+
+    return data;
+}
+
+
+int
+PropGetGNUstepWMAttr(Window window, GNUstepWMAttributes **attr)
+{
+    unsigned long *data;
+
+    data = (unsigned long*)PropGetCheckProperty(window, _XA_GNUSTEP_WM_ATTR,
+						_XA_GNUSTEP_WM_ATTR, 32, 9,
+						NULL);
+    
+    if (!data)
+	return False;
     
     *attr = malloc(sizeof(GNUstepWMAttributes));
     if (!*attr) {
@@ -156,92 +186,95 @@ PropGetGNUstepWMAttr(Window window, GNUstepWMAttributes **attr)
 }
 
 
- 
- 
-#ifdef MWM_HINTS
-int
-PropGetMotifWMHints(Window window, MWMHints **mwmhints)
-{
-    Atom type_ret;
-    int fmt_ret;
-    unsigned long nitems_ret;
-    unsigned long bytes_after_ret;
-    unsigned long *data;
-    
-    if (XGetWindowProperty(dpy, window, _XA_MOTIF_WM_HINTS, 0,
-			   PROP_MWM_HINTS_ELEMENTS,
-			   False, _XA_MOTIF_WM_HINTS,
-			   &type_ret, &fmt_ret, &nitems_ret, &bytes_after_ret,
-			   (unsigned char **)&data)!=Success)
-      return 0;
-    
-    if (type_ret!=_XA_MOTIF_WM_HINTS || fmt_ret!=32 
-	|| nitems_ret!=PROP_MWM_HINTS_ELEMENTS || !data)
-	return 0;
-    
-    *mwmhints = malloc(sizeof(MWMHints));
-    if (!*mwmhints) {
-	XFree(data);
-	return 0;
-    }
-
-    (*mwmhints)->flags = data[0];
-    (*mwmhints)->functions = data[1];
-    (*mwmhints)->decorations = data[2];
-    (*mwmhints)->inputMode = data[3];
-
-    XFree(data);
-
-    if (type_ret==_XA_MOTIF_WM_HINTS)
-      return 1;
-    else 
-      return 0;
-}
-#endif /* MWM_HINTS */
-
 
 void
 PropSetWMakerProtocols(Window root)
 {
-    Atom protocols[2];
+    Atom protocols[3];
     int count=0;
     
     protocols[count++] = _XA_WINDOWMAKER_MENU;
     protocols[count++] = _XA_WINDOWMAKER_WM_FUNCTION;
+    protocols[count++] = _XA_WINDOWMAKER_NOTICEBOARD;
 
     XChangeProperty(dpy, root, _XA_WINDOWMAKER_WM_PROTOCOLS, XA_ATOM,
 		    32, PropModeReplace,  (unsigned char *)protocols, count);
 }
 
 
+void
+PropSetIconTileHint(WScreen *scr, RImage *image)
+{
+    static Atom imageAtom = 0;
+    unsigned char *tmp;
+    int x, y;
+    
+    if (scr->info_window == None)
+	return;
+    
+    if (!imageAtom) {
+	/*
+	 * WIDTH, HEIGHT (16 bits, MSB First)
+	 * array of R,G,B,A bytes
+	 */
+	imageAtom = XInternAtom(dpy, "_RGBA_IMAGE", False);
+    }
+    
+    tmp = malloc(image->width * image->height * 4 + 4);
+    if (!tmp) {
+	wwarning("could not allocate memory to set _WINDOWMAKER_ICON_TILE hint");
+	return;
+    }
+    
+    tmp[0] = image->width>>8;
+    tmp[1] = image->width&0xff;
+    tmp[2] = image->height>>8;
+    tmp[3] = image->height&0xff;
+    
+    if (image->format == RRGBAFormat) {
+	memcpy(&tmp[4], image->data, image->width*image->height*4);
+    } else {
+	char *ptr = tmp+4;
+	char *src = image->data;
+	
+	for (y = 0; y < image->height; y++) {
+	    for (x = 0; x < image->width; x++) {
+		*ptr++ = *src++;
+		*ptr++ = *src++;
+		*ptr++ = *src++;
+		*ptr++ = 255;
+	    }
+	}
+    }
+    
+    XChangeProperty(dpy, scr->info_window, _XA_WINDOWMAKER_ICON_TILE,
+		    imageAtom, 8, PropModeReplace, tmp, 
+		    image->width * image->height * 4 + 4);
+    wfree(tmp);
+    
+}
+
+
 Window
 PropGetClientLeader(Window window)
 {
-    Atom type_ret;
-    int fmt_ret;
-    unsigned long nitems_ret;
-    unsigned long bytes_after_ret;
     Window *win;
     Window leader;
 
-    if (XGetWindowProperty(dpy, window, _XA_WM_CLIENT_LEADER, 0, 1,
-			   False, AnyPropertyType,
-			   &type_ret, &fmt_ret, &nitems_ret, &bytes_after_ret,
-			   (unsigned char**)&win)!=Success || !win)
-      return None;
+    win = (Window*)PropGetCheckProperty(window, _XA_WM_CLIENT_LEADER,
+					XA_WINDOW, 32, 1, NULL);
+
+    if (!win)
+	return None;
 
     leader = (Window)*win;
     XFree(win);
 
-    if (type_ret == XA_WINDOW && fmt_ret == 32 && nitems_ret == 1 
-	&& bytes_after_ret == 0)
-	return leader;
-    else
-	return None;
+    return leader;
 }
 
 
-#ifdef R6SM
+#ifdef XSMP_ENABLED
 char*
 PropGetClientID(Window window)
 {
@@ -290,7 +323,7 @@ PropGetWindowRole(Window window)
 	return NULL;
     }
 }
-#endif /* R6SM */
+#endif /* XSMP_ENABLED */
 
 
 void
@@ -313,10 +346,37 @@ PropWriteGNUstepWMAttr(Window window, GNUstepWMAttributes *attr)
 }
 
 
+int
+PropGetWindowState(Window window)
+{
+    long *data;
+    long state;
+
+    data = (long*)PropGetCheckProperty(window, _XA_WM_STATE, _XA_WM_STATE,
+				       32, 1, NULL);
+
+    if (!data)
+	return -1;
+
+    state = *data;
+    XFree(data);
+
+    return state;
+}
+
+
 void
 PropCleanUp(Window root)
 {
     XDeleteProperty(dpy, root, _XA_WINDOWMAKER_WM_PROTOCOLS);
-    
+
+    XDeleteProperty(dpy, root, _XA_WINDOWMAKER_NOTICEBOARD);
+
     XDeleteProperty(dpy, root, XA_WM_ICON_SIZE);
+
+#ifdef KWM_HINTS
+    XDeleteProperty(dpy, root, XInternAtom(dpy, "KWM_RUNNING", False));
+#endif
 }
+
+
