@@ -23,6 +23,8 @@
 
 #include "wconfig.h"
 
+#ifndef LITE
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -44,6 +46,10 @@ extern WPreferences wPreferences;
 extern Time LastTimestamp;
 
 
+static void observer(void *self, WMNotification *notif);
+static void wsobserver(void *self, WMNotification *notif);
+
+
 /*
  * FocusWindow
  *
@@ -89,6 +95,22 @@ OpenSwitchMenu(WScreen *scr, int x, int y, int keyboard)
 {
     WMenu *switchmenu = scr->switch_menu;
     WWindow *wwin;
+    static int initialized = 0;
+    
+    if (!initialized) {
+	initialized = 1;
+
+	WMAddNotificationObserver(observer, NULL, WMNManaged, NULL);
+	WMAddNotificationObserver(observer, NULL, WMNUnmanaged, NULL);
+	WMAddNotificationObserver(observer, NULL, WMNChangedWorkspace, NULL);
+	WMAddNotificationObserver(observer, NULL, WMNChangedState, NULL);
+	WMAddNotificationObserver(observer, NULL, WMNChangedFocus, NULL);
+	WMAddNotificationObserver(observer, NULL, WMNChangedStacking, NULL);
+	WMAddNotificationObserver(observer, NULL, WMNChangedName, NULL);
+
+	WMAddNotificationObserver(wsobserver, NULL, WMNWorkspaceChanged, NULL);
+	WMAddNotificationObserver(wsobserver, NULL, WMNWorkspaceNameChanged, NULL);	
+    }
 
     if (switchmenu) {
 	if (switchmenu->flags.mapped) {
@@ -100,12 +122,15 @@ OpenSwitchMenu(WScreen *scr, int x, int y, int keyboard)
 		if (keyboard)
 		    wMenuMapAt(switchmenu, 0, 0, True);
 		else
-		    wMenuMapCopyAt(switchmenu, x-switchmenu->frame->core->width/2,
-			       y-switchmenu->frame->top_width/2);
+		    wMenuMapCopyAt(switchmenu, 
+				   x-switchmenu->frame->core->width/2, y);
 	    }
 	} else {
-	    wMenuMapAt(switchmenu, x-switchmenu->frame->core->width/2,
-		       y-switchmenu->frame->top_width/2, keyboard);
+	    if (keyboard && x==scr->scr_width/2 && y==scr->scr_height/2) {
+		y = y - switchmenu->frame->core->height/2;
+	    }
+	    wMenuMapAt(switchmenu, x-switchmenu->frame->core->width/2, y,
+		       keyboard);
 	}
 	return;
     }
@@ -121,25 +146,65 @@ OpenSwitchMenu(WScreen *scr, int x, int y, int keyboard)
     }
     
     if (switchmenu) {
+        int newx, newy;
+
 	if (!switchmenu->flags.realized)
 	    wMenuRealize(switchmenu);
-	wMenuMapAt(switchmenu, x-switchmenu->frame->core->width/2,
-		   y-switchmenu->frame->top_width/2, keyboard);
+
+        if (keyboard && x==0 && y==0) {
+            newx = newy = 0;
+        } else if (keyboard && x==scr->scr_width/2 && y==scr->scr_height/2) {
+            newx = x - switchmenu->frame->core->width/2;
+            newy = y - switchmenu->frame->core->height/2;
+        } else {
+            newx = x - switchmenu->frame->core->width/2;
+            newy = y;
+        }
+        wMenuMapAt(switchmenu, newx, newy, keyboard);
     }
 }
 
+
+static int
+menuIndexForWindow(WMenu *menu, WWindow *wwin, int old_pos)
+{
+    int idx;
+
+    if (menu->entry_no == 0)
+	return -1;
+
+#define WS(i)  ((WWindow*)menu->entries[i]->clientdata)->frame->workspace
+    if (old_pos >= 0) {
+	if (WS(old_pos) >= wwin->frame->workspace
+	    && (old_pos == 0 || WS(old_pos-1) <= wwin->frame->workspace)) {
+	    return old_pos;
+	}
+    }
+#undef WS
+
+    for (idx = 0; idx < menu->entry_no; idx++) {
+	WWindow *tw = (WWindow*)menu->entries[idx]->clientdata;
+
+	if (!IS_OMNIPRESENT(tw) 
+	    && tw->frame->workspace > wwin->frame->workspace) {
+	    break;
+	}
+    }
+
+    return idx;
+}
+
+
 /*
- *
  * Update switch menu
- *
- *
  */
-void 
+void
 UpdateSwitchMenu(WScreen *scr, WWindow *wwin, int action)
 {
     WMenu *switchmenu = scr->switch_menu;
     WMenuEntry *entry;
     char title[MAX_MENU_TEXT_LENGTH+6];
+    int len = sizeof(title);
     int i;
     int checkVisibility = 0;
 
@@ -152,29 +217,38 @@ UpdateSwitchMenu(WScreen *scr, WWindow *wwin, int action)
      *    2.  When a window is destroyed.
      *
      *    3.  When a window changes it's title.
+     * 	  4.  When a window changes its workspace.
      */
     if (action == ACTION_ADD) {
 	char *t;
-	if (wwin->flags.internal_window 
-	    || wwin->window_flags.skip_window_list)
-	  return;
-	    
+	int idx;
+
+	if (wwin->flags.internal_window || WFLAGP(wwin, skip_window_list))
+	    return;
+	
 	if (wwin->frame->title)
-	  sprintf(title, "%s", wwin->frame->title);
+	  snprintf(title, len, "%s", wwin->frame->title);
 	else
-	  sprintf(title, "%s", DEF_WINDOW_TITLE);
+	  snprintf(title, len, "%s", DEF_WINDOW_TITLE);
 	t = ShrinkString(scr->menu_entry_font, title, MAX_WINDOWLIST_WIDTH);
-	entry = wMenuAddCallback(switchmenu, t, focusWindow, wwin);
-	free(t);
+
+	if (IS_OMNIPRESENT(wwin))
+	    idx = -1;
+	else {
+	    idx = menuIndexForWindow(switchmenu, wwin, -1);
+	}
+	
+	entry = wMenuInsertCallback(switchmenu, idx, t, focusWindow, wwin);
+	wfree(t);
 
 	entry->flags.indicator = 1;
 	entry->rtext = wmalloc(MAX_WORKSPACENAME_WIDTH+8);
-	if (wwin->window_flags.omnipresent)
-	    sprintf(entry->rtext, "[*]");
+	if (IS_OMNIPRESENT(wwin))
+	    snprintf(entry->rtext, MAX_WORKSPACENAME_WIDTH, "[*]");
 	else
-	    sprintf(entry->rtext, "[%s]", 
+	    snprintf(entry->rtext, MAX_WORKSPACENAME_WIDTH, "[%s]", 
 		    scr->workspaces[wwin->frame->workspace]->name);
-	
+
 	if (wwin->flags.hidden) {
 	    entry->flags.indicator_type = MI_HIDDEN;
 	    entry->flags.indicator_on = 1;
@@ -206,24 +280,57 @@ UpdateSwitchMenu(WScreen *scr, WWindow *wwin, int action)
 		    
 		 case ACTION_CHANGE:
 		    if (entry->text)
-		      free(entry->text);
+		      wfree(entry->text);
 		    
-		    if (wwin->frame->title)
-		      sprintf(title, "%s", wwin->frame->title);
-		    else
-		      sprintf(title, "%s", DEF_WINDOW_TITLE);
+                    if (wwin->frame->title)
+                        snprintf(title, MAX_MENU_TEXT_LENGTH, "%s",
+                                 wwin->frame->title);
+                    else
+                        snprintf(title, MAX_MENU_TEXT_LENGTH, "%s",
+				 DEF_WINDOW_TITLE);
 
-		    t = ShrinkString(scr->menu_entry_font, title, 
-				     MAX_WINDOWLIST_WIDTH);
+		    t = ShrinkString(scr->menu_entry_font, title, MAX_WINDOWLIST_WIDTH);
 		    entry->text = t;
-		    /* fall through to update workspace number */
+
+		    wMenuRealize(switchmenu);
+		    checkVisibility = 1;
+		    break;
+
 		 case ACTION_CHANGE_WORKSPACE:
 		    if (entry->rtext) {
-			if (wwin->window_flags.omnipresent)
-			    sprintf(entry->rtext, "[*]");
-			else
-			    sprintf(entry->rtext, "[%s]", 
-				    scr->workspaces[wwin->frame->workspace]->name);
+			int idx = -1;
+			char *t, *rt;
+			int it, ion;
+
+			if (IS_OMNIPRESENT(wwin)) {
+			    snprintf(entry->rtext, MAX_WORKSPACENAME_WIDTH,
+				     "[*]");
+			} else {
+			    snprintf(entry->rtext, MAX_WORKSPACENAME_WIDTH,
+				     "[%s]", scr->workspaces[wwin->frame->workspace]->name);
+			}
+
+			rt = entry->rtext;
+			entry->rtext = NULL;
+			t = entry->text;
+			entry->text = NULL;
+			
+			it = entry->flags.indicator_type;
+			ion = entry->flags.indicator_on;
+
+			wMenuRemoveItem(switchmenu, i);
+
+			if (!IS_OMNIPRESENT(wwin) && idx < 0) {
+			    idx = menuIndexForWindow(switchmenu, wwin, i);
+			}
+
+			entry = wMenuInsertCallback(switchmenu, idx, t,
+						    focusWindow, wwin);
+			wfree(t);
+			entry->rtext = rt;
+			entry->flags.indicator = 1;
+			entry->flags.indicator_type = it;
+			entry->flags.indicator_on = ion;
 		    }
 		    wMenuRealize(switchmenu);
 		    checkVisibility = 1;
@@ -244,7 +351,6 @@ UpdateSwitchMenu(WScreen *scr, WWindow *wwin, int action)
 			entry->flags.indicator_on = wwin->flags.focused;
 			entry->flags.indicator_type = MI_DIAMOND;
 		    }
-		    wMenuPaint(switchmenu);
 		    break;
 		}
 		break;
@@ -280,11 +386,11 @@ UpdateSwitchMenuWorkspace(WScreen *scr, int workspace)
 	wwin = (WWindow*)menu->entries[i]->clientdata;
 
 	if (wwin->frame->workspace==workspace 
-	    && !wwin->window_flags.omnipresent) {
-	    if (wwin->window_flags.omnipresent)
-		sprintf(menu->entries[i]->rtext, "[*]");
+	    && !IS_OMNIPRESENT(wwin)) {
+	    if (IS_OMNIPRESENT(wwin))
+		snprintf(menu->entries[i]->rtext, MAX_WORKSPACENAME_WIDTH,"[*]");
 	    else
-		sprintf(menu->entries[i]->rtext, "[%s]", 
+		snprintf(menu->entries[i]->rtext, MAX_WORKSPACENAME_WIDTH,"[%s]",
 			scr->workspaces[wwin->frame->workspace]->name);
 	    menu->flags.realized = 0;
 	}
@@ -292,3 +398,47 @@ UpdateSwitchMenuWorkspace(WScreen *scr, int workspace)
     if (!menu->flags.realized)
 	wMenuRealize(menu);
 }
+
+
+static void observer(void *self, WMNotification *notif)
+{
+    WWindow *wwin = (WWindow*)WMGetNotificationObject(notif);
+    const char *name = WMGetNotificationName(notif);
+    void *data = WMGetNotificationClientData(notif);
+
+    if (strcmp(name, WMNManaged) == 0 && wwin)
+	UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_ADD);
+    else if (strcmp(name, WMNUnmanaged) == 0 && wwin)
+	UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_REMOVE);
+    else if (strcmp(name, WMNChangedWorkspace) == 0 && wwin)
+	UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_CHANGE_WORKSPACE);
+    else if (strcmp(name, WMNChangedFocus) == 0 && wwin)
+	UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_CHANGE_STATE);
+    else if (strcmp(name, WMNChangedName) == 0 && wwin)
+	UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_CHANGE);
+    else if (strcmp(name, WMNChangedState) == 0 && wwin) {
+
+	if (strcmp((char*)data, "omnipresent") == 0) {
+	    UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_CHANGE_WORKSPACE);
+	} else {
+	    UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_CHANGE_STATE);
+	}
+    }
+}
+
+
+static void wsobserver(void *self, WMNotification *notif)
+{
+    WScreen *scr = (WScreen*)WMGetNotificationObject(notif);
+    const char *name = WMGetNotificationName(notif);
+    void *data = WMGetNotificationClientData(notif);
+
+    if (strcmp(name, WMNWorkspaceNameChanged) == 0) {
+	UpdateSwitchMenuWorkspace(scr, (int)data);
+    } else if (strcmp(name, WMNWorkspaceChanged) == 0) {
+	
+    }
+}
+
+
+#endif /* !LITE */
