@@ -1,7 +1,7 @@
 /*
  *  Window Maker window manager
  * 
- *  Copyright (c) 1997, 1998 Alfredo K. Kojima
+ *  Copyright (c) 1997-2000 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -43,21 +43,18 @@
 #define WTEX_MHGRADIENT	((1<<5)|WREL_BORDER_MASK)
 #define WTEX_MVGRADIENT	((1<<6)|WREL_BORDER_MASK)
 #define WTEX_MDGRADIENT	((1<<7)|WREL_BORDER_MASK)
-#define WTEX_PIXMAP	(1<<8)
+#define WTEX_IGRADIENT	((1<<8)|WREL_BORDER_MASK)
+#define WTEX_PIXMAP	(1<<10)
+#define WTEX_THGRADIENT	((1<<11)|WREL_BORDER_MASK)
+#define WTEX_TVGRADIENT	((1<<12)|WREL_BORDER_MASK)
+#define WTEX_TDGRADIENT	((1<<13)|WREL_BORDER_MASK)
+#define WTEX_FUNCTION	((1<<14)|WREL_BORDER_MASK)
 
 /* pixmap subtypes */
 #define WTP_TILE	2
 #define WTP_SCALE	4
 #define WTP_CENTER	6
 
-/*
- * (solid <color>)
- * (hgradient <color> <color>)
- * (vgradient <color> <color>)
- * (dgradient <color> <color>)
- * (pixmap <file> <mode>)
- */
-
 
 typedef struct {
     short type;			       /* type of texture */
@@ -89,8 +86,8 @@ typedef struct WTexGradient {
     XColor normal;
     GC normal_gc;
 
-    XColor color1;
-    XColor color2;
+    RColor color1;
+    RColor color2;
 } WTexGradient;
 
 
@@ -104,6 +101,19 @@ typedef struct WTexMGradient {
 } WTexMGradient;
 
 
+typedef struct WTexIGradient {
+    short type;
+    char dummy;
+    XColor normal;
+    GC normal_gc;
+
+    RColor colors1[2];
+    RColor colors2[2];
+    int thickness1;
+    int thickness2;
+} WTexIGradient;
+
+
 typedef struct WTexPixmap {
     short type;
     char subtype;
@@ -113,32 +123,52 @@ typedef struct WTexPixmap {
     struct RImage *pixmap;
 } WTexPixmap;
 
-typedef struct WTexCompose {
+typedef struct WTexTGradient {
     short type;
     char subtype;
     XColor normal;
     GC normal_gc;
 
-    union WTexture *back;
-    union WTexture *fore;
+    RColor color1;
+    RColor color2;
+    struct RImage *pixmap;
     int opacity;
-} WTexCompose;
+} WTexTGradient;
+
+typedef struct WTexFunction {
+    short type;
+    char subtype;
+    XColor normal;
+    GC normal_gc;
+
+    void *handle;
+    RImage *(*render) (int, char**, int, int, int);
+    int argc;
+    char **argv;
+} WTexFunction;
 
 typedef union WTexture {
     WTexAny any;    
     WTexSolid solid;
     WTexGradient gradient;
+    WTexIGradient igradient;
     WTexMGradient mgradient;
     WTexPixmap pixmap;
-    WTexCompose compose;
+    WTexTGradient tgradient;
+    WTexFunction function;
 } WTexture;
 
 
 WTexSolid *wTextureMakeSolid(WScreen*, XColor*);
-WTexGradient *wTextureMakeGradient(WScreen*, int, XColor*, XColor*);
+WTexGradient *wTextureMakeGradient(WScreen*, int, RColor*, RColor*);
 WTexMGradient *wTextureMakeMGradient(WScreen*, int, RColor**);
+WTexTGradient *wTextureMakeTGradient(WScreen*, int, RColor*, RColor*, char *, int);
+WTexIGradient *wTextureMakeIGradient(WScreen*, int, RColor[], int, RColor[]);
 WTexPixmap *wTextureMakePixmap(WScreen *scr, int style, char *pixmap_file, 
 			       XColor *color);
+#ifdef TEXTURE_PLUGIN
+WTexFunction *wTextureMakeFunction(WScreen*, char *, char *, int, char **);
+#endif
 void wTextureDestroy(WScreen*, WTexture*);
 void wTexturePaint(WTexture *, Pixmap *, WCoreWindow*, int, int);
 void wTextureRender(WScreen*, WTexture*, Pixmap*, int, int, int);
