@@ -3,7 +3,7 @@
  * 
  *  WindowMaker window manager
  * 
- *  Copyright (c) 1997 Alfredo K. Kojima
+ *  Copyright (c) 1997-1999 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
 
 #include "config.h"
 
-/*** Change it (wconfig.h) *after* you ran configure ***/
+/*** Change this file (wconfig.h) *after* you ran configure ***/
 
 /*
  *--------------------------------------------------------------------
@@ -37,6 +37,18 @@
  *--------------------------------------------------------------------
  */
 
+
+/*
+ * #undefine if you dont want texture plugin support or your system have
+ * some sort of problem with them.
+ */
+#define TEXTURE_PLUGIN
+
+#ifdef TEXTURE_PLUGIN
+#undef DRAWSTRING_PLUGIN
+#endif
+
+
 /* undefine ANIMATIONS if you don't want animations for iconification,
  * shading, icon arrangement etc. */
 #define ANIMATIONS
@@ -48,21 +60,7 @@
  */
 #define USECPP
 
-/* #define CPP_PATH /lib/cpp */
-
-/*
- * Internationalization (I18N) support
- * Multi-byte (japanese, korean, chinese etc.) character support
- * set by configure
- */
-#undef I18N_MB
-
-
-/*
- * sound support
- */
-#define WMSOUND
-
+/* #define CPP_PATH /usr/bin/cpp */
 
 /*
  * support for OffiX DND drag and drop in the Dock
@@ -70,9 +68,16 @@
 #define OFFIX_DND
 
 /*
- * support for XDE drang and drop in the Dock. still in beta
+ * support for XDND drop in the Dock. Experimental
  */
-#undef XDE_DND
+#ifdef GNOME_STUFF
+#define XDND
+#endif
+
+/*
+ * support for Motif window manager (mwm) window hints
+ */
+#define MWM_HINTS
 
 
 /*
@@ -101,16 +106,6 @@
  */
 #define FALLBACK_WINDOWMANAGER "blackbox"
 
-
-/* Define if you want MWM hint support (and consequently GNOME). */
-#define MWM_HINTS
-
-
-/* Define if you have a 5 button mouse and want to use button 4
- * (in the root window) for switching to the previous workspace
- * and 5 for the next */
-#undef MOUSE_WS_SWITCH
- 
 /*
  * Turn on a hack to make mouse and keyboard actions work even if
  * the NumLock or ScrollLock modifiers are turned on. They might
@@ -121,11 +116,6 @@
  */
 #define NUMLOCK_HACK
 
-/*
- * define REDUCE_APPICONS if you want apps with the same WM_INSTANCE &&
- * WM_CLASS to share an appicon
- */
-#undef REDUCE_APPICONS
 
 
 /*
@@ -145,6 +135,43 @@
 #undef CONFIGURE_WINDOW_WHILE_MOVING
 
 
+/*
+ * disable/enable workspace indicator in the dock
+ */
+#undef WS_INDICATOR 
+
+
+/*
+ * define HIDDENDOT if you want a dot to be shown in the application icon
+ * of applications that are hidden.
+ */
+#define HIDDENDOT
+
+
+/*
+ * Ignores the PPosition hint from clients. This is needed for some
+ * programs that have buggy implementations of such hint and place
+ * themselves in strange locations.
+ */
+#undef IGNORE_PPOSITION
+
+
+/*
+ * Do not scale application icon and miniwindow icon images.
+ */
+#undef DONT_SCALE_ICONS
+
+
+
+
+
+
+#define SILLYNESS
+
+
+
+
+
 /*
  *..........................................................................
  * The following options WILL NOT BE MADE RUN-TIME. Please do not request.
@@ -154,15 +181,16 @@
 
 /*
  * define SHADOW_RESIZEBAR if you want a resizebar with shadows like in
- * NextStep 3.x ??, instead of the default Openstep look. NEXTSTEP 3.3
- * also does not have these shadows.
+ * AfterStep, instead of the default Openstep look.
+ * NEXTSTEP 3.3 also does not have these shadows.
  */
 #undef SHADOW_RESIZEBAR
 
-
 /*
  * Define DEMATERIALIZE_ICON if you want the undocked icon animation
- * to be a progressive disaparison animation. 
+ * to be a progressive disaparison animation.
+ * This will cause all application icons to be created with Save Under
+ * enable.
  */
 #undef DEMATERIALIZE_ICON
 
@@ -173,11 +201,18 @@
 #undef ICON_KABOOM_EXTRA
 
 /*
- * #undef if you dont want the window creation animation when superfluous
- * is enabled.
+ * #define if you want the window creation animation when superfluous
+ * is enabled. Only enable one of them.
  */
 #undef WINDOW_BIRTH_ZOOM
 
+#undef WINDOW_BIRTH_ZOOM2
+
+/*
+ * whether arrow drawing in clip buttons should be gradiented
+ */
+#undef GRADIENT_CLIP_ARROWS
+
 
 /*
  *--------------------------------------------------------------------
@@ -205,11 +240,28 @@
 
 #define DEFAULTS_DIR "Defaults"
 
+#ifdef USE_TIFF
+#define DEF_BUTTON_IMAGES PKGDATADIR"/buttons.tiff"
+#else
+#define DEF_BUTTON_IMAGES PKGDATADIR"/buttons.xpm"
+#endif
+
+/* the file of the system wide submenu to be forced into the main menu */
+/* remove the comments if you want it */
+/*
+#define GLOBAL_SUBMENU_FILE PKGDATADIR"/GlobalMenu"
+*/
 
 /* pixmap path */
 #define DEF_PIXMAP_PATHS \
 "(\"~/pixmaps\",\"~/GNUstep/Library/WindowMaker/Pixmaps\",\""PIXMAPDIR"\")"
 
+#ifdef USER_MENU
+#define GLOBAL_USER_MENU_PATH PKGDATADIR"/UserMenus"
+#define DEF_USER_MENU_PATHS \
+"~/GNUstep/Library/WindowMaker/UserMenus:"GLOBAL_USER_MENU_PATH
+#endif
+
 /* icon path */
 #define DEF_ICON_PATHS \
 "(\"~/pixmaps\",\"~/GNUstep/Library/Icons\",\"/usr/include/X11/pixmaps/\",\""PIXMAPDIR"\")"
@@ -222,32 +274,44 @@
 #define DEF_FRAME_COLOR   "white"
 
 
-#ifdef I18N_MB
+#ifdef I18N
 #define DEF_TITLE_FONT "\"-*-*-medium-r-normal--14-*\""
 #define DEF_MENU_TITLE_FONT "\"-*-*-medium-r-normal--14-*\""
 #define DEF_MENU_ENTRY_FONT "\"-*-*-medium-r-normal--14-*\""
 #define DEF_ICON_TITLE_FONT "\"-*-*-medium-r-normal--10-*\""
 #define DEF_CLIP_TITLE_FONT "\"-*-*-medium-r-normal--10-*\""
 #define DEF_INFO_TEXT_FONT "\"-*-*-medium-r-normal--14-*\""
-#else /* I18N_MB */
+#define DEF_ADDITONAL_FONT "-*-*-medium-r-normal--12-*"
+
+#define DEF_WORKSPACE_NAME_FONT "\"-*-*-medium-r-normal--24-*\""
+#else /* !I18N */
 #define DEF_TITLE_FONT "\"-*-helvetica-bold-r-normal-*-12-*-*-*-*-*-*-*\""
 #define DEF_MENU_TITLE_FONT "\"-*-helvetica-bold-r-normal-*-12-*-*-*-*-*-*-*\""
 #define DEF_MENU_ENTRY_FONT "\"-*-helvetica-medium-r-normal-*-12-*-*-*-*-*-*-*\""
 #define DEF_ICON_TITLE_FONT "\"-*-helvetica-medium-r-normal-*-8-*-*-*-*-*-*-*\""
 #define DEF_CLIP_TITLE_FONT "\"-*-helvetica-bold-r-normal-*-10-*-*-*-*-*-*-*\""
 #define DEF_INFO_TEXT_FONT "\"-*-helvetica-medium-r-normal-*-12-*-*-*-*-*-*-*\""
-#endif /* !I18N_MB */
+
+#define DEF_WORKSPACE_NAME_FONT "\"-*-lucida-bold-r-*-*-24-*-*-*-*-*-*-*\""
+#endif /* !I18N */
 
 #define HELVETICA10_FONT "-*-helvetica-medium-r-normal-*-10-*-*-*-*-*-*-*"
 
 #define DEF_FRAME_THICKNESS 1	       /* linewidth of the move/resize frame */
 
+#define DEF_WINDOW_TITLE_EXTEND_SPACE	"0"
+#define DEF_MENU_TITLE_EXTEND_SPACE	"0"
+#define DEF_MENU_TEXT_EXTEND_SPACE	"0"
+#define TITLEBAR_EXTEND_SPACE 4
+
 #define DEF_XPM_CLOSENESS	40000 
 
 /* default position of application menus */
 #define DEF_APPMENU_X		10
 #define DEF_APPMENU_Y		10
 
+/* calculate window edge resistance from edge resistance */
+#define WIN_RESISTANCE(x)	(((x)*20)/30)
 
 /* Window level where icons reside */
 #define NORMAL_ICON_LEVEL WMNormalLevel
@@ -284,34 +348,37 @@
 #define BALLOON_DELAY	1000
 
 /* delay for menu item selection hysteresis */
-#define MENU_SELECT_DELAY 300
+#define MENU_SELECT_DELAY 200
 
-/* animation speed constants */
+/* delay for jumpback of scrolled menus */
+#define MENU_JUMP_BACK_DELAY 	400
+
+/* *** animation speed constants *** */
 
 /* icon slide */
-#define ICON_SLIDE_SLOWDOWN_UF	20
+#define ICON_SLIDE_SLOWDOWN_UF	1
 #define ICON_SLIDE_DELAY_UF	0
-#define ICON_SLIDE_STEPS_UF	15
+#define ICON_SLIDE_STEPS_UF	50
 
-#define ICON_SLIDE_SLOWDOWN_F	30
+#define ICON_SLIDE_SLOWDOWN_F	3
 #define ICON_SLIDE_DELAY_F	0
-#define ICON_SLIDE_STEPS_F	10
+#define ICON_SLIDE_STEPS_F	50
 
-#define ICON_SLIDE_SLOWDOWN_M	40
+#define ICON_SLIDE_SLOWDOWN_M	5
 #define ICON_SLIDE_DELAY_M	0
-#define ICON_SLIDE_STEPS_M	5
+#define ICON_SLIDE_STEPS_M	30
 
-#define ICON_SLIDE_SLOWDOWN_S	50
+#define ICON_SLIDE_SLOWDOWN_S	10
 #define ICON_SLIDE_DELAY_S	0
-#define ICON_SLIDE_STEPS_S	3
+#define ICON_SLIDE_STEPS_S	20
 
-#define ICON_SLIDE_SLOWDOWN_U	50
-#define ICON_SLIDE_DELAY_U	3
-#define ICON_SLIDE_STEPS_U	3
+#define ICON_SLIDE_SLOWDOWN_US	20
+#define ICON_SLIDE_DELAY_US	1
+#define ICON_SLIDE_STEPS_US	10
 
 /* menu scrolling */
 #define MENU_SCROLL_STEPS_UF	14
-#define MENU_SCROLL_DELAY_UF	0
+#define MENU_SCROLL_DELAY_UF	1
 
 #define MENU_SCROLL_STEPS_F	10
 #define MENU_SCROLL_DELAY_F	5
@@ -322,8 +389,9 @@
 #define MENU_SCROLL_STEPS_S	4
 #define MENU_SCROLL_DELAY_S	6
 
-#define MENU_SCROLL_STEPS_U	1
-#define MENU_SCROLL_DELAY_U	8
+#define MENU_SCROLL_STEPS_US	1
+#define MENU_SCROLL_DELAY_US	8
+
 
 /* shade animation */
 #define SHADE_STEPS_UF		5
@@ -338,8 +406,20 @@
 #define SHADE_STEPS_S		30
 #define SHADE_DELAY_S		0
 
-#define SHADE_STEPS_U		20
-#define SHADE_DELAY_U		10
+#define SHADE_STEPS_US		40
+#define SHADE_DELAY_US		10
+
+
+/* workspace name on switch display */
+#define WORKSPACE_NAME_FADE_DELAY 30
+
+#ifdef VIRTUAL_DESKTOP 
+/* workspace virtual edge speed */
+#define VIRTUALEDGE_SCROLL_VSTEP 30
+#define VIRTUALEDGE_SCROLL_HSTEP 30
+#endif
+
+#define WORKSPACE_NAME_DELAY	400
 
 /* window birth animation steps (DO NOT MAKE IT RUN-TIME) */
 #define WINDOW_BIRTH_STEPS	20
@@ -360,11 +440,14 @@
 #define PLACETEST_VSTEP	8
 
 
-#define DOCK_EXTRA_SPACE	1
+#define DOCK_EXTRA_SPACE	0
 
 /* Vicinity in which an icon can be attached to the clip */
 #define CLIP_ATTACH_VICINITY	1
 
+#define CLIP_BUTTON_SIZE  23
+
+
 /* The amount of space (in multiples of the icon size)
  * a docked icon must be dragged out to detach it */
 #define DOCK_DETTACH_THRESHOLD	3
@@ -372,6 +455,17 @@
 /* Delay (in ms) after which the clip will autocollapse when leaved */
 #define AUTO_COLLAPSE_DELAY     1000
 
+/* Delay (in ms) after which the clip will autoexpand when entered.
+ * Set this to zero if you want instant expanding. */
+#define AUTO_EXPAND_DELAY       600
+
+/* Delay (in ms) after which the clip will be lowered when leaved */
+#define AUTO_LOWER_DELAY        1000
+
+/* Delay (in ms) after which the clip will be raised when entered.
+ * Set this to zero if you want instant raise. */
+#define AUTO_RAISE_DELAY        600
+
 
 /* Max. number of icons the clip can have */
 #define CLIP_MAX_ICONS		32
@@ -382,7 +476,10 @@
 
 #define CURSOR_BLINK_RATE	300
 
-#define MOVE_THRESHOLD	3 /* how many pixels to move before dragging windows */
+#define MOVE_THRESHOLD	5 /* how many pixels to move before dragging windows
+			   * and other objects */
+
+#define KEY_CONTROL_WINDOW_WEIGHT 1
 
 #define HRESIZE_THRESHOLD	3
 
@@ -400,11 +497,12 @@
 /* don't put titles in miniwindows */
 #undef NO_MINIWINDOW_TITLES
 
-/*
- * disable/enable workspace indicator in the dock
- */
 
-#undef WS_INDICATOR 
+#define FRAME_BORDER_COLOR "black"
+
+
+/* for boxes with high mouse sampling rates (SGI) */
+#define DELAY_BETWEEN_MOUSE_SAMPLING  10
 
 
 /*
@@ -415,19 +513,18 @@
  */
 
 
+/* number of window shortcuts */
+#define MAX_WINDOW_SHORTCUTS 10
+
 
 #define WM_PI 3.14159265358979323846
 
-#define FRAME_BORDER_COLOR "black"
-
 #define FRAME_BORDER_WIDTH 1	       /* width of window border for frames */
 
 #define RESIZEBAR_HEIGHT 8	       /* height of the resizebar */
 #define RESIZEBAR_MIN_WIDTH 20	       /* min. width of handles-corner_width */
 #define RESIZEBAR_CORNER_WIDTH 28      /* width of the corner of resizebars */
 
-#define TITLEBAR_EXTRA_HEIGHT 8
-
 #define MENU_INDICATOR_SPACE	12
 
 /* minimum size for windows */
@@ -442,7 +539,12 @@
 #define MAX_ICON_WIDTH	60	       /* size of the icon pixmap */
 #define MAX_ICON_HEIGHT 48
 
-#define MAX_WORKSPACES  100
+/* KDE may have problems with more than 32 workspaces */
+#ifdef KWM_HINTS
+# define MAX_WORKSPACES 32
+#else
+# define MAX_WORKSPACES  100
+#endif
 
 #define MAX_MENU_TEXT_LENGTH 512
 
@@ -471,12 +573,23 @@
 #undef SHAPED_BALLOON
 #endif
 
-
 #ifdef XKB_MODELOCK
 #define KEEP_XKB_LOCK_STATUS
+/* This is a hidden feature.
+ * Choose just one of LANGUAGE_* hints.
+ * Icon can be changed in def_pixmaps.h.
+ * More icons are welcome.
+ */
+#define XKB_BUTTON_HINT
+#undef LANGUAGE_TH
+#undef LANGUAGE_SK
 #endif
 
-#if HAVE_LIBINTL_H && I18N
+#ifndef DEMATERIALIZE_ICON
+# define NORMAL_ICON_KABOOM
+#endif
+
+#if defined(HAVE_LIBINTL_H) && defined(I18N)
 # include <libintl.h>
 # define _(text) gettext(text)
 #else
