@@ -140,7 +140,8 @@ RGetImageFromXPMData(RContext *context, char **data)
 	    color_table[1][i] = green;
 	    color_table[2][i] = blue;
 	    color_table[3][i] = 255;
-	} else if (strncmp(&(data[line][j]), "None", 4)==0) {
+	} else if (strncmp(&(data[line][j]), "None", 4)==0
+		   || strncmp(&(data[line][j]), "none", 4)==0) {
 	    color_table[3][i] = 0;
 	    transp = 1;
 	} else {
@@ -155,10 +156,13 @@ RGetImageFromXPMData(RContext *context, char **data)
 	return NULL;
     }
 
-    r = image->data[0];
-    g = image->data[1];
-    b = image->data[2];
-    a = image->data[3];
+    r = image->data;
+    g = image->data+1;
+    b = image->data+2;
+    if (image->format == RRGBAFormat)
+	a = image->data+3;
+    else
+	a = NULL;
 
     for (i=0; i<h; i++) {
 	if (csize==1) {
@@ -172,11 +176,15 @@ RGetImageFromXPMData(RContext *context, char **data)
 		if (k==ccount)
 		    k = 0;
 		
-		*(r++) = color_table[0][k];
-		*(g++) = color_table[1][k];
-		*(b++) = color_table[2][k];
-		if (a)
-		    *(a++) = color_table[3][k];
+		*r = color_table[0][k];
+		*g = color_table[1][k];
+		*b = color_table[2][k];
+		if (a) {
+		    *a = color_table[3][k];
+		    r += 4; g += 4; b += 4; a += 4;
+		} else {
+		    r += 3; g += 3; b += 3;
+		}
 	    }
 	} else {
 	    for (j=0; j<w*2; j++) {
@@ -190,11 +198,15 @@ RGetImageFromXPMData(RContext *context, char **data)
 		if (k==ccount)
 		    k = 0;
 	
-		*(r++) = color_table[0][k];
-		*(g++) = color_table[1][k];
-		*(b++) = color_table[2][k];
-		if (a)
-		    *(a++) = color_table[3][k];
+		*r = color_table[0][k];
+		*g = color_table[1][k];
+		*b = color_table[2][k];
+		if (a) {
+		    *a = color_table[3][k];
+		    r += 4; g += 4; b += 4; a += 4;
+		} else {
+		    r += 3; g += 3; b += 3;
+		}
 	    }
 	}
 	line++;
@@ -211,7 +223,7 @@ RGetImageFromXPMData(RContext *context, char **data)
     alloca(0);
 #endif
     if (image)
-	RDestroyImage(image);
+	RReleaseImage(image);
     return NULL;
 }
 
@@ -315,7 +327,8 @@ RLoadXPM(RContext *context, char *file, int index)
 	    color_table[1][i] = green;
 	    color_table[2][i] = blue;
 	    color_table[3][i] = 255;
-	} else if (strncmp(&(line[j]), "None", 4)==0) {
+	} else if (strncmp(&(line[j]), "None", 4)==0
+		   || strncmp(&(line[j]), "none", 4)==0) {
 	    color_table[3][i] = 0;
 	    transp = 1;
 	} else {
@@ -330,10 +343,13 @@ RLoadXPM(RContext *context, char *file, int index)
 	return NULL;
     }
 
-    r = image->data[0];
-    g = image->data[1];
-    b = image->data[2];
-    a = image->data[3];
+    r = image->data;
+    g = image->data+1;
+    b = image->data+2;
+    if (image->format == RRGBAFormat)
+	a = image->data+3;
+    else
+	a = NULL;
 
     for (i=0; i<h; i++) {
 	if (!fgets(buffer, bsize, f))
@@ -353,11 +369,15 @@ RLoadXPM(RContext *context, char *file, int index)
 		if (k==ccount)
 		    k = 0;
 		
-		*(r++) = color_table[0][k];
-		*(g++) = color_table[1][k];
-		*(b++) = color_table[2][k];
-		if (a)
-		    *(a++) = color_table[3][k];
+		*r = color_table[0][k];
+		*g = color_table[1][k];
+		*b = color_table[2][k];
+                if (a) {
+                    *a = color_table[3][k];
+                    r += 4; g += 4; b += 4; a += 4;
+                } else {
+                    r += 3; g += 3; b += 3;
+                }
 	    }
 	} else {
 	    for (j=1; j<=w*2; j++) {
@@ -372,11 +392,15 @@ RLoadXPM(RContext *context, char *file, int index)
 		    k = 0;
 		}
 	
-		*(r++) = color_table[0][k];
-		*(g++) = color_table[1][k];
-		*(b++) = color_table[2][k];
-		if (a)
-		    *(a++) = color_table[3][k];
+		*r = color_table[0][k];
+		*g = color_table[1][k];
+		*b = color_table[2][k];
+                if (a) {
+                    *a = color_table[3][k];
+                    r += 4; g += 4; b += 4; a += 4;
+                } else {
+                    r += 3; g += 3; b += 3;
+                }
 	    }
 	}
     }
@@ -394,7 +418,7 @@ RLoadXPM(RContext *context, char *file, int index)
     alloca(0);
 #endif
     if (image)
-	RDestroyImage(image);
+	RReleaseImage(image);
     return NULL;
     
   bad_file:
@@ -404,7 +428,7 @@ RLoadXPM(RContext *context, char *file, int index)
     alloca(0);
 #endif
     if (image)
-	RDestroyImage(image);
+	RReleaseImage(image);
     return NULL;
 }
 
@@ -415,7 +439,7 @@ typedef struct XPMColor {
     unsigned char red;
     unsigned char green;
     unsigned char blue;
-    char text[12];
+    int index;
     struct XPMColor *next;
 } XPMColor;
 
@@ -482,23 +506,34 @@ addcolor(XPMColor **list, unsigned r, unsigned g, unsigned b, int *colors)
 }
 
 
-static void
-outputcolormap(FILE *file, XPMColor *colormap, int colorCount)
+static char*
+index2str(char *buffer, int index, int charsPerPixel)
 {
-    int j;
-    int i,index;
+    int i;
+
+    for (i=0; i<charsPerPixel; i++) {
+	buffer[i] = I2CHAR(index&63);
+	index >>= 6;
+    }
+    buffer[i] = 0;
+    
+    return buffer;
+}
+
+
+static void
+outputcolormap(FILE *file, XPMColor *colormap, int charsPerPixel)
+{
+    int index;
+    char buf[128];
 
     if (!colormap)
 	return;
 
     for (index=0; colormap!=NULL; colormap=colormap->next,index++) {
-	j = index;
-	for (i=0; i<colorCount/64+1; i++) {
-	    colormap->text[i] = I2CHAR(j&63);
-	    j >>= 5;
-	}
-	colormap->text[i] = 0;
-	fprintf(file, "\"%s c #%02x%02x%02x\",\n", colormap->text, colormap->red,
+	colormap->index = index;
+	fprintf(file, "\"%s c #%02x%02x%02x\",\n", 
+		index2str(buf, index, charsPerPixel), colormap->red,
 		colormap->green, colormap->blue);
     }
 }
@@ -525,12 +560,14 @@ RSaveXPM(RImage *image, char *filename)
     FILE *file;
     int x, y;
     int colorCount=0;
+    int charsPerPixel;
     XPMColor *colormap = NULL;
     XPMColor *tmpc;
     int i;
     int ok = 0;
     unsigned char *r, *g, *b, *a;
     char transp[16];
+    char buf[128];
 
     file = fopen(filename, "w+");
     if (!file) {
@@ -542,32 +579,43 @@ RSaveXPM(RImage *image, char *filename)
 
     fprintf(file, "static char *image[] = {\n");
 
-    r = image->data[0];
-    g = image->data[1];
-    b = image->data[2];
-    a = image->data[3];
+    r = image->data;
+    g = image->data+1;
+    b = image->data+2;
+    if (image->format == RRGBAFormat)
+	a = image->data+3;
+    else
+	a = NULL;
 
     /* first pass: make colormap for the image */
     if (a)
 	colorCount = 1;
     for (y = 0; y < image->height; y++) {
 	for (x = 0; x < image->width; x++) {
-	    if (!a || *a++>127)
+            if (!a || *a>127) {
 		if (!addcolor(&colormap, *r, *g, *b, &colorCount)) {
 		    goto uhoh;
-		}
-	    r++; g++; b++;
+                }
+            }
+            if (a) {
+                r += 4; g += 4; b += 4; a += 4;
+            } else {
+                r += 3; g += 3; b += 3;
+            }
 	}
     }
 
+    charsPerPixel = 1;
+    while ((1 << charsPerPixel*6) < colorCount)
+        charsPerPixel++;
+
     /* write header info */
     fprintf(file, "\"%i %i %i %i\",\n", image->width, image->height, 
-	    colorCount, colorCount/64+1);
-
+	    colorCount, charsPerPixel);
 
     /* write colormap data */
-    if (image->data[3]) {
-	for (i=0; i<colorCount/64+1; i++)
+    if (a) {
+	for (i=0; i<charsPerPixel; i++)
 	    transp[i] = ' ';
 	transp[i] = 0;
 
@@ -575,12 +623,16 @@ RSaveXPM(RImage *image, char *filename)
     }
     
     i = 0;
-    outputcolormap(file, colormap, colorCount);
+    outputcolormap(file, colormap, charsPerPixel);
 
-    r = image->data[0];
-    g = image->data[1];
-    b = image->data[2];
-    a = image->data[3];
+
+    r = image->data;
+    g = image->data+1;
+    b = image->data+2;
+    if (image->format == RRGBAFormat)
+	a = image->data+3;
+    else
+	a = NULL;
 
     /* write data */
     for (y = 0; y < image->height; y++) {
@@ -589,15 +641,19 @@ RSaveXPM(RImage *image, char *filename)
 
 	for (x = 0; x < image->width; x++) {
 
-	    if (!a || *a++>127) {
+	    if (!a || *a>127) {
 		tmpc = lookfor(colormap, (unsigned)*r<<16|(unsigned)*g<<8|(unsigned)*b);
 
-		fprintf(file, tmpc->text);
+		fprintf(file, index2str(buf, tmpc->index, charsPerPixel));
 	    } else {
 		fprintf(file, transp);
 	    }
 
-	    r++; g++; b++;
+            if (a) {
+                r += 4; g += 4; b += 4; a += 4;
+            } else {
+                r += 3; g += 3; b += 3;
+            }
 	}
 
 	if (y < image->height-1)
